@@ -32,9 +32,10 @@ extern void *calloc(), *malloc(), *realloc();
 #include "P_.h"
 #include "astro.h"
 #include "circum.h"
+#include "preferences.h"
 
 extern Widget	toplevel_w;
-extern char	*myclass;
+extern Colormap xe_cm;
 
 extern FILE *fopenh P_((char *name, char *how));
 extern Now *mm_get_now P_((void));
@@ -43,7 +44,7 @@ extern char *syserrstr P_((void));
 extern double atod P_((char *buf));
 extern int db_n P_((void));
 extern int existsh P_((char *filename));
-extern void db_scaninit P_((DBScan *sp));
+extern void db_scaninit P_((DBScan *sp, int mask, ObjF *op, int nop));
 extern void db_update P_((Obj *op));
 extern void fs_pangle P_((char out[], double a));
 extern void get_something P_((Widget w, char *resource, XtArgVal value));
@@ -59,6 +60,8 @@ extern void sv_point P_((Obj *op));
 extern void timestamp P_((Now *np, Widget w));
 extern void watch_cursor P_((int want));
 extern void xe_msg P_((char *msg, int app_modal));
+extern void wtip P_((Widget w, char *tip));
+
 
 #define	MINSEP	degrad(.1)	/* smallest sep we allow.
 				 * this is not a hard limit, just a sanity
@@ -86,25 +89,25 @@ typedef struct {
 static void c_create_form P_((void));
 static void c_help_cb P_((Widget w, XtPointer client, XtPointer call));
 static void c_helpon_cb P_((Widget w, XtPointer client, XtPointer call));
+static void c_unmap_cb P_((Widget w, XtPointer client, XtPointer call));
 static void c_close_cb P_((Widget w, XtPointer client, XtPointer call));
 static void c_flist_cb P_((Widget w, XtPointer client, XtPointer call));
-static void c_topo_cb P_((Widget w, XtPointer client, XtPointer call));
 static void c_actlist_cb P_((Widget w, XtPointer client, XtPointer call));
 static void c_skypoint_cb P_((Widget w, XtPointer client, XtPointer call));
 static void c_go_cb P_((Widget w, XtPointer client, XtPointer call));
 static int sky_point P_((void));
 static void do_search P_((void));
-static int get_options P_((double *sp, double *mp, int *wp, int *tp));
+static int get_options P_((double *sp, double *mp, int *wp));
 static void free_band P_((Band bp[], int nb));
-static int init_bands P_((double sep, double mag, int topo, Band **bpp));
+static int init_bands P_((double sep, double mag, Band **bpp));
 static int add_obj P_((Band *bp, Obj *op));
-static int find_close P_((Band bp[], int nb, double maxsep, int topo,
-    int wander, Pair **ppp));
+static int find_close P_((Band bp[], int nb, double maxsep, int wander,
+    Pair **ppp));
 static int add_pair P_((Obj *op1, Obj *op2, double sep, Pair **ppp,
     int *mp, int *np));
 static int add_pair P_((Obj *op1, Obj *op2, double sep, Pair **ppp,
     int *mp, int *np));
-static int dspl_pairs P_((Pair p[], int np, int topo, double sep));
+static int dspl_pairs P_((Pair p[], int np, double sep));
 
 static void c_create_flist_w P_((void));
 static void c_flistok_cb P_((Widget w, XtPointer client, XtPointer call));
@@ -121,8 +124,6 @@ static Widget timestmp_w;	/* label for time stamp */
 static Widget count_w;		/* label for count of objects in list */
 static Widget list_w;		/* the list of close objects */
 static Widget flist_w;		/* the file list dialog */
-static Widget topo_w;		/* toggle to choose topo/geo */
-static Widget geo_w;		/* toggle to choose topo/geo */
 static Widget wander_w;		/* toggle to omit pairs of fixed objs*/
 static Widget autoup_w;		/* toggle to automatically redo on updates */
 
@@ -163,7 +164,7 @@ Cursor c;
 {
 	Window win;
 
-	if (cform_w && (win = XtWindow(cform_w))) {
+	if (cform_w && (win = XtWindow(cform_w)) != 0) {
 	    Display *dsp = XtDisplay(cform_w);
 	    if (c)
 		XDefineCursor (dsp, win, c);
@@ -201,15 +202,18 @@ c_create_form ()
 	XtSetArg(args[n], XmNdefaultPosition, False);  n++;
 	XtSetArg (args[n], XmNverticalSpacing, 5); n++;
 	XtSetArg (args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
+	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
 	cform_w = XmCreateFormDialog(toplevel_w, "CloseObjs", args, n);
+	set_something (cform_w, XmNcolormap, (XtArgVal)xe_cm);
 	XtAddCallback (cform_w, XmNmapCallback, prompt_map_cb, NULL);
+	XtAddCallback (cform_w, XmNunmapCallback, c_unmap_cb, NULL);
 	XtAddCallback (cform_w, XmNhelpCallback, c_help_cb, NULL);
 
 	/* set some stuff in the parent DialogShell.
 	 * setting XmNdialogTitle in the Form didn't work..
 	 */
 	n = 0;
-	XtSetArg (args[n], XmNtitle, "xephem Close Objects"); n++;
+	XtSetArg (args[n], XmNtitle, "xephem Close Pairs"); n++;
 	XtSetValues (XtParent(cform_w), args, n);
 
 	/* create the menu bar across the top */
@@ -239,6 +243,7 @@ c_create_form ()
 	    w = XmCreatePushButton (pd_w, "Search", args, n);
 	    XtAddCallback (w, XmNactivateCallback, c_go_cb, 0);
 	    set_xmstring(w, XmNlabelString, "Go");
+	    wtip (w, "Scan one time for close pairs");
 	    XtManageChild (w);
 
 	    /* add a separator */
@@ -253,6 +258,7 @@ c_create_form ()
 	    XtSetArg (args[n], XmNvisibleWhenOff, True); n++;
 	    autoup_w = XmCreateToggleButton (pd_w, "AutoUp", args, n);
 	    set_xmstring(autoup_w, XmNlabelString, "Auto update");
+	    wtip (autoup_w, "When on, perform a new scan after each Main menu Update");
 	    XtManageChild (autoup_w);
 
 	    /* make the sky point button */
@@ -261,6 +267,7 @@ c_create_form ()
 	    w = XmCreatePushButton (pd_w, "SkyPt", args, n);
 	    XtAddCallback (w, XmNactivateCallback, c_skypoint_cb, 0);
 	    set_xmstring(w, XmNlabelString, "Sky point");
+	    wtip (w, "Center the Sky View on the pair selected in the list");
 	    XtManageChild (w);
 
 	    /* make the file list button */
@@ -269,6 +276,7 @@ c_create_form ()
 	    w = XmCreatePushButton (pd_w, "File", args, n);
 	    XtAddCallback (w, XmNactivateCallback, c_flist_cb, 0);
 	    set_xmstring(w, XmNlabelString, "List to file...");
+	    wtip (w, "Save the current list to a file");
 	    XtManageChild (w);
 
 	    /* add a separator */
@@ -282,6 +290,7 @@ c_create_form ()
 	    n = 0;
 	    w = XmCreatePushButton (pd_w, "Close", args, n);
 	    XtAddCallback (w, XmNactivateCallback, c_close_cb, 0);
+	    wtip (w, "Close this dialog");
 	    XtManageChild (w);
 
 	/* make the options pulldown */
@@ -291,7 +300,7 @@ c_create_form ()
 
 	    n = 0;
 	    XtSetArg (args[n], XmNsubMenuId, pd_w);  n++;
-	    XtSetArg (args[n], XmNmnemonic, 'D'); n++;
+	    XtSetArg (args[n], XmNmnemonic, 'O'); n++;
 	    cb_w = XmCreateCascadeButton (mb_w, "OptionsCB", args, n);
 	    set_xmstring (cb_w, XmNlabelString, "Options");
 	    XtManageChild (cb_w);
@@ -302,35 +311,9 @@ c_create_form ()
 	    XtSetArg (args[n], XmNvisibleWhenOff, True); n++;
 	    wander_w = XmCreateToggleButton (pd_w, "OmitFixedPairs", args, n);
 	    set_xmstring(wander_w, XmNlabelString, "Omit fixed pairs");
+	    wtip (wander_w, "When on, both objects in a pair can not be of type FIXED");
 	    XtManageChild (wander_w);
 
-	    /* simulate a radio box for the topo/geo pair */
-
-	    n = 0;
-	    w = XmCreateSeparator (pd_w, "Sep1", args, n);
-	    XtManageChild (w);
-
-	    n = 0;
-	    XtSetArg (args[n], XmNvisibleWhenOff, True); n++;
-	    XtSetArg (args[n], XmNindicatorType, XmONE_OF_MANY); n++;
-	    topo_w = XmCreateToggleButton (pd_w, "Topo", args, n);
-	    set_xmstring (topo_w, XmNlabelString, "Topocentric separation");
-	    XtAddCallback (topo_w, XmNvalueChangedCallback, c_topo_cb, NULL);
-	    XtManageChild (topo_w);
-
-	    n = 0;
-	    XtSetArg (args[n], XmNvisibleWhenOff, True); n++;
-	    XtSetArg (args[n], XmNindicatorType, XmONE_OF_MANY); n++;
-	    geo_w = XmCreateToggleButton (pd_w, "Geo", args, n);
-	    set_xmstring (geo_w, XmNlabelString, "Geocentric separation");
-	    XtAddCallback (geo_w, XmNvalueChangedCallback, c_topo_cb, NULL);
-	    XtManageChild (geo_w);
-	    if(XmToggleButtonGetState(geo_w) == XmToggleButtonGetState(topo_w)){
-		xe_msg("Topo and Geo buttons are the same -- using topo",0);
-		XmToggleButtonSetState (topo_w, True, False);
-		XmToggleButtonSetState (geo_w, False, False);
-	    }
-
 	/* make the help pulldown */
 
 	n = 0;
@@ -381,6 +364,7 @@ c_create_form ()
 	XtSetArg (args[n], XmNleftPosition, 53); n++;
 	XtSetArg (args[n], XmNcolumns, 10); n++;
 	sep_w = XmCreateTextField (cform_w, "Sep", args, n);
+	wtip (sep_w, "Enter desired minimum separation");
 	XtManageChild (sep_w);
 
 	/* make the limiting mag label and entry field */
@@ -406,6 +390,7 @@ c_create_form ()
 	XtSetArg (args[n], XmNleftPosition, 53); n++;
 	XtSetArg (args[n], XmNcolumns, 10); n++;
 	mag_w = XmCreateTextField (cform_w, "Mag", args, n);
+	wtip (mag_w, "Enter desired limiting magnitude");
 	XtManageChild (mag_w);
 
 	/* make a separator */
@@ -438,7 +423,8 @@ c_create_form ()
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
 	timestmp_w = XmCreateLabel (cform_w, "TSL", args, n);
-	set_xmstring (timestmp_w, XmNlabelString, "    ");
+	set_xmstring (timestmp_w, XmNlabelString, " -- ");
+	wtip (timestmp_w, "Date and Time for which data are computed");
 	XtManageChild (timestmp_w);
 
 	/* make the list between the count and the time */
@@ -454,6 +440,7 @@ c_create_form ()
 	XtSetArg (args[n], XmNlistSizePolicy, XmCONSTANT); n++;
 	list_w = XmCreateScrolledList (cform_w, "List", args, n);
 	XtAddCallback (list_w, XmNdefaultActionCallback, c_actlist_cb, NULL);
+	wtip (list_w, "The list of close pairs");
 	XtManageChild (list_w);
 }
 
@@ -503,6 +490,18 @@ XtPointer call;
 	    XtManageChild (flist_w);
 }
 
+/* called when the main dialog is unmapped */
+/* ARGSUSED */
+static void
+c_unmap_cb (w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
+{
+	if (flist_w && XtIsManaged(flist_w))
+	    XtUnmanageChild (flist_w);
+}
+
 /* called when the Close pushbutton is activated */
 /* ARGSUSED */
 static void
@@ -511,23 +510,10 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
+	/* let the unmap do the rest of the work */
 	XtUnmanageChild (cform_w);
 }
 
-/* called when either the topo or geo toggle button changes.
- * we simulate a radio box by flipping the other too.
- */
-/* ARGSUSED */
-static void
-c_topo_cb (w, client, call)
-Widget w;
-XtPointer client;
-XtPointer call;
-{
-	XmToggleButtonSetState (w == topo_w ? geo_w : topo_w,
-				!XmToggleButtonGetState(w), False);
-}
-
 /* called when a list item is double-clicked */
 /* ARGSUSED */
 static void
@@ -594,8 +580,8 @@ sky_point()
 		break;
 
 	/* scan the db for the object */
-	for (db_scaninit (&dbs); (op = db_scan(&dbs)) != NULL; )
-	    if (op->type != UNDEFOBJ && strcmp (op->o_name, objname) == 0)
+	for (db_scaninit (&dbs, ALLM, NULL, 0); (op = db_scan(&dbs)) != NULL; )
+	    if (strcmp (op->o_name, objname) == 0)
 		break;
 
 	/* point sky if found */
@@ -611,25 +597,6 @@ sky_point()
 	return (0);
 }
 
-/* given two pointers to Obj *, return how they sort by alt in qsort-style.
- */
-static int
-azcmp_f (v1, v2)
-qsort_arg v1;
-qsort_arg v2;
-{
-	Obj *op1 = *((Obj **)v1);
-	Obj *op2 = *((Obj **)v2);
-	double az1 = op1->s_az;
-	double az2 = op2->s_az;
-
-	if (az1 < az2)
-	    return (-1);
-	if (az1 > az2)
-	    return (1);
-	return (0);
-}
-
 /* given two pointers to Obj *, return how they sort by dec in qsort-style.
  */
 static int
@@ -655,14 +622,13 @@ do_search()
 	double sep;	/* desired max separation, rads */
 	double mag;	/* limiting magnitude */
 	int wander;	/* 1 if want to limit list to at least one wanderer */
-	int topo;	/* work in topocentric alt/az, else geocentric ra/dec */
 	Band *bp = NULL;/* malloced array of nb bands */
 	int nb;		/* number of dec (or alt) bands */
 	Pair *pp = NULL;/* malloced list of close pairs */
 	int np;		/* number of pairs */
 
 	/* retrieve user option settings */
-	if (get_options (&sep, &mag, &wander, &topo) < 0)
+	if (get_options (&sep, &mag, &wander) < 0)
 	    return;
 
 	/* ok, let's get to the real work */
@@ -671,7 +637,7 @@ do_search()
 	/* compute band size, deal and sort objects into their own and adjacent
 	 * bands.
 	 */
-	nb = init_bands (sep, mag, topo, &bp);
+	nb = init_bands (sep, mag, &bp);
 	if (nb < 0) {
 	    watch_cursor (0);
 	    return;
@@ -680,14 +646,14 @@ do_search()
 	/* look through bands for all pairs closer than sep.
 	 * form sorted lists of them in malloced pp array.
 	 */
-	np = find_close (bp, nb, sep, topo, wander, &pp);
+	np = find_close (bp, nb, sep, wander, &pp);
 
 	/* finished with the bands now */
 	free_band (bp, nb);
 
 	/* report what we found */
 	if (np > 0)
-	    (void) dspl_pairs (pp, np, topo, sep);
+	    (void) dspl_pairs (pp, np, sep);
 	else
 	    xe_msg ("No such pairs found.", 1);
 
@@ -701,11 +667,10 @@ do_search()
  * return 0 if ok, else -1
  */
 static int
-get_options (sp, mp, wp, tp)
+get_options (sp, mp, wp)
 double *sp;	/* max separation, rads */
 double *mp;	/* limiting magnitude */
 int *wp;	/* wanderer flag */
-int *tp;	/* topocentric flag */
 {
 	char *str;
 
@@ -726,7 +691,6 @@ int *tp;	/* topocentric flag */
 	XtFree (str);
 
 	*wp = XmToggleButtonGetState (wander_w);
-	*tp = XmToggleButtonGetState (topo_w);
 
 	return (0);
 }
@@ -759,14 +723,13 @@ int nb;
  * N.B. caller must free_band(*bpp) if we return >= 0.
  */
 static int
-init_bands (sep, mag, topo, bpp)
+init_bands (sep, mag, bpp)
 double sep;
 double mag;
-int topo;
 Band **bpp;
 {
 #define	BAND(x)		((int)floor(((x)+PI/2)/bandsiz))
-	int (*cmp_f)();
+	int topo = pref_get (PREF_EQUATORIAL) == PREF_TOPO;
 	double bandsiz;
 	Band *bp;
 	DBScan dbs;
@@ -801,22 +764,20 @@ Band **bpp;
 	 */
 	nobj = db_n();
 	n = 0;
-	for (db_scaninit(&dbs); (op = db_scan(&dbs)) != NULL; ) {
+	for (db_scaninit(&dbs, ALLM, NULL, 0); (op = db_scan(&dbs)) != NULL; ) {
 	    int b;
 
 	    pm_set (33 * n++ / nobj);	/* about 0-33% of time here */
 
-	    /* qualify object */
-	    if (op->type == UNDEFOBJ)
-		continue;
+	    /* update object */
 	    db_update (op);
-	    if (op->s_mag/MAGSCALE > mag)
+	    if (get_mag(op) > mag)
 		continue;
 	    if (topo && op->s_alt < 0.0)
 		continue;
 
 	    /* find which band op fits into */
-	    b = topo ? BAND(op->s_alt) : BAND(op->s_dec);
+	    b = BAND(op->s_dec);
 	    if (b < 0 || b > nb) {
 		(void) sprintf (msg, "%s is out of band: %d.", op->o_name, b);
 		xe_msg (msg, 0);
@@ -838,10 +799,9 @@ Band **bpp;
 
 	/* sort each band by ra (az)
 	 */
-	cmp_f = topo ? azcmp_f : racmp_f;
 	for (i = 0; i < nb; i++)
 	    if (bp[i].nuse > 0)
-		qsort ((void *)bp[i].opp, bp[i].nuse, sizeof (Obj *), cmp_f);
+		qsort ((void *)bp[i].opp, bp[i].nuse, sizeof (Obj *), racmp_f);
 
 	/* report answers and return */
 	*bpp = bp;
@@ -897,11 +857,10 @@ Obj *op;
  * N.B. caller must free *ppp if it is != NULL no matter what we return.
  */
 static int
-find_close (bp, nb, maxsep, topo, wander, ppp)
+find_close (bp, nb, maxsep, wander, ppp)
 Band bp[];	/* list of bands to scan */
 int nb;		/* number of bands */
 double maxsep;	/* max sep we are looking for, rads */
-int topo;	/* 1 if want topocentric alt/az or geocentric ra/dec */
 int wander;	/* 1 if limit to pairs with at least one wanderer (ss object) */
 Pair **ppp;	/* return list of pairs we malloc */
 {
@@ -927,8 +886,8 @@ Pair **ppp;	/* return list of pairs we malloc */
 		int j;
 
 		/* set up for finding sep -- see solve_sphere() -- i is c */
-		double cc = sin(topo ? opi->s_alt : opi->s_dec);
-		double sc = cos(topo ? opi->s_alt : opi->s_dec);
+		double cc = sin(opi->s_dec);
+		double sc = cos(opi->s_dec);
 
 		/* scan forward for close objects in the same band.
 		 * wrap, but not so far as to reach self again.
@@ -942,13 +901,13 @@ Pair **ppp;	/* return list of pairs we malloc */
 		    double sep, csep;
 
 		    /* omit if both are fixed and want at least one wanderer */
-		    if (wander && opi->type == FIXED && opj->type == FIXED)
+		    if (wander && is_type(opi,FIXEDM) && is_type(opj,FIXEDM))
 			continue;
 
 		    /* find the exact separation */
-		    cb = sin(topo ? opj->s_alt : opj->s_dec); /* j is b */
-		    sb = cos(topo ? opj->s_alt : opj->s_dec);
-		    A = topo ? opi->s_az - opj->s_az : opi->s_ra - opj->s_ra;
+		    cb = sin(opj->s_dec); /* j is b */
+		    sb = cos(opj->s_dec);
+		    A = opi->s_ra - opj->s_ra;
 		    cA = cos(A);
 		    csep = cb*cc + sb*sc*cA;
 		    if (csep >  1.0) csep =  1.0;	/* just paranoid */
@@ -1077,10 +1036,9 @@ qsort_arg v2;
  * return 0 if ok else -1.
  */
 static int
-dspl_pairs (pp, np, topo, sep)
+dspl_pairs (pp, np, sep)
 Pair pp[];
 int np;
-int topo;
 double sep;
 {
 	XmString *xmstrtbl;
@@ -1119,9 +1077,9 @@ double sep;
 
 	    fs_pangle (sepstr, (double)ppi->sep);
 	    (void) sprintf (buf, "%6.2f %-*.*s   %6.2f %-*.*s   %s",
-					    ppi->op1->s_mag/MAGSCALE,
+					    get_mag(ppi->op1),
 					    MAXNM-1, MAXNM-1, ppi->op1->o_name,
-					    ppi->op2->s_mag/MAGSCALE,
+					    get_mag(ppi->op2),
 					    MAXNM-1, MAXNM-1, ppi->op2->o_name,
 					    sepstr);
 	    xmstrtbl[ns++] = XmStringCreateSimple (buf);
@@ -1141,7 +1099,8 @@ double sep;
 	/* update the messages */
 	fs_pangle (sepstr, sep);
 	(void) sprintf (buf, "%d %s Pair%s <=%s", ns,
-		topo ? "Topocentric" : "Geocentric", ns==1 ? "" : "s", sepstr);
+	    pref_get(PREF_EQUATORIAL)==PREF_TOPO ? "Topocentric" : "Geocentric",
+	    					ns==1 ? "" : "s", sepstr);
 	set_xmstring (count_w, XmNlabelString, buf);
 	timestamp (mm_get_now(), timestmp_w);
 
@@ -1159,7 +1118,9 @@ c_create_flist_w()
 
 	n = 0;
 	XtSetArg(args[n], XmNdefaultPosition, False);  n++;
+	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
 	flist_w = XmCreatePromptDialog (toplevel_w, "CloseList", args,n);
+	set_something (flist_w, XmNcolormap, (XtArgVal)xe_cm);
 	set_xmstring (flist_w, XmNdialogTitle, "xephem Close list");
 	set_xmstring (flist_w, XmNselectionLabelString, "File name:");
 	XtUnmanageChild (XmSelectionBoxGetChild(flist_w, XmDIALOG_HELP_BUTTON));
@@ -1183,12 +1144,14 @@ XtPointer call;
 
 	if (strlen(name) == 0) {
 	    xe_msg ("Please enter a file name.", 1);
+	    XtFree (name);
 	    return;
 	}
 
 	get_something (list_w, XmNitemCount, (XtArgVal)&icount);
 	if (icount == 0) {
 	    xe_msg ("No items in list -- no file action.", 1);
+	    XtFree (name);
 	    return;
 	}
 
