@@ -5,10 +5,13 @@
 #include <ctype.h>
 #include <math.h>
 
+
 #if defined(__STDC__)
 #include <stdlib.h>
 #include <string.h>
+typedef const void * qsort_arg;
 #else
+typedef void * qsort_arg;
 extern void *malloc(), *realloc();
 #endif
 
@@ -19,6 +22,11 @@ extern int read();
 extern int close();
 #endif
 
+#ifndef SEEK_SET
+#define	SEEK_SET 0
+#define	SEEK_END 2
+#endif
+
 #include <X11/Intrinsic.h>
 
 #include "P_.h"
@@ -27,26 +35,26 @@ extern int close();
 #include "preferences.h"
 
 extern Widget toplevel_w;
-extern char *myclass;
 extern XtAppContext xe_app;
 #define XtD XtDisplay(toplevel_w)
 
 
 extern FILE *fopenh P_((char *name, char *how));
 extern Now *mm_get_now P_((void));
+extern char *getXRes P_((char *name));
 extern char *syserrstr P_((void));
 extern double atod P_((char *buf));
 extern int db_n P_((void));
 extern int db_n_cp P_((void));
-extern int db_read P_((FILE *fp));
+extern int db_read P_((FILE *fp, int nodups));
 extern int db_set_field P_((char bp[], int id, PrefDateFormat pref, Obj *op));
+extern double delra P_((double dra));
 extern int obj_cir P_((Now *np, Obj *op));
-extern int openh P_((char *name, int flags));
+extern int openh P_((char *name, int flags, ...));
 extern void all_newdb P_((int appended));
 extern void db_connect_fifo P_((void));
 extern void db_del_all P_((void));
 extern void db_del_cp P_((void));
-extern void db_scaninit P_((DBScan *sp));
 extern void db_set_cp P_((void));
 extern void db_clr_cp P_((void));
 extern void db_update P_((Obj *op));
@@ -64,7 +72,6 @@ extern void zero_mem P_((void *loc, unsigned len));
 static int db_crack_line P_((char s[]));
 static Obj *db_new P_((int t));
 static void db_init P_((void));
-static void db_nodups P_((void));
 static int nxt_db P_((char buf[], int blen, FILE *fp));
 static int line_candidate P_((char *buf));
 static void crack_year P_((char *bp, PrefDateFormat pref, double *p));
@@ -74,6 +81,7 @@ static void dbfifo_cb P_((XtPointer client, int *fdp, XtInputId *idp));
 #define	MAXDBLINE	256	/* longest allowed db line */
 #define	FLDSEP		','	/* major field separator */
 #define	SUBFLD		'|'	/* subfield separator */
+#define	MAXFLDS	20		/* must be more than on any expected line */
 
 #define	DBFIFO_MSG	'!'	/* introduces a message line from the DBFIFO */
 
@@ -85,7 +93,7 @@ static void dbfifo_cb P_((XtPointer client, int *fdp, XtInputId *idp));
  * circumstances for a given db object.
  * N.B. For this to work, call db_update() before using a Obj *.
  */
-static Objage_t db_age;
+static ObjAge_t db_age;
 
 /* the "database".
  * one such struct per object type. the space for objects is malloced in
@@ -115,12 +123,9 @@ static DBMem db[NOBJTYPES];	/* this is the head for each object */
  */
 static int db_fifofd = -1;
 static XtInputId db_fifoid;
+static void db_free P_((DBMem *dmp));
 
-
-/* set whenever we want to take care to eliminate any items in the db with
- *   duplicate names.
- */
-static int db_wantnodups;
+static char nomem[] = "Insufficient memory to add to database\n";
 
 /* return number of objects in the database.
  * this includes the NOBJ basic objects.
@@ -175,7 +180,7 @@ int id;
 	}
 
 	op = OBJP(PLANET,id);
-	if (op->type != UNDEFOBJ)
+	if (op->o_type != UNDEFOBJ)
 	    db_update(op);
 	return (op);
 }
@@ -199,20 +204,18 @@ db_loadinitial()
 	int i;
 
 	/* get the initial directory name */
-	dir = XGetDefault (XtD, myclass, dbd);
+	dir = getXRes (dbd);
 	if (!dir) {
 	    dir = ".";
-	    (void) sprintf (msg, "Can not get %.100s.%.100s -- using %s",
-							    myclass, dbd, dir);
+	    (void) sprintf (msg, "Can not find %s -- using %s", dbd, dir);
 	    xe_msg (msg, 0);
 	}
 
 	/* get the initial list of files */
-	fns = XGetDefault (XtD, myclass, dbi);
+	fns = getXRes (dbi);
 	if (!fns) {
 	    fns = "basic.edb";
-	    (void) sprintf (msg ,"Can not get %.100s.%.100s -- using %s",
-							myclass, dbi, fns);
+	    (void) sprintf (msg ,"Can not find %s -- using %s", dbi, fns);
 	    xe_msg (msg, 0);
 	}
 
@@ -247,7 +250,10 @@ db_loadinitial()
 		xe_msg (msg, 1);
 		continue;
 	    }
-	    if (db_read (fp) < 0) {
+	    /*
+	    printf ("Reading %s\n", fnf[i]);
+	    */
+	    if (db_read (fp, 1) < 0) {
 		(void) sprintf (msg, "Error reading `%.200s'", fn);
 		xe_msg (msg, 0);
 	    }
@@ -260,10 +266,10 @@ db_loadinitial()
 /* set our official copy of the given user-defined object to *op */
 void
 db_setuserobj(id, op)
-int id;	/* OBJX or OBJY */
+int id;	/* OBJXYZ */
 Obj *op;
 {
-	if (id == OBJX || id == OBJY) {
+	if (id == OBJX || id == OBJY || id == OBJZ) {
 	    Obj *bop = OBJP(PLANET,id);
 	    *bop = *op;
 	} else {
@@ -280,26 +286,33 @@ db_invalidate()
 	if (!DBINITED)
 	    db_init();
 
-	db_age++;
+	db_age++;	/* ok if wraps */
 }
 
-/* initialize the given DBScan for a database scan. the idea is to call this
- *   once, then repeatedly call db_scan() to get all objects in the db.
+/* initialize the given DBScan for a database scan. mask is a collection of
+ *   *M masks for the desired types of objects. op/nop describe a list of
+ *   ObjF which will also be scanned in addition to what is in the database.
+ * the idea is to call this once, then repeatedly call db_scan() to get all
+ *   objects in the db of those types, then those is op (if any).
  * return NULL when there are no more.
  * N.B. nothing should be assumed as to the order these are returned.
- * N.B. the s_ fields are *not* updated -- call db_update() when you need that.
  */
 void
-db_scaninit (sp)
+db_scaninit (sp, mask, op, nop)
 DBScan *sp;
+int mask;
+ObjF *op;
+int nop;
 {
 	sp->t = 0;
 	sp->n = 0;
+	sp->m = mask;
+	sp->op = op;
+	sp->nop = nop;
 }
 
 /* fetch the next object.
- * N.B. this will return the user defined objects which may still
- *   have op->type == UNDEFOBJ.
+ * N.B. the s_ fields are *not* updated -- call db_update() when you need that.
  */
 Obj *
 db_scan (sp)
@@ -310,18 +323,34 @@ DBScan *sp;
 	if (!DBINITED)
 	    db_init();
 
-	while (sp->n >= db[sp->t].nobj) {
-	    if (++sp->t == NOBJTYPES)
-		return (0);
-	    sp->n = 0;
+    	/* check op/nop only after everything else is complete */
+    doop:
+	if (sp->t == NOBJTYPES) {
+	    if (sp->op && sp->n < sp->nop)
+		return ((Obj*)&sp->op[sp->n++]);
+	    else
+		return (NULL);
 	}
 
-	op = OBJP(sp->t, sp->n);
-	sp->n++;
+	/* outter loop is just to skip over undefined user objects, if any. */
+	do {
+	    /* advance to the next type if we have scanned all the ones of
+	     * the current type or if the type is not desired.
+	     */
+	    while (sp->n >= db[sp->t].nobj || !((1<<sp->t) & sp->m)) {
+		sp->n = 0;
+		if (++sp->t == NOBJTYPES) {
+		    /* no more in the real db -- try op */
+		    goto doop;
+		}
+	    }
+
+	    op = OBJP(sp->t, sp->n);
+	    sp->n++;
+	} while (op->o_type == UNDEFOBJ);
 
 	return (op);
 }
-	
 
 /* see to it that all the s_* fields in the given db object are up to date.
  * always recompute the user defined objects because we don't know when
@@ -338,17 +367,18 @@ Obj *op;
 	if (!DBINITED)
 	    db_init();
 
-	if (op->type == UNDEFOBJ) {
+	if (op->o_type == UNDEFOBJ) {
 	    printf ("%s: called with UNDEFOBJ pointer\n", me);
 	    exit (1);
 	} 
-	if ((int)op->type >= NOBJTYPES) {
+	if ((int)op->o_type >= NOBJTYPES) {
 	    printf ("%s: called with bad pointer\n", me);
 	    exit (1);
 	} 
 
 	if (op->o_age != db_age ||
-			op == OBJP(PLANET,OBJX) || op == OBJP(PLANET,OBJY)) {
+		    op == OBJP(PLANET,OBJX) || op == OBJP(PLANET,OBJY ||
+					       op == OBJP(PLANET,OBJZ))) {
 	    if (obj_cir (mm_get_now(), op) < 0) {
 		printf ("%s: bad object\n", me);
 		exit(1);
@@ -371,16 +401,7 @@ db_del_all()
 	    /* N.B. PLANET entries are fixed -- not malloced */
 	    if (dmp == &db[PLANET])
 		continue;
-	    if (dmp->dblist) {
-		int i;
-		for (i = 0; i < dmp->nmem/DBCHUNK; i++)
-		    free (dmp->dblist[i]);
-		free ((char *)dmp->dblist);
-		dmp->dblist = NULL;
-	    }
-	    dmp->nobj = 0;
-	    dmp->nmem = 0;
-	    dmp->ncp = 0;
+	    db_free (dmp);
 	}
 }
 
@@ -413,8 +434,8 @@ db_set_cp()
 	    dmp->ncp = dmp->nobj;
 }
 
-/* set the checkpoints to 0 then, if active, also eliminate dups.
- * N.B. but not the PLANT one.
+/* set the checkpoints back to 0.
+ * N.B. never fiddle with PLANET's count.
  */
 void
 db_clr_cp()
@@ -429,38 +450,73 @@ db_clr_cp()
 	    if (dmp != &db[PLANET])
 		dmp->ncp = 0;
 	}
-
-	if (db_wantnodups)
-	    db_nodups();
 }
 
-/* called to indicate whether we want to eliminate dups.
- * if turning it on, we go ahead and eliminate them right now and set a
- *   flag so whenever new items are read dups are discarded then too.
- * if turning it off, we just reset the local flag.
- */
-void
-db_setnodups (whether)
-int whether;
+/* compare 2 pointers to strings in qsort fashion */
+static int
+name_cmpf (s1, s2)
+qsort_arg s1, s2;
 {
-	if (whether) {
-	    db_nodups();
-	    all_newdb(0);
+	return (strcmp (*(char **)s1, *(char **)s2));
+}
+
+/* see whether name is in names[nnames] (which is sorted in ascending order).
+ * actually, only use up to smaller of the first ',' or MAXNM-1 in name.
+ * if so, return 0, else -1
+ */
+static int
+chk_name (name, names, nnames)
+char *name;
+char **names;
+int nnames;
+{
+	int t, b, m;
+	int nl;
+	int s;
+
+	/* check for no names at all */
+	if (nnames <= 0)
+	    return (-1);
+
+	/* find portion of name up to first comma, up to MAXNM-1 */
+	for (nl = 0; name[nl] && name[nl] != ','; nl++)
+	    continue;
+	if (name[nl] == '\0')
+	    return (-1);
+	if (nl > MAXNM-1)
+	    nl = MAXNM-1;
+
+	/* binary search */
+	t = nnames - 1;
+	b = 0;
+	while (b <= t) {
+	    m = (t+b)/2;
+	    s = strncmp (name, names[m], nl);
+	    if (s == 0)
+		return (0);
+	    if (s < 0)
+		t = m-1;
+	    else
+		b = m+1;
 	}
 
-	db_wantnodups = whether;
+	return (-1);
 }
 
 /* read the given database file into memory.
  * add to the existing list.
+ * if nodups, skip objects whose name already appears in memory.
  * stop gracefully if we run out of memory.
  * return 0 if all ok, else -1.
  */
 int
-db_read (fp)
+db_read (fp, nodups)
 FILE *fp;
+int nodups;
 {
 	char buf[MAXDBLINE];
+	char **names = 0;
+	int nnames = 0;
 	long len;
 	int ret;
 
@@ -468,24 +524,48 @@ FILE *fp;
 	    db_init();
 
 	/* set up to run the progress meter based on file position */
-	(void) fseek (fp, 0L, 2);
+	(void) fseek (fp, 0L, SEEK_END);
 	len = ftell (fp);
-	(void) fseek (fp, 0L, 0);
+	(void) fseek (fp, 0L, SEEK_SET);
 	pm_set (0);
 
+	if (nodups) {
+	    /* build sorted list of existing names */
+	    DBScan dbs;
+	    int mask = ALLM;
+	    Obj *op;
+
+	    nnames = 0;
+	    names = (char **) malloc (db_n() * sizeof(char *));
+	    if (!names) {
+		xe_msg ("No memory to check dups", 1);
+		return (-1);
+	    }
+	    for (db_scaninit(&dbs,mask,NULL, 0); (op = db_scan(&dbs))!=NULL; )
+		names[nnames++] = op->o_name;
+	    if (nnames > db_n()) {
+		printf ("db_read: too many objects! %d %d\n", db_n(), nnames);
+		exit (1);
+	    }
+
+	    qsort ((void *)names, nnames, sizeof(char *), name_cmpf);
+	}
+
 	/* read each line from the file and add to the db */
 	ret = 0;
 	while (nxt_db (buf, sizeof(buf), fp) == 0) {
 	    pm_set ((int)(ftell(fp)*100/len)); /* update progress meter */
+	    if (nodups && chk_name (buf, names, nnames) == 0)
+		continue;
 	    if (db_crack_line (buf) < 0) {
 		ret = -1;
 		break;
 	    }
 	}
 
-	/* discard any duplicates */
-	if (db_wantnodups)
-	    db_nodups();
+	/* finished with names */
+	if (names)
+	    free ((void *)names);
 
 	/* should have hit eof */
 	if (ret == 0)
@@ -504,7 +584,7 @@ db_connect_fifo()
 	char *fn;
 
 	/* get the fifo name from the X resource */
-	fn = XGetDefault (XtD, myclass, dbfifores);
+	fn = getXRes (dbfifores);
 	if (!fn) {
 	    char msg[256];
 	    (void) sprintf (msg, "No `%.200s' resource.\n", dbfifores);
@@ -575,10 +655,23 @@ Obj *op;
 	    op->f_epoch = tmp;
 	    break;
 	case F_MAG:
-	    op->f_mag = atod (bp) * MAGSCALE;
+	    set_fmag (op, atod(bp));
 	    break;
 	case F_SIZE:
-	    op->f_size = atod (bp);
+	    op->f_size = (short)floor(atod(bp) + 0.5);
+	    {
+		/* galaxies can have minor axis and position angle subfields */
+		char *sflds[MAXFLDS];
+		int nsf = get_fields(bp, SUBFLD, sflds);
+
+		if (nsf == 3) {
+		    set_ratio(op, op->s_size, atod(sflds[1]));
+		    set_pa(op,degrad(atod(sflds[2])));
+		} else {
+		    set_ratio(op,1,1);	/* round */
+		    set_pa(op,0.0);
+		}
+	    }
 	    break;
 	case F_CLASS:
 	    switch (bp[0]) {
@@ -614,7 +707,7 @@ Obj *op;
 	    op->e_a = atod (bp);
 	    break;
 	case E_N:
-	    op->e_n = atod (bp);
+	    /* retired */
 	    break;
 	case E_E:
 	    op->e_e = atod (bp);
@@ -766,8 +859,6 @@ static int
 db_crack_line (s)
 char s[];
 {
-	static char nomem[] = "Insufficient memory to load this database\n";
-#define	MAXFLDS	20		/* must be more than on any expected line */
 	char *flds[MAXFLDS];	/* point to each field for easy reference */
 	char *sflds[MAXFLDS];	/* point to each sub field for easy reference */
 	char copy[MAXDBLINE];	/* work copy; leave s untouched */
@@ -805,16 +896,22 @@ char s[];
 		xe_msg (nomem, 0);
 		return (-1);
 	    }
-	    op->type = FIXED;
+	    op->o_type = FIXED;
 	    nsf = get_fields(flds[1], SUBFLD, sflds);
 	    if (nsf > 1 && db_set_field (sflds[1], F_CLASS, PREF_MDY, op) < 0) {
-		(void) sprintf(msg,"Bad class `%c' for fixed object `%.450s'\n",
-							    *sflds[1], flds[0]);
+		(void) sprintf (msg,"%.100s: Bad class `%c'",flds[0],*sflds[1]);
 		xe_msg (msg, 0);
 		return (-1);
 	    }
-	    if (nsf > 2)
+	    if (nsf > 2) {
+		if (op->f_class == 'G') {
+		    (void) sprintf (msg, "%.100s: no spectra with galaxies",
+								    flds[0]);
+		    xe_msg (msg, 0);
+		    return (-1);
+		}
 		(void) db_set_field (sflds[2], F_SPECT, PREF_MDY, op);
+	    }
 	    for (i = 2; i < ASIZ(ids)+2; i++)
 		(void) db_set_field (flds[i], ids[i-2], PREF_MDY, op);
 	    (void) db_set_field (nf>5 && flds[5][0] ? flds[5] : "2000",
@@ -840,7 +937,7 @@ char s[];
 		xe_msg (nomem, 0);
 		return (-1);
 	    }
-	    op->type = ELLIPTICAL;
+	    op->o_type = ELLIPTICAL;
 	    for (i = 2; i < ASIZ(ids)+2; i++)
 		(void) db_set_field (flds[i], ids[i-2], PREF_MDY, op);
 	    if (nf == 14)
@@ -862,7 +959,7 @@ char s[];
 		xe_msg (nomem, 0);
 		return (-1);
 	    }
-	    op->type = HYPERBOLIC;
+	    op->o_type = HYPERBOLIC;
 	    for (i = 2; i < ASIZ(ids)+2; i++)
 		(void) db_set_field (flds[i], ids[i-2], PREF_MDY, op);
 	    if (nf == 12)
@@ -884,7 +981,7 @@ char s[];
 		xe_msg (nomem, 0);
 		return (-1);
 	    }
-	    op->type = PARABOLIC;
+	    op->o_type = PARABOLIC;
 	    for (i = 2; i < ASIZ(ids)+2; i++)
 		(void) db_set_field (flds[i], ids[i-2], PREF_MDY, op);
 	    if (nf == 11)
@@ -907,7 +1004,7 @@ char s[];
 		xe_msg (nomem, 0);
 		return (-1);
 	    }
-	    op->type = EARTHSAT;
+	    op->o_type = EARTHSAT;
 	    for (i = 2; i < ASIZ(ids)+2; i++)
 		(void) db_set_field (flds[i], ids[i-2], PREF_MDY, op);
 	    break;
@@ -977,7 +1074,7 @@ db_init()
 	    "Mercury", "Venus", "Mars", "Jupiter", "Saturn",
 	    "Uranus", "Neptune", "Pluto", "Sun", "Moon",
 	};
-	static Obj plan_objs[NOBJ];	/* OBJX/Y can be anything so use Obj */
+	static Obj plan_objs[NOBJ];	/* OBJXYZ can be anything so use Obj */
 	static char *plan_dblist[1] = {(char *)plan_objs};
 	int i;
 
@@ -1002,141 +1099,28 @@ db_init()
 	db[PLANET].ncp = NOBJ;
 	for (i = MERCURY; i <= MOON; i++) {
 	    Obj *op = OBJP (PLANET, i);
-	    op->type = PLANET;
+	    op->o_type = PLANET;
 	    (void) strncpy (op->o_name, planet_names[i], sizeof(op->o_name)-1);
-	    op->pl.code = i;
+	    op->pl.pl_code = i;
 	}
 }
 
-/* save the type of object being sorted (for use by db_nodups()) */
-static int dup_t;
-
-/* qsort name comparison function (for use by db_nodups()) */
-static int
-dup_namecmpf (v1, v2)
-#ifdef __STDC__
-const
-#endif
-void *v1, *v2;
-{
-	int n1 = *((int *)v1);
-	int n2 = *((int *)v2);
-	Obj *op1 = OBJP (dup_t, n1);
-	Obj *op2 = OBJP (dup_t, n2);
-
-	return (strcmp (op1->o_name, op2->o_name));
-}
-
-/* qsort n comparison function (for use by db_nodups()) */
-static int
-dup_ncmpf (v1, v2)
-#ifdef __STDC__
-const
-#endif
-void *v1, *v2;
-{
-	int n1 = *((int *)v1);
-	int n2 = *((int *)v2);
-
-	return (n1 - n2);
-}
-
-/* eliminate all db entries with duplicate names in each type of object */
+/* free all the memory associated with the given DBMem */
 static void
-db_nodups()
+db_free (dmp)
+DBMem *dmp;
 {
-	int t;
-
-	watch_cursor (1);
-
-	for (t = 0; t < NOBJTYPES; t++) {
-	    DBMem *dbm;
-	    Obj *op1, *op2;
-	    int *idx;
-	    int n, s;
-	    int i, j;
-	    int nxtidx;
-	    int ndups;
-
-	    /* skip undefined objects (of course) and leave the predefined
-	     * objects alone.
-	     */
-	    if (t == UNDEFOBJ || t == PLANET)
-		continue;
-
-	    /* handy references */
-	    dbm = &db[t];
-	    n = dbm->nobj;
-	    s = dbm->size;
-
-	    /* build a list of indices sorted by name */
-	    idx = (int *) malloc (n * sizeof(int));
-	    if (!idx) {
-		xe_msg ("No memory for dup temp array.", 0);
-		continue;
-	    }
-	    for (i = 0; i < n; i++)
-		idx[i] = i;
-	    dup_t = t;
-	    qsort ((void *)idx, n, sizeof(int), dup_namecmpf);
-
-	    /* throw out all indeces which only occur once *in place*.
-	     * this will leave us with ndups at the front of idx.
-	     */
-	    for (i = 1, ndups = 0; i < n; i++) {
-		op1 = OBJP(t,idx[i]);
-		op2 = OBJP(t,idx[i-1]);
-		if (strcmp (op1->o_name, op2->o_name) == 0)
-		    idx[ndups++] = idx[i];
-	    }
-
-	    if (ndups == 0)
-		goto done;	/* no dups */
-
-	    /* sort the dup list in ascending order for faster checking */
-	    qsort ((void *)idx, ndups, sizeof(int), dup_ncmpf);
-
-	    /* now compress each of the ndups entries in idx out of the real db.
-	     * don't discard anything prior to the last checkpoint.
-	     * consider starting at idx[0] since it's the first dup to go.
-	     * exit loop with j being the new net number of objects.
-	     */
-	    nxtidx = 0;
-	    for (i = j = idx[0] > dbm->ncp ? idx[0] : dbm->ncp; i < n; i++) {
-		int in, k;
-
-		/* see if i is in idx[].
-		 * nxtidx and testing idx[k] against i keeps a bound on the
-		 * portion of idx which might contain i.
-		 */
-		in = 0;
-		for (k = nxtidx; k < ndups && idx[k] <= i; k++)
-		    if (idx[k] == i) {
-			in = 1;
-			nxtidx = k+1;
-			break;
-		    }
-
-		/* if i is in idx[] then it's a dup and we don't copy it */
-		if (!in) {
-		    /* copy i to j (avoid a copy onto itself) */
-		    if (i > j) {
-			op1 = OBJP(t,i);
-			op2 = OBJP(t,j);
-			(void) memcpy ((void *)op2, (void *)op1, s);
-		    }
-		    j++;
-		}
-	    }
-
-	    dbm->nobj = j;
-
-	done:
-	    free ((char *)idx);
-		
+	if (dmp->dblist) {
+	    int i;
+	    for (i = 0; i < dmp->nmem/DBCHUNK; i++)
+		free (dmp->dblist[i]);
+	    free ((char *)dmp->dblist);
+	    dmp->dblist = NULL;
 	}
 
-	watch_cursor (0);
+	dmp->nobj = 0;
+	dmp->nmem = 0;
+	dmp->ncp = 0;
 }
 
 /* read database file fp and put next valid entry (sans trailing \n) into buf.
@@ -1306,12 +1290,10 @@ XtInputId *idp;         /* pointer to input id */
 	    npartial = 0;
 	}
 
-	/* turn off the watch cursor if there is not likely to be more coming.
-	 * Also, throw out dups and inform everyone about all the new stuff.
+	/* if there is not likely to be more coming inform everyone about all
+	 * the new stuff and turn off the watch cursor.
 	 */
 	if (!npartial) {
-	    if (db_wantnodups)
-		db_nodups();
 	    all_newdb (1);
 	    watch_cursor (0);
 	}
