@@ -18,6 +18,7 @@
 #include <Xm/ToggleB.h>
 #include <Xm/RowColumn.h>
 #include <Xm/FileSB.h>
+#include <Xm/Text.h>
 #include <Xm/Separator.h>
 
 #include "P_.h"
@@ -25,30 +26,32 @@
 #include "circum.h"
 
 extern Widget	toplevel_w;
-extern char	*myclass;
 #define	XtD	XtDisplay(toplevel_w)
+extern Colormap xe_cm;
 
 extern FILE *fopenh P_((char *name, char *how));
 extern Obj *db_scan P_((DBScan *sp));
+extern char *getXRes P_((char *name));
 extern char *syserrstr P_((void));
 extern int db_n P_((void));
 extern int db_n_cp P_((void));
-extern int db_read P_((FILE *fp));
+extern int db_read P_((FILE *fp, int nodups));
 extern void all_newdb P_((int appended));
 extern void db_clr_cp P_((void));
 extern void db_connect_fifo P_((void));
 extern void db_del_all P_((void));
 extern void db_del_cp P_((void));
-extern void db_scaninit P_((DBScan *sp));
+extern void db_scaninit P_((DBScan *sp, int mask, ObjF *op, int nop));
 extern void db_set_cp P_((void));
-extern void db_setnodups P_((int whether));
 extern void hlp_dialog P_((char *tag, char *deflt[], int ndeflt));
 extern void prompt_map_cb P_((Widget w, XtPointer client, XtPointer call));
 extern void set_something P_((Widget w, char *resource, XtArgVal value));
 extern void query P_((Widget tw, char *msg, char *label1, char *label2,
-    char *label3, void (*func1)(), void (*func2)(), void (*func3)()));
+    char *label3, void (*func1)(void), void (*func2)(void),
+    void (*func3)(void)));
 extern void set_xmstring P_((Widget w, char *resource, char *txt));
 extern void watch_cursor P_((int want));
+extern void wtip P_((Widget w, char *tip));
 extern void xe_msg P_((char *msg, int app_modal));
 
 static void db_create_form P_((void));
@@ -59,7 +62,6 @@ static void db_append_cb P_((Widget w, XtPointer client, XtPointer data));
 static void db_del_cp_cb P_((Widget w, XtPointer client, XtPointer data));
 static void db_set_cp_cb P_((Widget w, XtPointer client, XtPointer data));
 static void db_clr_cp_cb P_((Widget w, XtPointer client, XtPointer data));
-static void db_nodups_cb P_((Widget w, XtPointer client, XtPointer data));
 static void delete_all_confirmed P_((void));
 static void db_del_all_cb P_((Widget w, XtPointer client, XtPointer data));
 static void db_openfifo_cb P_((Widget w, XtPointer client, XtPointer data));
@@ -67,8 +69,9 @@ static void db_help_cb P_((Widget w, XtPointer client, XtPointer data));
 static void db_helpon_cb P_((Widget w, XtPointer client, XtPointer data));
 static void db_close_cb P_((Widget w, XtPointer client, XtPointer data));
 
-static Widget dbform_w;			/* main Form dialog */
-static Widget report_w;			/* label with the dbstats */
+static Widget dbform_w;		/* main Form dialog */
+static Widget report_w;		/* label with the dbstats */
+static Widget nodups_w;		/* TB whether to avoid dups on file reads */
 
 void
 db_manage()
@@ -87,8 +90,8 @@ db_manage()
 /* called when the database beyond NOBJ has changed in any way.
  * as odd as this seems since *this* menu controls the db contents, this was
  *   added when the db fifo input path was added. it continued to be handy
- *   when initial db files could be loaded automatically and we started 
- *   to include info about the user-defined objects in the tallys in v2.7.
+ *   when initial db files could be loaded automatically and we introduced
+ *   fields stars in 2.9.
  * all we do is update our tally, if we are up at all.
  */
 /* ARGSUSED */
@@ -100,13 +103,13 @@ int appended;
 	    db_set_report();
 }
 
-/* called when either of the user-defined objects has changed.
+/* called when any of the user-defined objects have changed.
  * all we do is update our tally, if we are up at all.
  */
 /* ARGSUSED */
 void
 db_newobj (id)
-int id;	/* OBJX or OBJY */
+int id;	/* OBJXYZ */
 {
 	if (dbform_w && XtIsManaged (dbform_w))
 	    db_set_report();
@@ -119,7 +122,7 @@ Cursor c;
 {
 	Window win;
 
-	if (dbform_w && (win = XtWindow(dbform_w))) {
+	if (dbform_w && (win = XtWindow(dbform_w)) != 0) {
 	    Display *dsp = XtDisplay(dbform_w);
 	    if (c)
 		XDefineCursor (dsp, win, c);
@@ -158,7 +161,9 @@ db_create_form ()
 	XtSetArg(args[n], XmNdefaultPosition, False);  n++;
 	XtSetArg (args[n], XmNverticalSpacing, 5); n++;
 	XtSetArg (args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
+	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
 	dbform_w = XmCreateFormDialog(toplevel_w, "DBManage", args, n);
+	set_something (dbform_w, XmNcolormap, (XtArgVal)xe_cm);
 	XtAddCallback (dbform_w, XmNmapCallback, prompt_map_cb, NULL);
 	XtAddCallback (dbform_w, XmNhelpCallback, db_help_cb, 0);
 
@@ -166,7 +171,7 @@ db_create_form ()
 	 * setting XmNdialogTitle in the Form didn't work..
 	 */
 	n = 0;
-	XtSetArg (args[n], XmNtitle, "xephem Data Base"); n++;
+	XtSetArg (args[n], XmNtitle, "xephem Database Load/Delete"); n++;
 	XtSetValues (XtParent(dbform_w), args, n);
 
 	/* create the menu bar across the top */
@@ -190,24 +195,24 @@ db_create_form ()
 	    set_xmstring (cb_w, XmNlabelString, "Control");
 	    XtManageChild (cb_w);
 
-	    /* make the nodups toggle */
-
-	    n = 0;
-	    XtSetArg (args[n], XmNvisibleWhenOff, True); n++;
-	    w = XmCreateToggleButton (pd_w, "NoDups", args, n);
-	    XtAddCallback (w, XmNvalueChangedCallback, db_nodups_cb, 0);
-	    set_xmstring(w, XmNlabelString, "No dups");
-	    XtManageChild (w);
-	    db_setnodups (XmToggleButtonGetState(w));
-
 	    /* make the open fifo button */
 
 	    n = 0;
 	    w = XmCreatePushButton (pd_w, "OpenFIFO", args, n);
 	    XtAddCallback (w, XmNactivateCallback, db_openfifo_cb, 0);
 	    set_xmstring(w, XmNlabelString, "Open DBFIFO");
+	    wtip (w, "Activate import path for Object definitions");
 	    XtManageChild (w);
 
+	    /* make the no-dups toggle button */
+
+	    n = 0;
+	    XtSetArg (args[n], XmNvisibleWhenOff, True);  n++;
+	    nodups_w = XmCreateToggleButton (pd_w, "NoDups", args, n);
+	    set_xmstring(nodups_w, XmNlabelString, "No Dups");
+	    wtip (nodups_w, "When on, skip objects whose name already exists in memory when reading");
+	    XtManageChild (nodups_w);
+
 	    /* add a separator */
 
 	    n = 0;
@@ -219,6 +224,7 @@ db_create_form ()
 	    n = 0;
 	    w = XmCreatePushButton (pd_w, "Close", args, n);
 	    XtAddCallback (w, XmNactivateCallback, db_close_cb, 0);
+	    wtip (w, "Close this dialog");
 	    XtManageChild (w);
 
 	/* make the Delete pulldown */
@@ -239,6 +245,7 @@ db_create_form ()
 	    w = XmCreatePushButton (pd_w, "SetCP", args, n);
 	    XtAddCallback (w, XmNactivateCallback, db_set_cp_cb, 0);
 	    set_xmstring(w, XmNlabelString, "Set Checkpoint");
+	    wtip (w, "Tag the current set of objects in memory");
 	    XtManageChild (w);
 
 	    /* make the clear-checkpoint button */
@@ -247,6 +254,7 @@ db_create_form ()
 	    w = XmCreatePushButton (pd_w, "ClrCP", args, n);
 	    XtAddCallback (w, XmNactivateCallback, db_clr_cp_cb, 0);
 	    set_xmstring(w, XmNlabelString, "Clear Checkpoint");
+	    wtip (w, "Discard tags set on objects currently in memory");
 	    XtManageChild (w);
 
 	    /* add a separator */
@@ -261,6 +269,7 @@ db_create_form ()
 	    w = XmCreatePushButton (pd_w, "DelToCP", args, n);
 	    XtAddCallback (w, XmNactivateCallback, db_del_cp_cb, 0);
 	    set_xmstring(w, XmNlabelString, "Delete to Checkpoint");
+	    wtip (w, "Delete all objects in memory which have not been tagged");
 	    XtManageChild (w);
 
 	    /* make the delete-all button */
@@ -268,6 +277,7 @@ db_create_form ()
 	    n = 0;
 	    w = XmCreatePushButton (pd_w, "DelAll", args, n);
 	    XtAddCallback (w, XmNactivateCallback, db_del_all_cb, 0);
+	    wtip (w, "Delete all objects from memory");
 	    set_xmstring(w, XmNlabelString, "Delete All");
 	    XtManageChild (w);
 
@@ -298,15 +308,21 @@ db_create_form ()
 		XmStringFree(str);
 	    }
 
-	/* make the report label */
+	/* make the report output-only text */
 
 	n = 0;
 	XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
 	XtSetArg (args[n], XmNtopWidget, mb_w); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
-	XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
-	report_w = XmCreateLabel (dbform_w, "Report", args, n);
+	XtSetArg (args[n], XmNeditable, False); n++;
+	XtSetArg (args[n], XmNeditMode, XmMULTI_LINE_EDIT); n++;
+	XtSetArg (args[n], XmNcursorPositionVisible, False); n++;
+	XtSetArg (args[n], XmNblinkRate, 0); n++;
+	XtSetArg (args[n], XmNrows, 17); n++;
+	XtSetArg (args[n], XmNcolumns, 32); n++;
+	report_w = XmCreateText (dbform_w, "Report", args, n);
+	wtip (report_w, "Breakdown of number and types of objects in memory");
 	XtManageChild (report_w);
 
 	/* create separator below report label */
@@ -330,6 +346,7 @@ db_create_form ()
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
 	fsb_w = XmCreateFileSelectionBox (dbform_w, "FSB", args, n);
 	initFSB (fsb_w);	/* do this before managing for Ultrix :-) */
+	wtip (fsb_w, "Select a file to read and add objects to memory");
 	XtManageChild (fsb_w);
 
 	    /* use the Ok button to mean Append */
@@ -361,38 +378,40 @@ Widget fsb_w;
 	char msg[128];
 	char *res;
 
-	res = XGetDefault (XtD, myclass, dbd);
+	res = getXRes (dbd);
 	if (!res) {
 	    res = "edb";
-	    (void)sprintf(msg,"Can not get %s.%s -- using %s", myclass,dbd,res);
+	    (void)sprintf(msg,"Can not get %s -- using %s", dbd, res);
 	    xe_msg (msg, 0);
 	}
 	set_xmstring (fsb_w, XmNdirectory, res);
 
-	res = XGetDefault (XtD, myclass, dbp);
+	res = getXRes (dbp);
 	if (!res) {
 	    res = "*.edb";
-	    (void)sprintf(msg,"Can not get %s.%s -- using %s", myclass,dbp,res);
+	    (void)sprintf(msg,"Can not get %s -- using %s", dbp, res);
 	    xe_msg (msg, 0);
 	}
 	set_xmstring (fsb_w, XmNpattern, res);
 }
 
 /* compile the stats report into report_w.
- * this is a cheap way to indicate the number of objects in the db.
+ * N.B. we do not include the planets nor the user objects.
  */
 static void
 db_set_report()
 {
 	DBScan dbs;
 	char report[1024];
+	int mask = ALLM & ~PLANETM;
 	Obj *op;
 	int nes=0, ne=0, np=0, nh=0;
 	int nc=0, ng=0, nj=0, nn=0, npn=0, nl=0, nq=0, nr=0, ns=0, no=0;
+	int ndash;
 	int t=0;
 
-	for (db_scaninit(&dbs); (op=db_scan(&dbs))!=NULL; ) {
-	    switch (op->type) {
+	for (db_scaninit(&dbs, mask, NULL, 0); (op = db_scan(&dbs))!=NULL; ){
+	    switch (op->o_type) {
 	    case FIXED:
 		switch (op->f_class) {
 		case 'C': case 'U': case 'O': nc++; t++; break;
@@ -412,14 +431,15 @@ db_set_report()
 	    case HYPERBOLIC: nh++; t++; break;
 	    case PARABOLIC: np++; t++; break;
 	    case EARTHSAT: nes++; t++; break;
-	    case PLANET: break;
 	    case UNDEFOBJ: break;
 	    default:
-		printf ("Unknown object type: %d\n", op->type);
+		printf ("Unknown object type: %d\n", op->o_type);
 		exit (1);
 	    }
 	}
 
+	ndash = t > 0 ? (int)(log10((double)t) + 1) : 1;
+
 	(void) sprintf (report, "\
 %6d Solar -- elliptical\n\
 %6d Solar -- hyperbolic\n\
@@ -435,14 +455,15 @@ db_set_report()
 %6d Supernova Remnants (R)\n\
 %6d Stars (S,V,D,B,M,T)\n\
 %6d Undefined\n\
-------\n\
-%6d Total objects in memory\n\
-%6d Objects checkpointed\n\
-\n\
+%*s%.*s\n\
+%6d Total Objects in Memory\n\
+%6d Objects Checkpointed\n\
 ",
-	ne, nh, np, nes, nc, ng, npn, nn, nl, nq, nj, nr, ns, no, t,
-							    db_n_cp()-NOBJ);
-	set_xmstring (report_w, XmNlabelString, report);
+	ne, nh, np, nes, nc, ng, npn, nn, nl, nq, nj, nr, ns, no,
+	6-ndash, "", ndash, "------",
+	t, db_n_cp()-NOBJ);
+
+	set_something (report_w, XmNvalue, (XtArgVal)report);
 }
 
 /* try to open name for read access.
@@ -489,7 +510,7 @@ XtPointer data;
 	XmStringGetLtoR (s->value, XmSTRING_DEFAULT_CHARSET, &sp);
 	fp = fileopen (sp);
 	if (fp) {
-	    if (db_read (fp) < 0) {
+	    if (db_read (fp, XmToggleButtonGetState (nodups_w)) < 0) {
 		char msg[128];
 		(void) sprintf (msg, "Error reading `%.100s'", sp);
 		xe_msg (msg, 0);
@@ -535,21 +556,10 @@ Widget w;
 XtPointer client;
 XtPointer data;
 {
-	db_clr_cp();	/* also clears out dups if enabled */
+	db_clr_cp();
 	db_set_report();
 }
 
-/* callback from the nodups toggle */
-/* ARGSUSED */
-static void
-db_nodups_cb (w, client, data)
-Widget w;
-XtPointer client;
-XtPointer data;
-{
-	db_setnodups (XmToggleButtonGetState(w));
-}
-
 /* function to call when the Delete All confirmation is affirmative */
 static void
 delete_all_confirmed()
