@@ -24,9 +24,12 @@
 #include "preferences.h"
 
 extern void xe_msg P_((char *msg, int app_modal));
+extern void now_lst P_((Now *np, double *lstp));
 
 typedef double MAT3x3[3][3];
 
+#define ESAT_MAG        2       /* default satellite magnitude */
+
 static void GetSatelliteParams P_((Obj *op));
 static void GetSiteParams P_((Now *np));
 static void esat_main P_((double CrntTime, Obj *op));
@@ -55,12 +58,12 @@ static void GetBearings P_((double SatX, double SatY, double SatZ,
     double *Azimuth, double *Elevation));
 static int Eclipsed P_((double SatX, double SatY, double SatZ,
     double SatRadius, double CrntTime));
-static void InitOrbitRoutines P_((double EpochDay));
+static void InitOrbitRoutines P_((double EpochDay, int AtEod));
 
 
 
 /* given a Now and an Obj with info about an earth satellite in the es_* fields
- * fill in the s_* sky fields describing the satellite.{-
+ * fill in the s_* sky fields describing the satellite.
  * as usual, we compute the geocentric ra/dec precessed to np->n_epoch and
  * compute topocentric altitude accounting for refraction.
  * return 0 if all ok, else -1.
@@ -71,7 +74,7 @@ Now *np;
 Obj *op;
 {
 	double CrntTime;
-	double dalt;
+	double ra, dec, alt;
 
 	/* extract the xephem data forms into those used by orbit.
 	 * (we reuse orbit's function names that read these from files.)
@@ -102,21 +105,29 @@ Obj *op;
 	fflush (stdout);
 #endif
 
-	/* precess ra/dec to current epoch if it's not EOD */
-	if (epoch != EOD) {
-	    double dra = op->s_ra, ddec = op->s_dec;
-	    precess (mjd, epoch, &dra, &ddec);
-	    op->s_ra = dra;
-	    op->s_dec = ddec;
-	}
+	/* correct altitude for refraction */
+	alt = op->s_alt;
+	refract (pressure, temp, alt, &alt);
+	op->s_alt = alt;
 
-	/* allow for refraction of altitude */
-	dalt = op->s_alt;
-	refract (pressure, temp, dalt, &dalt);
-	op->s_alt = dalt;
+	/* find s_ra/dec, depending on current options. */
+	if (pref_get(PREF_EQUATORIAL) == PREF_TOPO) {
+	    double ha, lst;
+	    aa_hadec (lat, alt, (double)op->s_az, &ha, &dec);
+	    now_lst (np, &lst);
+	    ra = hrrad(lst) - ha;
+	    range (&ra, 2*PI);
+	} else {
+	    ra = op->s_gaera;
+	    dec = op->s_gaedec;
+	}
+	if (epoch != EOD)
+	    precess (mjd, epoch, &ra, &dec);
+	op->s_ra = ra;
+	op->s_dec = dec;
 
 	/* just make up a size and brightness */
-	op->s_mag = ESAT_MAG *MAGSCALE;
+	set_smag (op, ESAT_MAG);
 	op->s_size = 0;
 
 	return (0);
@@ -257,6 +268,9 @@ Now *np;
 #endif
 }
  
+/* compute geocentric gaera/dec @ eod, and topocentric alt/az _sans refraction_.
+ * as well as satellites-specific fields.
+ */
 static void
 esat_main (CrntTime, op)
 double CrntTime;
@@ -269,8 +283,8 @@ Obj *op;
 	double MeanAnomaly,TrueAnomaly;
 	double SemiMajorAxis;
 	double Radius;              /* From geocenter                  */
-	double SatX,SatY,SatZ;	/* In Right Ascension based system */
-	double SatVX,SatVY,SatVZ;   /* Kilometers/second		   */
+	double SatX,SatY,SatZ;	    /* In Right Ascension based system */
+	double SatVX,SatVY,SatVZ;   /* Kilometers/second	       */
 	double SiteX,SiteY,SiteZ;
 	double SiteVX,SiteVY;
 	double SiteMatrix[3][3];
@@ -282,7 +296,7 @@ Obj *op;
 	double RangeRate;
 	double dtmp;
 
-	InitOrbitRoutines(CrntTime);
+	InitOrbitRoutines(CrntTime, 1);
 
 	SemiMajorAxis = 331.25 * exp(2*log(MinutesPerDay/epochMeanMotion)/3);
 	GetPrecession(SemiMajorAxis,Eccentricity,Inclination,&RAANPrecession,
@@ -322,8 +336,8 @@ Obj *op;
 	/* ECD: insure 0..2PI ra */
 	dtmp = atan2 (SatY, SatX);
 	range (&dtmp, 2*PI);
-	op->s_ra = dtmp;
-	op->s_dec = atan2 (SatZ, sqrt(SatX*SatX + SatY*SatY));
+	op->s_gaera = dtmp;
+	op->s_gaedec = atan2 (SatZ, sqrt(SatX*SatX + SatY*SatY));
 
 	GetSitPosition(SiteLat,SiteLong,SiteAltitude,CrntTime,
 		    &SiteX,&SiteY,&SiteZ,&SiteVX,&SiteVY,SiteMatrix);
@@ -688,8 +702,9 @@ double SatX,SatY,SatZ,SatRadius,CrntTime;
    Also init the sidereal reference				*/
 
 static void
-InitOrbitRoutines(EpochDay)
+InitOrbitRoutines(EpochDay, AtEod)
 double EpochDay;
+int AtEod;
 {
     double T,T2,T3,Omega;
     int n;
@@ -705,7 +720,7 @@ double EpochDay;
     SidReference -= floor(SidReference);
 
     /* Omega is used to correct for the nutation and the abberation */
-    Omega = (259.18 - 1934.142*T) * RadiansPerDegree;
+    Omega = AtEod ? (259.18 - 1934.142*T) * RadiansPerDegree : 0.0;
     n = Omega / PI2;
     Omega -= n*PI2;
 
