@@ -158,7 +158,7 @@ int fracbase;
 	}
 }
 
-/* print angle ra, in radians, to widget w according to the precision pref.
+/* print ra, in radians, to widget w in hours according to the precision pref.
  */
 void
 f_ra (w, ra)
@@ -168,10 +168,10 @@ double ra;
 	if (pref_get(PREF_DPYPREC) == PREF_LOPREC)
 	    f_sexa (w, radhr(ra), 2, 600);
 	else
-	    f_sexa (w, radhr(ra), 2, 36000);
+	    f_sexa (w, radhr(ra), 2, 360000);
 }
 
-/* print angle ra, in radians, into string out[] according to precision pref.
+/* print ra, in radians, into string out[] in hours according to precision pref.
  */
 void
 fs_ra (out, ra)
@@ -181,7 +181,35 @@ double ra;
 	if (pref_get(PREF_DPYPREC) == PREF_LOPREC)
 	    fs_sexa (out, radhr(ra), 2, 600);
 	else
-	    fs_sexa (out, radhr(ra), 2, 36000);
+	    fs_sexa (out, radhr(ra), 2, 360000);
+}
+
+/* print dec, a, in rads, as degrees to widget w according to desired
+ * precision preference.
+ */
+void
+f_prdec(w, a)
+Widget w;
+double a;
+{
+	if (pref_get(PREF_DPYPREC) == PREF_LOPREC)
+	    f_sexa (w, raddeg(a), 3, 60);
+	else
+	    f_sexa (w, raddeg(a), 3, 36000);
+}
+
+/* print dec, a, in rads, as degrees into string out[] according to desired
+ * precision preference.
+ */
+void
+fs_prdec(out, a)
+char out[];
+double a;
+{
+	if (pref_get(PREF_DPYPREC) == PREF_LOPREC)
+	    fs_sexa (out, raddeg(a), 3, 60);
+	else
+	    fs_sexa (out, raddeg(a), 3, 36000);
 }
 
 /* print time, t, as hh:mm:ss */
@@ -404,7 +432,11 @@ double *np;	/* new value */
 #define	SCANSEX(new,old)						\
 	if (*bp == '-') {nneg = 1; bp++;}				\
 	if (sscanf (bp, "%lf", &new) != 1) new = old;			\
-	while((c=*bp)){bp++;if(c==':'||c=='/'||c==';'||c==','||c=='-')break;}
+	while ((c=(*bp)) != '\0') {					\
+	    bp++;							\
+	    if (c==':' || c=='/' || c==';' || c==',' || c=='-')		\
+		break;							\
+	}
 
 	/* get h:m:s components of o in case bp[] defers.
 	 * and constrain to positive for now.
@@ -455,12 +487,12 @@ int pref;       /* one of PREF_X for PREF_DATE_FORMAT */
 int *m, *y;
 double *d;
 {
-        char *bp0, c;
         double comp[3]; /* the three components */
         int set[3];     /* set[i] is 1 if component i is present */
 	int in;		/* scan state: 1 while we are in a component */
 	int ncomp;	/* incremented as each component is discovered */
 	int ndp;	/* number of decimal points in last component */
+        char *bp0 = NULL, c;
 
 	set[0] = set[1] = set[2] = 0;
 	ncomp = 0;
@@ -471,7 +503,7 @@ double *d;
 	 * '\0' counts as a component terminator too.
 	 */
 	do {
-	    /* here, bp is the next character to be investigated */
+	    /* here, *bp is the next character to be investigated */
 	    c = *bp;
 	    if (c == '.' || isdigit(c) || (c == '-' && !in)) {
 		/* found what passes for a floating point number */
