@@ -28,9 +28,10 @@
 
 extern Widget toplevel_w;
 #define	XtD XtDisplay(toplevel_w)
-extern char *myclass;
+extern Colormap xe_cm;
 
 extern FILE *fopenh P_((char *name, char *how));
+extern char *getXRes P_((char *name));
 extern void prompt_map_cb P_((Widget w, XtPointer client, XtPointer call));
 extern void set_something P_((Widget w, char *resource, XtArgVal value));
 extern void xe_msg P_((char *msg, int app_modal));
@@ -106,7 +107,9 @@ char *tag;
 	XtSetArg (args[n], XmNdefaultPosition, False); n++;
 	XtSetArg (args[n], XmNautoUnmanage, False); n++;
 	XtSetArg (args[n], XmNfractionBase, 9); n++;
+	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
 	hlp_w = XmCreateFormDialog (toplevel_w, "Help", args, n);
+	set_something (hlp_w, XmNcolormap, (XtArgVal)xe_cm);
 	XtAddCallback (hlp_w, XmNmapCallback, prompt_map_cb, NULL);
 
 	(void) sprintf (title, "xephem Help on %s", tag);
@@ -138,7 +141,7 @@ char *tag;
 	XtSetArg (args[n], XmNeditMode, XmMULTI_LINE_EDIT); n++;
 	XtSetArg (args[n], XmNeditable, False); n++;
 	XtSetArg (args[n], XmNcursorPositionVisible, False); n++;
-	t_w = XmCreateScrolledText (hlp_w, "ScrolledText", args, n);
+	t_w = XmCreateScrolledText (hlp_w, "HelpText", args, n);
 	XtManageChild (t_w);
 
 	return (t_w);
@@ -173,7 +176,7 @@ char *tag;
 	FILE *fp;
 
 	if (!fn) {
-	    fn = XGetDefault (XtD, myclass, "HELPFILE");
+	    fn = getXRes ("HELPFILE");
 	    if (!fn)
 		fn = hlpfdef;
 	}
