@@ -23,25 +23,32 @@
 #include "P_.h"
 #include "astro.h"
 #include "circum.h"
+#include "preferences.h"
+#include "ps.h"
 
 
 extern Widget	toplevel_w;
 #define	XtD	XtDisplay(toplevel_w)
+extern Colormap xe_cm;
 
 extern Now *mm_get_now P_((void));
 extern Obj *db_basic P_((int id));
 extern Obj *db_scan P_((DBScan *sp));
+extern double delra P_((double dra));
 extern int any_ison P_((void));
-extern int magdiam P_((int fmag, double scale, double mag, double size));
-extern int obj_cir P_((Now *np, Obj *op));
+extern int fs_fetch P_((Now *np, double ra, double dec, double fov,
+    double mag, ObjF **opp));
+extern int magdiam P_((int fmag, int magstp, double scale, double mag,
+    double size));
 extern int skyloc_fifo P_((int test, double ra, double dec, double yr,
     double fov, int mag));
-extern void db_scaninit P_((DBScan *sp));
+extern void db_scaninit P_((DBScan *sp, int mask, ObjF *op, int nop));
 extern void db_update P_((Obj *op));
+extern void f_prdec P_((Widget w, double a));
 extern void f_double P_((Widget w, char *fmt, double f));
-extern void f_pangle P_((Widget w, double a));
 extern void f_ra P_((Widget w, double ra));
-extern void fs_pangle P_((char out[], double a));
+extern void fs_manage P_((void));
+extern void fs_prdec P_((char out[], double a));
 extern void fs_ra P_((char out[], double ra));
 extern void get_something P_((Widget w, char *resource, XtArgVal value));
 extern void hlp_dialog P_((char *tag, char *deflt[], int ndeflt));
@@ -57,6 +64,7 @@ extern void sv_draw_obj P_((Display *dsp, Drawable win, GC gc, Obj *op, int x,
     int y, int diam, int dotsonly));
 extern void timestamp P_((Now *np, Widget w));
 extern void watch_cursor P_((int want));
+extern void wtip P_((Widget w, char *tip));
 extern void xe_msg P_((char *buf, int app_modal));
 
 #define	MINR	10	/* min distance for picking, pixels */
@@ -86,6 +94,7 @@ static void jm_sstats_cb P_((Widget w, XtPointer client, XtPointer call));
 static void jm_option_cb P_((Widget w, XtPointer client, XtPointer call));
 static void jm_scale_cb P_((Widget w, XtPointer client, XtPointer call));
 static void jm_activate_cb P_((Widget w, XtPointer client, XtPointer call));
+static void jm_unmap_cb P_((Widget w, XtPointer client, XtPointer call));
 static void jm_close_cb P_((Widget w, XtPointer client, XtPointer call));
 static void jm_movie_cb P_((Widget w, XtPointer client, XtPointer call));
 static void jm_da_exp_cb P_((Widget w, XtPointer client, XtPointer call));
@@ -95,6 +104,8 @@ static void jm_fill_popup P_((ScreenObj *sop));
 static void jm_help_cb P_((Widget w, XtPointer client, XtPointer call));
 static void jm_helpon_cb P_((Widget w, XtPointer client, XtPointer call));
 static void jm_locfifo_cb P_((Widget w, XtPointer client, XtPointer call));
+static void j_calibline P_((Display *dsp, Drawable win, GC gc, int xc, int yc,
+    char *tag, int tw, int th, int l));
 
 static void add_screenobj P_((Obj *op, int x, int y));
 static void reset_screenobj P_((void));
@@ -113,6 +124,10 @@ static Widget limmag_w;		/* limiting magnitude scale */
 static Widget dt_w;		/* main date/time stamp widget */
 static Widget sdt_w;		/* statistics date/time stamp widget */
 static Widget skybkg_w;		/* toggle for controlling sky background */
+static Pixmap jm_pm;		/* main pixmap */
+static GC j_fgc, j_bgc, j_xgc;	/* various colors and operators */
+static XFontStruct *j_fs;	/* font for labels */
+static int j_cw, j_ch;		/* size of label font */
 static int jm_selecting;	/* set while our fields are being selected */
 static int brmoons;		/* whether we want to brightten the moons */
 static int tags;		/* whether we want tags on the drawing */
@@ -130,6 +145,15 @@ static Widget jmpu_mag_w;	/* popup Mag label */
 #define	MAXSCALE	10.0	/* max scale mag factor */
 #define	MOVIE_STEPSZ	0.25	/* movie step size, hours */
 
+/* field star support */
+static ObjF *fstars;		/* malloced list of field stars, or NULL */
+static int nfstars;		/* number of entries in fstars[] */
+static double fsdec, fsra;	/* location when field stars were loaded */
+#define	FSFOV	degrad(1.0)	/* size of FOV to fetch, rads */
+#define	FSMAG	20.0		/* limiting mag for fetch */
+#define	FSMOVE	degrad(.2)	/* reload when jup has moved this far, rads */
+static void jm_loadfs P_((double ra, double dec));
+
 /* arrays that use NM use [0] for the planet itself, and [1..NM-1] for moons.
  */
 #define	NM	5		/* number of moons + 1 for planet */
@@ -164,11 +188,9 @@ jm_manage ()
 	    jm_create_jsform_w();
 	}
 	
-	if (XtIsManaged(jupform_w)) {
+	if (XtIsManaged(jupform_w))
 	    XtUnmanageChild (jupform_w);
-	    if ((jswasman = XtIsManaged(jsform_w)))
-		XtUnmanageChild (jsform_w);
-	} else {
+	else {
 	    XtManageChild (jupform_w);
 	    if (jswasman) {
 		XtManageChild (jsform_w);
@@ -219,7 +241,7 @@ Cursor c;
 {
 	Window win;
 
-	if (jupform_w && (win = XtWindow(jupform_w))) {
+	if (jupform_w && (win = XtWindow(jupform_w)) != 0) {
 	    Display *dsp = XtDisplay(jupform_w);
 	    if (c)
 		XDefineCursor (dsp, win, c);
@@ -227,7 +249,7 @@ Cursor c;
 		XUndefineCursor (dsp, win);
 	}
 
-	if (jsform_w && (win = XtWindow(jsform_w))) {
+	if (jsform_w && (win = XtWindow(jsform_w)) != 0) {
 	    Display *dsp = XtDisplay(jsform_w);
 	    if (c)
 		XDefineCursor (dsp, win, c);
@@ -245,13 +267,19 @@ jm_create_form_w()
 	    char *label;	/* label */
 	    int *flagp;		/* pointer to global flag, or NULL if none */
 	    Widget *wp;		/* widget to save, or NULL */
+	    char *tip;		/* widget tip */
 	} Option;
 	static Option options[] = {
-	    {"SkyBkg",	   "Sky background",	&skybkg,  &skybkg_w},
-	    {"BrightMoons","Bright moons",	&brmoons},
-	    {"Tags",	   "Tags",		&tags},
-	    {"FlipTB",	   "Flip T/B",		&flip_tb},
-	    {"FlipLR",	   "Flip L/R",		&flip_lr},
+	    {"SkyBkg",	   "Sky background",	&skybkg,  &skybkg_w,
+	    	"When on, sky will include database objects and Field Stars"},
+	    {"BrightMoons","Bright moons",	&brmoons, NULL,
+		"Display moons disproportionately bright, even if below limit"},
+	    {"Tags",	   "Tags",		&tags,    NULL,
+		"Label each moon with its Roman Numeral sequence number"},
+	    {"FlipTB",	   "Flip T/B",		&flip_tb, NULL,
+		"Flip the display top-to-bottom"},
+	    {"FlipLR",	   "Flip L/R",		&flip_lr, NULL,
+		"Flip the display left-to-right"},
 	};
 	typedef struct {
 	    char *label;	/* what goes on the help label */
@@ -276,8 +304,11 @@ jm_create_form_w()
 	XtSetArg (args[n], XmNautoUnmanage, False); n++;
 	XtSetArg (args[n], XmNdefaultPosition, False); n++;
 	XtSetArg (args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
+	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
 	jupform_w = XmCreateFormDialog (toplevel_w, "Jupiter", args, n);
+	set_something (jupform_w, XmNcolormap, (XtArgVal)xe_cm);
 	XtAddCallback (jupform_w, XmNhelpCallback, jm_help_cb, 0);
+	XtAddCallback (jupform_w, XmNunmapCallback, jm_unmap_cb, 0);
 
 	/* set some stuff in the parent DialogShell.
 	 * setting XmNdialogTitle in the Form didn't work..
@@ -307,12 +338,22 @@ jm_create_form_w()
 	    set_xmstring (cb_w, XmNlabelString, "Control");
 	    XtManageChild (cb_w);
 
+	    /* add the "Field stars" push button */
+
+	    n = 0;
+	    w = XmCreatePushButton (pd_w, "FS", args, n);
+	    set_xmstring (w, XmNlabelString, "Setup Field Stars ...");
+	    XtAddCallback (w, XmNactivateCallback, (XtCallbackProc)fs_manage,0);
+	    wtip (w, "Define where GSC and PPM catalogs are to be found");
+	    XtManageChild (w);
+
 	    /* add the "LOCFIFO" push button */
 
 	    n = 0;
 	    w = XmCreatePushButton (pd_w, "LOCFIFO", args, n);
 	    set_xmstring (w, XmNlabelString, "Send to LOCFIFO");
 	    XtAddCallback (w, XmNactivateCallback, jm_locfifo_cb, 0);
+	    wtip (w, "Open export connection");
 	    XtManageChild (w);
 
 	    /* add the "movie" push button */
@@ -321,6 +362,7 @@ jm_create_form_w()
 	    w = XmCreatePushButton (pd_w, "Movie", args, n);
 	    set_xmstring (w, XmNlabelString, "Movie Demo");
 	    XtAddCallback (w, XmNactivateCallback, jm_movie_cb, 0);
+	    wtip (w, "Start/Stop a demonstration animation sequence");
 	    XtManageChild (w);
 
 	    /* add the "close" push button beneath a separator */
@@ -332,6 +374,7 @@ jm_create_form_w()
 	    n = 0;
 	    w = XmCreatePushButton (pd_w, "Close", args, n);
 	    XtAddCallback (w, XmNactivateCallback, jm_close_cb, 0);
+	    wtip (w, "Close this and all supporting dialogs");
 	    XtManageChild (w);
 
 	/* make the View pulldown */
@@ -357,11 +400,13 @@ jm_create_form_w()
 		XtAddCallback (w, XmNvalueChangedCallback, jm_option_cb, 
 						    (XtPointer)(op->flagp));
 		set_xmstring (w, XmNlabelString, op->label);
-		XtManageChild (w);
 		if (op->flagp)
 		    *(op->flagp) = XmToggleButtonGetState(w);
 		if (op->wp)
 		    *op->wp = w;
+		if (op->tip)
+		    wtip (w, op->tip);
+		XtManageChild (w);
 	    }
 
 	    /* add a separator */
@@ -376,6 +421,7 @@ jm_create_form_w()
 	    w = XmCreatePushButton (pd_w, "Stats", args, n);
 	    set_xmstring (w, XmNlabelString, "More info...");
 	    XtAddCallback (w, XmNactivateCallback, jm_sstats_cb, NULL);
+	    wtip (w, "Display additional details");
 	    XtManageChild (w);
 
 	/* make the help pulldown */
@@ -413,6 +459,7 @@ jm_create_form_w()
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
 	dt_w = XmCreateLabel (jupform_w, "DateStamp", args, n);
 	timestamp (mm_get_now(), dt_w);	/* establishes size */
+	wtip (dt_w, "Date and Time for which map is computed");
 	XtManageChild (dt_w);
 
 	/* make the scale widget.
@@ -433,6 +480,7 @@ jm_create_form_w()
 	scale_w = XmCreateScale (jupform_w, "Scale", args, n);
 	XtAddCallback (scale_w, XmNdragCallback, jm_scale_cb, 0);
 	XtAddCallback (scale_w, XmNvalueChangedCallback, jm_scale_cb, 0);
+	wtip (scale_w, "Zoom in and out");
 	XtManageChild (scale_w);
 
 	/* make the limiting mag scale widget.
@@ -454,6 +502,7 @@ jm_create_form_w()
 	limmag_w = XmCreateScale (jupform_w, "LimMag", args, n);
 	XtAddCallback (limmag_w, XmNdragCallback, jm_scale_cb, 0);
 	XtAddCallback (limmag_w, XmNvalueChangedCallback, jm_scale_cb, 0);
+	wtip (limmag_w, "Adjust the brightness and limiting magnitude");
 	XtManageChild (limmag_w);
 
 	/* make a frame for the drawing area.
@@ -491,14 +540,21 @@ jm_create_jsform_w()
 	    char *rcname;	/* name of rowcolumn */
 	    char *collabel;	/* column label */
 	    char *suffix;	/* suffix for plot/list/search name */
+	    char *tip;		/* widget tip */
 	} MoonColumn;
 	static MoonColumn mc[] = {
-	    {CX,   "JupX",   "X (+E)",     "X"},
-	    {CY,   "JupY",   "Y (+S)",     "Y"},
-	    {CZ,   "JupZ",   "Z (+front)", "Z"},
-	    {CRA,  "JupRA",  "RA",         "RA"},
-	    {CDEC, "JupDec", "Dec",        "Dec"},
-	    {CMAG, "JupMag", "Mag",        "Mag"},
+	    {CX,   "JupX",   "X (+E)",     "X",
+		"Apparent displacement east of planetary center"},
+	    {CY,   "JupY",   "Y (+S)",     "Y",
+		"Apparent displacement south of planetary center"},
+	    {CZ,   "JupZ",   "Z (+front)", "Z",
+		"Jupiter radii towards Earth from planetary center"},
+	    {CRA,  "JupRA",  "RA",         "RA",
+		"Right Ascension (to Main's settings)"},
+	    {CDEC, "JupDec", "Dec",        "Dec",
+		"Declination (to Main's settings)"},
+	    {CMAG, "JupMag", "Mag",        "Mag",
+		"Apparent visual magnitude"},
 	};
 	Widget w;
 	Widget rc_w, title_w, sep_w;
@@ -511,7 +567,9 @@ jm_create_jsform_w()
 	XtSetArg (args[n], XmNautoUnmanage, False); n++;
 	XtSetArg (args[n], XmNresizePolicy, XmRESIZE_ANY); n++;
 	XtSetArg (args[n], XmNverticalSpacing, 5); n++;
+	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
 	jsform_w = XmCreateFormDialog (toplevel_w, "JupiterStats", args, n);
+	set_something (jsform_w, XmNcolormap, (XtArgVal)xe_cm);
 
 	/* set some stuff in the parent DialogShell.
 	 * setting XmNdialogTitle in the Form didn't work..
@@ -552,6 +610,7 @@ jm_create_jsform_w()
 	    XtSetArg (args[n], XmNuserData, "Jupiter.CMLI"); n++;
 	    cmlI_w = XmCreatePushButton (rc_w, "JupCMLI", args, n);
 	    XtAddCallback (cmlI_w, XmNactivateCallback, jm_activate_cb, 0);
+	    wtip (cmlI_w, "Longitude currently facing Earth, in system I coordinates");
 	    XtManageChild (cmlI_w);
 
 	/* make the Sys II r/c */
@@ -575,6 +634,7 @@ jm_create_jsform_w()
 	    XtSetArg (args[n], XmNuserData, "Jupiter.CMLII"); n++;
 	    cmlII_w = XmCreatePushButton (rc_w, "JupCMLII", args, n);
 	    XtAddCallback (cmlII_w, XmNactivateCallback, jm_activate_cb, 0);
+	    wtip (cmlII_w, "Longitude currently facing Earth, in system II coordinates");
 	    XtManageChild (cmlII_w);
 
 	/* make table title label */
@@ -604,8 +664,9 @@ jm_create_jsform_w()
 
 	    n = 0;
 	    w = XmCreateLabel (rc_w, "TL", args, n);
-	    XtManageChild (w);
 	    set_xmstring (w, XmNlabelString, "Tag");
+	    wtip (w, "Roman Numeral sequence designation of moon");
+	    XtManageChild (w);
 
 	    for (i = 0; i < NM; i++) {
 		char *tag = mnames[i].tag;
@@ -631,8 +692,9 @@ jm_create_jsform_w()
 
 	    n = 0;
 	    w = XmCreateLabel (rc_w, "NL", args, n);
-	    XtManageChild (w);
 	    set_xmstring (w, XmNlabelString, "Name");
+	    wtip (w, "Common name of body");
+	    XtManageChild (w);
 
 	    for (i = 0; i < NM; i++) {
 		n = 0;
@@ -660,8 +722,10 @@ jm_create_jsform_w()
 
 	    n = 0;
 	    w = XmCreateLabel (rc_w, "JupLab", args, n);
-	    XtManageChild (w);
 	    set_xmstring (w, XmNlabelString, mp->collabel);
+	    if (mp->tip)
+		wtip (w, mp->tip);
+	    XtManageChild (w);
 
 	    for (j = 0; j < NM; j++) {
 		char *sel;
@@ -696,6 +760,7 @@ jm_create_jsform_w()
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
 	sdt_w = XmCreateLabel (jsform_w, "JDateStamp", args, n);
+	wtip (sdt_w, "Date and Time for which data are computed");
 	XtManageChild (sdt_w);
 
 	/* make a separator */
@@ -714,12 +779,14 @@ jm_create_jsform_w()
 	XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
 	XtSetArg (args[n], XmNtopWidget, sep_w); n++;
 	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
+        XtSetArg (args[n], XmNbottomOffset, 5); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
 	XtSetArg (args[n], XmNleftPosition, 30); n++;
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
 	XtSetArg (args[n], XmNrightPosition, 70); n++;
 	w = XmCreatePushButton (jsform_w, "Close", args, n);
 	XtAddCallback (w, XmNactivateCallback, jm_sstats_cb, 0);
+	wtip (w, "Close this dialog");
 	XtManageChild (w);
 }
 
@@ -822,7 +889,7 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
-	if ((jswasman = XtIsManaged(jsform_w)))
+	if ((jswasman = XtIsManaged(jsform_w)) != 0)
 	    XtUnmanageChild (jsform_w);
 	else {
 	    XtManageChild (jsform_w);
@@ -876,6 +943,32 @@ XtPointer call;
 	}
 }
 
+/* callback when main dialog is unmapped */
+/* ARGSUSED */
+static void
+jm_unmap_cb (w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
+{
+	if ((jswasman = (jsform_w && XtIsManaged(jsform_w))) != 0)
+	    XtUnmanageChild (jsform_w);
+        if (jm_pm) {
+	    XFreePixmap (XtDisplay(jda_w), jm_pm);
+	    jm_pm = (Pixmap) NULL;
+	}
+
+	/* free any field stars */
+	if (fstars) {
+	    free ((void *)fstars);
+	    fstars = NULL;
+	    nfstars = 0;
+	}
+
+	/* stop movie that might be running */
+	mm_movie (0.0);
+}
+
 /* callback from the main Close button */
 /* ARGSUSED */
 static void
@@ -884,12 +977,8 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
+	/* let unmap do all the work */
 	XtUnmanageChild (jupform_w);
-	if ((jswasman = XtIsManaged(jsform_w)))
-	    XtUnmanageChild (jsform_w);
-
-	/* stop movie that might be running */
-	mm_movie (0.0);
 }
 
 /* callback from the Movie button
@@ -917,7 +1006,13 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
+	static last_nx, last_ny;
 	XmDrawingAreaCallbackStruct *c = (XmDrawingAreaCallbackStruct *)call;
+	Window win = XtWindow(w);
+	Display *dsp = XtDisplay(w);
+	unsigned int nx, ny, bw, d;
+	Window root;
+	int x, y;
 
 	/* filter out a few oddball cases */
 	switch (c->reason) {
@@ -930,11 +1025,14 @@ XtPointer call;
 
 	    if (!before) {
 		XSetWindowAttributes swa;
+		unsigned long mask = CWBitGravity | CWBackingStore;
+
 		swa.bit_gravity = ForgetGravity;
-		XChangeWindowAttributes (e->display, e->window, 
-							    CWBitGravity, &swa);
+		swa.backing_store = NotUseful; /* we use a pixmap */
+		XChangeWindowAttributes (dsp, win, mask, &swa);
 		before = 1;
 	    }
+
 	    /* wait for the last in the series */
 	    if (e->count != 0)
 		return;
@@ -945,7 +1043,14 @@ XtPointer call;
 	    exit(1);
 	}
 
-	jm_update (mm_get_now(), 1);
+	XGetGeometry(dsp, win, &root, &x, &y, &nx, &ny, &bw, &d);
+	if (!jm_pm || nx != last_nx || ny != last_ny) {
+	    jm_pm = XCreatePixmap (dsp, win, nx, ny, d);
+	    last_nx = nx;
+	    last_ny = ny;
+	    jm_update (mm_get_now(), 1);
+	} else
+	    XCopyArea (dsp, jm_pm, win, j_fgc, 0, 0, nx, ny, 0, 0);
 }
 
 /* callback from mouse or keyboard input over drawing area */
@@ -1024,7 +1129,7 @@ ScreenObj *sop;
 	char buf[64], buf2[64];
 
 	name = sop->o.o_name;
-	mag = sop->o.s_mag/MAGSCALE;
+	mag = get_mag(&sop->o);
 	ra = sop->o.s_ra;
 	dec = sop->o.s_dec;
 
@@ -1035,7 +1140,7 @@ ScreenObj *sop;
 	(void) sprintf (buf2, "  RA: %s", buf);
 	set_xmstring (jmpu_ra_w, XmNlabelString, buf2);
 
-	fs_pangle (buf, dec);
+	fs_prdec (buf, dec);
 	(void) sprintf (buf2, " Dec: %s", buf);
 	set_xmstring (jmpu_dec_w, XmNlabelString, buf2);
 
@@ -1140,7 +1245,7 @@ close_screenobj (x, y)
 int x, y;
 {
 	int minr;
-	ScreenObj *sop, *minsop;
+	ScreenObj *sop, *minsop = NULL;
 
 	minr = 2*MINR;
 	for (sop = &screenobj[nscreenobj]; --sop >= screenobj; ) {
@@ -1166,6 +1271,7 @@ typedef struct {
 static void jupiter_data();
 static void moonradec();
 static void jm_draw_map ();
+static void make_gcs ();
 
 /* called to recompute and fill in values for the jupiter menu.
  * don't bother if it doesn't exist or is unmanaged now or no one is logging.
@@ -1215,7 +1321,7 @@ int how_much;
 		f_double (j_w[i][CZ], fmt, md[i].z);
 		f_double (j_w[i][CMAG], "%4.1f", md[i].mag);
 		f_ra (j_w[i][CRA], md[i].ra);
-		f_pangle (j_w[i][CDEC], md[i].dec);
+		f_prdec (j_w[i][CDEC], md[i].dec);
 	    }
 
 	    f_double (cmlI_w, fmt, raddeg(cmlI));
@@ -1248,11 +1354,6 @@ double salpha, calpha;
 double cmlII;
 MoonData md[NM];
 {
-	static GC j_fgc, j_bgc, j_xgc;
-	static XFontStruct *s_fs;
-	static last_nx, last_ny;
-	static int cw, ch;
-	static Pixmap pm;
 	Display *dsp = XtDisplay(w);
 	Window win = XtWindow(w);
 	Window root;
@@ -1271,66 +1372,44 @@ MoonData md[NM];
 #define	XCORD(x)	((int)(((int)nx)/2.0 + ew*MAPSCALE(x) + 0.5))
 #define	YCORD(y)	((int)(((int)ny)/2.0 - ns*MAPSCALE(y) + 0.5))
 
-	if (!j_fgc) {
-	    XGCValues gcv;
-	    unsigned int gcm;
-	    Pixel fg, bg;
-
-	    gcm = GCForeground;
-	    get_something (w, XmNforeground, (XtArgVal)&fg);
-	    gcv.foreground = fg;
-	    j_fgc = XCreateGC (dsp, win, gcm, &gcv);
-	    s_fs = XQueryFont (dsp, XGContextFromGC (j_fgc));
-	    cw = s_fs->max_bounds.width;
-	    ch = s_fs->max_bounds.ascent + s_fs->max_bounds.descent;
-
-	    gcm = GCForeground;
-	    get_something (w, XmNbackground, (XtArgVal)&bg);
-	    gcv.foreground = bg;
-	    j_bgc = XCreateGC (dsp, win, gcm, &gcv);
-
-	    gcm = GCForeground | GCFunction;
-	    gcv.foreground = fg ^ bg;
-	    gcv.function = GXxor;
-	    j_xgc = XCreateGC (dsp, win, gcm, &gcv);
-	}
+	if (!j_fgc)
+	    make_gcs(w);
 
 	XmScaleGetValue (limmag_w, &fmag);
 	XmScaleGetValue (scale_w, &sv);
 	scale = pow(MAXSCALE, sv/100.0);
 
 	XGetGeometry(dsp, win, &root, &x, &y, &nx, &ny, &bw, &d);
-	if (!pm || nx != last_nx || ny != last_ny) {
-	    if (pm)
-		XFreePixmap (dsp, pm);
-	    pm = XCreatePixmap (dsp, win, nx, ny, d);
-	    last_nx = nx;
-	    last_ny = ny;
-	}
 
-	XFillRectangle (dsp, pm, j_bgc, 0, 0, nx, ny);
+	XFillRectangle (dsp, jm_pm, j_bgc, 0, 0, nx, ny);
 
 	/* prepare for a new list of things on the screen */
 	reset_screenobj();
 
 	/* draw background objects, if desired */
 	if (skybkg)
-	    sky_background(pm, nx, ny, fmag, md[0].ra, md[0].dec, salpha,calpha,
-			jupsize/MAPSCALE(2), jupsize*NORM, flip_tb, flip_lr);
+	    sky_background(jm_pm, nx, ny, fmag, md[0].ra, md[0].dec, salpha,
+		calpha, jupsize/MAPSCALE(2), jupsize*NORM, flip_tb, flip_lr);
 
 	/* draw labels */
 	c = flip_lr ? 'W' : 'E';
-	XDrawString(dsp, pm, j_fgc, nx-cw-1, ny/2-2, &c, 1);
+	XDrawString(dsp, jm_pm, j_fgc, nx-j_cw-1, ny/2-2, &c, 1);
 	c = flip_tb ? 'N' : 'S';
-	XDrawString(dsp, pm, j_fgc, (nx-cw)/2-1, s_fs->ascent, &c, 1);
+	XDrawString(dsp, jm_pm, j_fgc, (nx-j_cw)/2-1, j_fs->ascent, &c, 1);
 
 	/* draw Jupiter in center with unit radius */
-	XFillArc (dsp, pm, j_fgc, nx/2-(int)MAPSCALE(1), ny/2-(int)MAPSCALE(1),
-			    (int)MAPSCALE(2), (int)MAPSCALE(2), 0, 360*64);
+	XPSFillArc(dsp,jm_pm, j_fgc, nx/2-(int)MAPSCALE(1),
+					ny/2-(int)MAPSCALE(1), (int)MAPSCALE(2),
+					(int)MAPSCALE(2), 0, 360*64);
 	add_screenobj (db_basic(JUPITER), nx/2, ny/2);
 
 	/* draw the GRS */
-	jgrs_draw (pm, (int)MAPSCALE(1), nx/2, ny/2, flip_tb, flip_lr, cmlII);
+	jgrs_draw (jm_pm, (int)MAPSCALE(1), nx/2, ny/2, flip_tb,flip_lr,cmlII);
+
+	/* draw 1' calibration line */
+	if (tags)
+	    j_calibline (dsp, jm_pm, j_fgc, nx/2, 5*ny/6, "1'", j_cw*2, j_ch+4,
+				(int)MAPSCALE(degrad(1.0/60.0)/(jupsize/2)));
 
 	/* draw each moon
 	 */
@@ -1346,7 +1425,7 @@ MoonData md[NM];
 
 	    if (!outside && !infront)
 		continue;	/* behind jupiter */
-	    diam = magdiam (fmag, jupsize/MAPSCALE(2), mag, 0.0);
+	    diam = magdiam (fmag, 1, jupsize/MAPSCALE(2), mag, 0.0);
 	    if (brmoons)
 		diam += 3;
 	    if (diam <= 0)
@@ -1355,28 +1434,80 @@ MoonData md[NM];
 	    x = XCORD(mx);
 	    y = YCORD(my);
 	    if (diam == 1)
-		XDrawPoint(dsp, pm, j_xgc, x, y);
+		XDrawPoint(dsp, jm_pm, j_xgc, x, y);
 	    else
-		XFillArc (dsp, pm, j_xgc, x-diam/2, y-diam/2, diam, diam,
+		XPSFillArc (dsp, jm_pm, j_xgc, x-diam/2, y-diam/2, diam, diam,
 								    0, 360*64);
 
 	    /* add object to list of screen objects drawn */
 	    (void) strcpy (o.o_name, mnames[i].full);
-	    o.s_mag = mag*MAGSCALE;
+	    set_smag (&o, mag);
 	    o.s_ra = md[i].ra;
 	    o.s_dec = md[i].dec;
 	    add_screenobj (&o, x, y);
 
 	    if (tags && mnames[i].tag)
-		XDrawString(dsp, pm, j_xgc, x-cw/2, y+2*ch,
+		XDrawString(dsp, jm_pm, j_xgc, x-j_cw/2, y+2*j_ch,
 					mnames[i].tag, strlen(mnames[i].tag));
 	}
 
-	XCopyArea (dsp, pm, win, j_fgc, 0, 0, nx, ny, 0, 0);
+	XCopyArea (dsp, jm_pm, win, j_fgc, 0, 0, nx, ny, 0, 0);
+}
+
+static void
+make_gcs (w)
+Widget w;
+{
+	Display *dsp = XtDisplay(w);
+	Window win = XtWindow(w);
+	XGCValues gcv;
+	unsigned int gcm;
+	Pixel fg, bg;
+
+	gcm = GCForeground;
+	get_something (w, XmNforeground, (XtArgVal)&fg);
+	gcv.foreground = fg;
+	j_fgc = XCreateGC (dsp, win, gcm, &gcv);
+	j_fs = XQueryFont (dsp, XGContextFromGC (j_fgc));
+	j_cw = j_fs->max_bounds.width;
+	j_ch = j_fs->max_bounds.ascent + j_fs->max_bounds.descent;
+
+	gcm = GCForeground;
+	get_something (w, XmNbackground, (XtArgVal)&bg);
+	gcv.foreground = bg;
+	j_bgc = XCreateGC (dsp, win, gcm, &gcv);
+
+	gcm = GCForeground | GCFunction;
+	gcv.foreground = fg ^ bg;
+	gcv.function = GXxor;
+	j_xgc = XCreateGC (dsp, win, gcm, &gcv);
+}
+
+/* draw a calibration line l pixels long centered at [xc,yc] marked with tag
+ * with is in a bounding box tw x th.
+ */
+static void
+j_calibline (dsp, win, gc, xc, yc, tag, tw, th, l)
+Display *dsp;
+Drawable win;
+GC gc;
+int xc, yc;
+char *tag;
+int tw, th;
+int l;
+{
+	int lx = xc - l/2;
+	int rx = lx + l;
+
+	XDrawLine (dsp, win, gc, lx, yc, rx, yc);
+	XDrawLine (dsp, win, gc, lx, yc-3, lx, yc+3);
+	XDrawLine (dsp, win, gc, rx, yc-3, rx, yc+3);
+	XDrawString (dsp, win, gc, xc-tw/2, yc+th, tag, strlen(tag));
 }
 
 /* draw all database objects in a small sky patch with the given orientation.
- * also save objects and screen locs in the global screenobj array.
+ * get field stars too if enabled and we have moved enough.
+ * save objects and screen locs in the global screenobj array for picking.
  * this is used to draw the backgrounds for the planet closeups.
  * Based on work by: Dan Bruton <WDB3926@acs.tamu.edu>
  */
@@ -1391,20 +1522,27 @@ double scale;		/* rads per pixel */
 double rad;		/* maximum radius to draw away from ra0/dec0, rads */
 int fliptb, fliplr;	/* flip direction; default is S up E right */
 {
+	static before;
 	double cdec0 = cos(dec0);
 	DBScan dbs;
 	Obj *op;
 
+	if (!before && pref_get(PREF_EQUATORIAL) == PREF_GEO) {
+	    xe_msg ("Equatorial preference should probably be set to Topocentric", 1);
+	    before = 1;
+	}
+
+        /* update field star list */
+	jm_loadfs (ra0, dec0);
+
 	/* scan the database and draw whatever is near */
-	for (db_scaninit(&dbs); (op = db_scan (&dbs)) != NULL; ) {
+	for (db_scaninit(&dbs, ALLM, fstars, nfstars);
+					    (op = db_scan (&dbs)) != NULL; ) {
 	    double dra, ddec;
 	    GC gc;
 	    int dx, dy, x, y;
 	    int diam;
 
-	    if (op->type == UNDEFOBJ)
-		continue;
-
 	    if (is_planet (op, JUPITER)) {
 		/* we draw it elsewhere */
 		continue;
@@ -1413,7 +1551,7 @@ int fliptb, fliplr;	/* flip direction; default is S up E right */
 	    db_update (op);
 
 	    /* find size, in pixels. */
-	    diam = magdiam (fmag, scale, op->s_mag/MAGSCALE,
+	    diam = magdiam (fmag, 1, scale, get_mag(op),
 					    degrad((double)op->s_size/3600.0));
 	    if (diam <= 0)
 		continue;
@@ -1421,7 +1559,7 @@ int fliptb, fliplr;	/* flip direction; default is S up E right */
 	    /* find x/y location if it's in the general area */
 	    dra = op->s_ra - ra0;	/* + E */
 	    ddec = dec0 - op->s_dec;	/* + S */
-	    if (fabs(ddec) > rad || (PI-fabs(dra-PI))*cdec0 > rad)
+	    if (fabs(ddec) > rad || delra(dra)*cdec0 > rad)
 		continue;
 	    dx = (dra*calpha - ddec*salpha)/scale;
 	    dy = (dra*salpha + ddec*calpha)/scale;
@@ -1441,6 +1579,35 @@ int fliptb, fliplr;	/* flip direction; default is S up E right */
 	sv_draw_obj (XtD, win, (GC)0, NULL, 0, 0, 0, 0);	/* flush */
 }
 
+/* load field stars around the given location, unless current set is
+ * already close enough.
+ */
+static void
+jm_loadfs (ra, dec)
+double ra, dec;
+{
+	Now *np = mm_get_now();
+
+	if (fstars && fabs(dec-fsdec)<FSMOVE && cos(dec)*delra(ra-fsra)<FSMOVE)
+	    return;
+
+	if (fstars) {
+	    free ((void *)fstars);
+	    fstars = NULL;
+	    nfstars = 0;
+	}
+
+        nfstars = fs_fetch (np, ra, dec, FSFOV, FSMAG, &fstars);
+
+	if (nfstars > 0) {
+	    char msg[128];
+            (void) sprintf (msg, "Jupiter View added %d field stars", nfstars);
+	    xe_msg (msg, 0);
+	    fsdec = dec;
+	    fsra = ra;
+	}
+}
+
 /* given jupiter loc in md[0].ra/dec and size, and location of each moon in 
  * md[1..NM-1].x/y in jup radii, find ra/dec of each moon in md[1..NM-1].ra/dec.
  */
@@ -1573,7 +1740,7 @@ MoonData md[NM];	/* fill in md[0].ra/dec/mag/x/y/z for jupiter,
 	    md[0].x = 0;
 	    md[0].y = 0;
 	    md[0].z = 0;
-	    md[0].mag = op->s_mag/MAGSCALE;
+	    md[0].mag = get_mag(op);
 
 	    md[0].ra = op->s_ra;
 	    md[0].dec = op->s_dec;
