@@ -23,6 +23,7 @@
 #include "P_.h"
 
 extern Widget toplevel_w;
+extern Colormap xe_cm;
 
 extern FILE *fopenh P_((char *name, char *how));
 extern char *syserrstr P_((void));
@@ -32,8 +33,11 @@ extern void all_selection_mode P_((int whether));
 extern void f_string P_((Widget w, char *s));
 extern void hlp_dialog P_((char *tag, char *deflt[], int ndeflt));
 extern void query P_((Widget tw, char *msg, char *label1, char *label2,
-    char *label3, void (*func1)(), void (*func2)(), void (*func3)()));
+    char *label3, void (*func1)(void), void (*func2)(void),
+    void (*func3)(void)));
+extern void set_something P_((Widget w, char *resource, XtArgVal value));
 extern void set_xmstring P_((Widget w, char *resource, char *txt));
+extern void wtip P_((Widget w, char *tip));
 extern void xe_msg P_((char *msg, int app_modal));
 
 static void lst_select P_((int whether));
@@ -49,6 +53,9 @@ static void lst_try_overwrite P_((void));
 static void lst_try_cancel P_((void));
 static void lst_try_turn_on P_((void));
 static void lst_turn_on P_((char *how));
+static void lst_hdr P_((void));
+
+#define	COMMENT	'*'		/* comment character */
 
 /* max number of fields we can keep track of at once to list */
 #define MAXLSTFLDS	10
@@ -56,17 +63,19 @@ static void lst_turn_on P_((char *how));
 #define MAXFLDNAM	32	/* longest allowed field name */
 
 static Widget lstform_w;
-static Widget select_w, active_w, prompt_w;
+static Widget select_w, active_w, prompt_w, colhdr_w;
 static Widget title_w, filename_w;
 static Widget table_w[MAXLSTFLDS];	/* row indeces follow.. */
 
 #define	DEF_LSTFN	"ephem.lst"	/* default plot file name */
 static FILE *lst_fp;            /* the listing file; == 0 means don't plot */
+static int lst_new;		/* 1 when open until first set are printed */
 
 
 /* lst_activate_cb client values. */
-#define	SELECT	0
-#define	ACTIVE	1
+typedef enum {
+    SELECT, ACTIVE, COLHDR
+} Options;
 
 /* store the name and string value of each field to track.
  * we get the label straight from the Text widget in the table as needed.
@@ -74,6 +83,7 @@ static FILE *lst_fp;            /* the listing file; == 0 means don't plot */
 typedef struct {
     char l_name[MAXFLDNAM];	/* name of field we are listing */
     char l_str[MAXLSTSTR];	/* last know string value of field */
+    int l_w;			/* column width -- 0 until known */
 } LstFld;
 static LstFld lstflds[MAXLSTFLDS];
 static int nlstflds;		/* number of lstflds[] in actual use */
@@ -145,9 +155,18 @@ listing()
 	if (lst_fp) {
 	    /* list in order of original selection */
 	    LstFld *lp;
+
+	    /* print headings if this the first time */
+	    if (lst_new) {
+		lst_hdr();
+		lst_new = 0;
+	    }
+
+	    /* now print the fields */
 	    for (lp = lstflds; lp < &lstflds[nlstflds]; lp++)
-		(void) fprintf (lst_fp, "%s  ", lp->l_str);
+		(void) fprintf (lst_fp, "  %-*s", lp->l_w, lp->l_str);
 	    (void) fprintf (lst_fp, "\n");
+	    fflush (lst_fp);	/* to allow monitoring */
 	}
 }
 
@@ -164,7 +183,7 @@ Cursor c;
 {
 	Window win;
 
-	if (lstform_w && (win = XtWindow(lstform_w))) {
+	if (lstform_w && (win = XtWindow(lstform_w)) != 0) {
 	    Display *dsp = XtDisplay(lstform_w);
 	    if (c)
 		XDefineCursor (dsp, win, c);
@@ -190,9 +209,14 @@ lst_create_form()
 	    char *title;
 	    int cb_data;
 	    Widget *wp;
+	    char *tip;
 	} tbs[] = {
-	    {"Select fields", SELECT, &select_w},
-	    {"List to file", ACTIVE, &active_w}
+	    {"Select fields", SELECT, &select_w,
+		"When on, data fields eligible for listing are selectable buttons"},
+	    {"List to file", ACTIVE, &active_w,
+		"When on, selected fields are written to the named file at each main Update"},
+	    {"Column Headings", COLHDR, &colhdr_w,
+		"Whether file format will include a heading over each column"},
 	};
 	XmString str;
 	Widget w, rc_w, f_w;
@@ -204,7 +228,9 @@ lst_create_form()
 	n = 0;
 	XtSetArg (args[n], XmNautoUnmanage, False); n++;
 	XtSetArg (args[n], XmNdefaultPosition, False); n++;
+	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
 	lstform_w = XmCreateFormDialog (toplevel_w, "List", args, n);
+	set_something (lstform_w, XmNcolormap, (XtArgVal)xe_cm);
 	XtAddCallback (lstform_w, XmNhelpCallback, lst_help_cb, 0);
 
 	n = 0;
@@ -215,11 +241,16 @@ lst_create_form()
 
 	n = 0;
 	XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
+	XtSetArg (args[n], XmNtopOffset, 10); n++;
 	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
+	XtSetArg (args[n], XmNbottomOffset, 10); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
+	XtSetArg (args[n], XmNleftOffset, 10); n++;
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
+	XtSetArg (args[n], XmNrightOffset, 10); n++;
 	XtSetArg (args[n], XmNisAligned, False); n++;
 	XtSetArg (args[n], XmNadjustMargin, False); n++;
+	XtSetArg (args[n], XmNspacing, 5); n++;
 	rc_w = XmCreateRowColumn (lstform_w, "ListRC", args, n);
 	XtManageChild (rc_w);
 
@@ -232,11 +263,13 @@ lst_create_form()
 	    XtSetArg (args[n], XmNlabelString, str); n++;
 	    w = XmCreateToggleButton(rc_w, "ListTB", args, n);
 	    XmStringFree (str);
-	    XtManageChild (w);
 	    XtAddCallback(w, XmNvalueChangedCallback, lst_activate_cb,
 						    (XtPointer)tbs[i].cb_data);
 	    if (tbs[i].wp)
 		*tbs[i].wp = w;
+	    if (tbs[i].tip)
+		wtip (w, tbs[i].tip);
+	    XtManageChild (w);
 	}
 
 	/* create filename text area and its label */
@@ -255,6 +288,7 @@ lst_create_form()
 	if (strlen(deffn) == 0)
 	    XmTextSetString (filename_w, DEF_LSTFN);
 	XtFree (deffn);
+	wtip (filename_w, "Enter name of file to write");
 	XtManageChild (filename_w);
 
 	/* create title text area and its label */
@@ -269,6 +303,7 @@ lst_create_form()
 
 	n = 0;
 	title_w = XmCreateText (rc_w, "ListTitle", args, n);
+	wtip (title_w, "Enter a title to be written to the file");
 	XtManageChild (title_w);
 
 	/* create prompt line -- it will be managed as necessary */
@@ -304,6 +339,7 @@ lst_create_form()
 	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
 	    XtSetArg (args[n], XmNrightPosition, 3); n++;
 	    w = XmCreatePushButton (f_w, "Close", args, n);
+	    wtip (w, "Close this dialog (but continue listing if active)");
 	    XtAddCallback (w, XmNactivateCallback, lst_close_cb, 0);
 	    XtManageChild (w);
 
@@ -315,6 +351,7 @@ lst_create_form()
 	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
 	    XtSetArg (args[n], XmNrightPosition, 6); n++;
 	    w = XmCreatePushButton (f_w, "Help", args, n);
+	    wtip (w, "More detailed usage information");
 	    XtAddCallback (w, XmNactivateCallback, lst_help_cb, 0);
 	    XtManageChild (w);
 }
@@ -329,9 +366,9 @@ XtPointer client;
 XtPointer call;
 {
 	XmToggleButtonCallbackStruct *t = (XmToggleButtonCallbackStruct *) call;
-	int what = (int) client;
+	Options op = (Options) client;
 
-	switch (what) {
+	switch (op) {
 	case SELECT:
 	    if (t->set) {
 		/* first turn off listing, if on, while we change things */
@@ -344,6 +381,7 @@ XtPointer call;
 	    } else
 		lst_stop_selecting();
 	    break;
+
 	case ACTIVE:
 	    if (t->set) {
 		/* first turn off selecting, if on */
@@ -353,6 +391,9 @@ XtPointer call;
 	    } else
 		lst_turn_off();
 	    break;
+
+	case COLHDR:
+	    break;	/* toggle state is sufficient */
 	}
 }
 
@@ -479,17 +520,86 @@ char *how;	/* fopen how argument */
 	    char *buf;
 	    XmToggleButtonSetState (active_w, False, False);
 	    buf = XtMalloc (strlen(txt)+100);
-	    (void) sprintf (buf, "Can not open %s: %s", txt, syserrstr());
+	    (void) sprintf (buf, "%s: %s", txt, syserrstr());
 	    xe_msg (buf, 1);
 	    XtFree (buf);
 	}
 	XtFree (txt);
-	
-	if (lst_fp) {
-	    /* add a title if desired */
-	    txt = XmTextGetString (title_w);
-	    if (txt[0] != '\0')
-		(void) fprintf (lst_fp, "* %s\n", txt);
-	    XtFree (txt);
+
+	lst_new = 1;	/* trigger fresh column headings */
+	/* TODO: not when appending? */
+}
+
+/* print the title.
+ * then set each l_w. if column headings are enabled, use and also print them.
+ *   else just use l_str.
+ */
+static void
+lst_hdr ()
+{
+	LstFld *lp;
+	int col;
+	char *txt;
+
+	/* add a title if desired */
+	txt = XmTextGetString (title_w);
+	if (txt[0] != '\0')
+	    (void) fprintf (lst_fp, "%c %s\n", COMMENT, txt);
+	XtFree (txt);
+
+	col = XmToggleButtonGetState (colhdr_w);
+
+	/* set lp->l_w to max of str, prefix and suffix lengths */
+	for (lp = lstflds; lp < &lstflds[nlstflds]; lp++) {
+	    int l = strlen (lp->l_str);
+	    if (col) {
+		int nl = strlen(lp->l_name);
+		char *dp;
+
+		for (dp = lp->l_name; *dp && *dp != '.'; dp++)
+		    continue;
+		if (*dp) {
+		    int pl = dp - lp->l_name;	/* prefix */
+		    int sl = nl - pl - 1; 	/* suffix */
+		    if (pl > l) l = pl;
+		    if (sl > l) l = sl;
+		} else {
+		    if (nl > l) l = nl;
+		}
+	    }
+	    lp->l_w = l;
 	}
+
+	if (col) {
+	    /* print first row of column headings */
+	    for (lp = lstflds; lp < &lstflds[nlstflds]; lp++) {
+		char cmt = lp == lstflds ? COMMENT : ' ';
+		char *dp;
+
+		for (dp = lp->l_name; *dp && *dp != '.'; dp++)
+		    continue;
+		if (*dp)
+		    fprintf (lst_fp, "%c %-*.*s", cmt, lp->l_w,
+						    dp-lp->l_name, lp->l_name);
+		else
+		    fprintf (lst_fp, "%c %-*s", cmt, lp->l_w, lp->l_name);
+	    }
+	    fprintf (lst_fp, "\n");
+
+	    /* print second row of column headings */
+	    for (lp = lstflds; lp < &lstflds[nlstflds]; lp++) {
+		char cmt = lp == lstflds ? COMMENT : ' ';
+		char *dp;
+
+		for (dp = lp->l_name; *dp && *dp != '.'; dp++)
+		    continue;
+		if (*dp)
+		    fprintf (lst_fp, "%c %-*s", cmt, lp->l_w, dp+1);
+		else
+		    fprintf (lst_fp, "%c %-*s", cmt, lp->l_w, "");
+	    }
+	    fprintf (lst_fp, "\n");
+	}
+
+	fflush (lst_fp); /* to allow monitoring */
 }
