@@ -3,7 +3,7 @@
  */
 
 typedef struct {
-    short lg, lt;	/* longitude and latitude (*100 for earth) */
+    short lg, lt;	/* longitude and latitude, degs (*100 for earth) */
 } MCoord;
 
 typedef struct {
