@@ -12,6 +12,7 @@ extern FILE *fdopen();
 
 #if defined(__STDC__)
 #include <stdlib.h>
+#include <string.h>
 #else
 extern char *getenv();
 extern void *malloc();
@@ -45,10 +46,10 @@ extern int close();
 #endif
 
 extern Widget	toplevel_w;
-extern char	*myclass;
 #define	XtD	XtDisplay(toplevel_w)
 
 extern Now *mm_get_now P_((void));
+extern char *getXRes P_((char *name));
 extern int any_ison P_((void));
 extern int listing_ison P_((void));
 extern int plot_ison P_((void));
@@ -67,18 +68,21 @@ extern void e_cursor P_((Cursor c));
 extern void e_newobj P_((int dbidx));
 extern void e_selection_mode P_((int whether));
 extern void e_update P_((Now *np, int force));
+extern void fs_cursor P_((Cursor c));
 extern void fs_date P_((char out[], double jd));
 extern void fs_time P_((char out[], double t));
 extern void fs_tz P_((char *timezonename, int tzpref, Now *np));
 extern void get_something P_((Widget w, char *resource, XtArgVal value));
 extern void get_xmstring P_((Widget w, char *resource, char **txtp));
 extern void jm_cursor P_((Cursor c));
+extern void jm_newdb P_((int appended));
 extern void jm_selection_mode P_((int whether));
 extern void jm_update P_((Now *np, int how_much));
 extern void lst_cursor P_((Cursor c));
 extern void lst_log P_((char *name, char *str));
 extern void lst_selection P_((char *name));
 extern void m_cursor P_((Cursor c));
+extern void m_newdb P_((int appended));
 extern void m_selection_mode P_((int whether));
 extern void m_update P_((Now *np, int how_much));
 extern void main_cursor P_((Cursor c));
@@ -89,13 +93,14 @@ extern void mm_selection_mode P_((int whether));
 extern void msg_cursor P_((Cursor c));
 extern void obj_cursor P_((Cursor c));
 extern void obj_newdb P_((int appended));
-extern void obj_pickgc P_((Obj *op, Widget w, GC *gcp));
 extern void obj_update P_((Now *np, int howmuch));
 extern void plt_cursor P_((Cursor c));
 extern void plt_log P_((char *name, double value));
 extern void plt_selection P_((char *name));
 extern void pm_cursor P_((Cursor c));
+extern void XPS_cursor P_((Cursor c));
 extern void set_xmstring P_((Widget w, char *resource, char *txt));
+extern void sf_cursor P_((Cursor c));
 extern void sm_cursor P_((Cursor c));
 extern void sm_newdb P_((int appended));
 extern void sm_selection_mode P_((int whether));
@@ -105,6 +110,7 @@ extern void srch_log P_((char *name, double value));
 extern void srch_selection P_((char *name));
 extern void srch_selection_mode P_((int whether));
 extern void ss_cursor P_((Cursor c));
+extern void ss_newdb P_((int appended));
 extern void ss_newobj P_((int dbidx));
 extern void ss_update P_((Now *np, int how_much));
 extern void sv_cursor P_((Cursor c));
@@ -155,6 +161,7 @@ int want;
 	db_cursor(c);
 	dm_cursor(c);
 	e_cursor(c);
+	fs_cursor(c);
 	jm_cursor(c);
 	lst_cursor(c);
 	m_cursor(c);
@@ -164,6 +171,8 @@ int want;
 	obj_cursor(c);
 	plt_cursor(c);
 	pm_cursor(c);
+	XPS_cursor(c);
+	sf_cursor(c);
 	sm_cursor(c);
 	srch_cursor(c);
 	ss_cursor(c);
@@ -172,6 +181,7 @@ int want;
 	v_cursor(c);
  
 	XFlush (XtD);
+	XmUpdateDisplay (toplevel_w);
 }
 
 /* update stuff on all major views */
@@ -222,8 +232,11 @@ int appended;
 
 	obj_newdb(appended);
 	sm_newdb(appended);
+	jm_newdb(appended);
+	ss_newdb(appended);
 	sv_newdb(appended);
 	db_newdb (appended);
+	m_newdb (appended);
 
 	watch_cursor (0);
 }
@@ -348,12 +361,12 @@ Obj *op;
 	    {'Q', "Quasar"},
 	    {'R', "Supernova Remnant"},
 	    {'S', "Star"},
-	    {'T', "Stellar Object"},
+	    {'T', "Star-like Object"},
 	    {'U', "Cluster, with nebulosity"},
 	    {'V', "Variable Star"},
 	};
 
-	switch (op->type) {
+	switch (op->o_type) {
 	case FIXED:
 	    if (op->f_class) {
 		int i;
@@ -361,7 +374,7 @@ Obj *op;
 		    if (fixed_class_map[i].class == op->f_class)
 			return (fixed_class_map[i].desc);
 	    }
-	    return ("Fixed");
+	    return (op->o_type == FIXED ? "Fixed" : "Field Star");
 	case PARABOLIC:
 	    return ("Solar - Parabolic");
 	case HYPERBOLIC:
@@ -373,7 +386,7 @@ Obj *op;
 	case EARTHSAT:
 	    return ("Earth Sat");
 	default:
-	    printf ("obj_description: unknown type: 0x%x\n", op->type);
+	    printf ("obj_description: unknown type: 0x%x\n", op->o_type);
 	    exit (1);
 	    return (NULL);	/* for lint */
 	}
@@ -417,7 +430,9 @@ Widget w;
 	XtFree (txtp);
 }
 
-/* given a Now *, find the local sidereal time */
+/* given a Now *, find the local sidereal time, in hours.
+ * if epoch == EOD we return apparent time, else just mean time.
+ */
 void
 now_lst (np, lst)
 Now *np;
@@ -432,6 +447,15 @@ double *lst;
 
 	utc_gst (mjd_day(mjd), mjd_hr(mjd), lst);
 	*lst += radhr(lng);
+
+	if (epoch == EOD) {
+	    double eps, deps, dpsi;
+
+	    obliquity(mjd, &eps);
+	    nutation(mjd, &deps, &dpsi);
+	    *lst += radhr(dpsi*cos(eps+deps));
+	}
+
 	range (lst, 24.0);
 
 	last_mjd = mjd;
@@ -439,6 +463,7 @@ double *lst;
 	last_lst = *lst;
 }
 
+
 /* return !0 if any of the button/data capture tools are active, else 0.
  */
 int
@@ -536,12 +561,14 @@ double *mp;
 	*mp = h + 5.0*log10(rp*rho) - 2.5*log10((1-g)*Psi_1 + g*Psi_2);
 }
 
-/* given faintest desired mag, image scale and object magnitude and size,
- * return diameter to draw object, in pixels, or 0 if dimmer than fmag.
+/* given faintest desired mag, mag step magstp, image scale and object
+ * magnitude and size, return diameter to draw object, in pixels, or 0 if
+ * dimmer than fmag.
  */
 int
-magdiam (fmag, scale, mag, size)
+magdiam (fmag, magstp, scale, mag, size)
 int fmag;	/* faintest mag */
+int magstp;	/* mag range per dot size */
 double scale;	/* rads per pixel */
 double mag;	/* magnitude */
 double size;	/* rads, or 0 */
@@ -550,7 +577,7 @@ double size;	/* rads, or 0 */
 	
 	if (mag > fmag)
 	    return (0);
-	diam = (int)(fmag - mag + 1);
+	diam = (int)((fmag - mag)/magstp + 1);
 	sized = (int)(size/scale + 0.5);
 	if (sized > diam)
 	    diam = sized;
@@ -581,7 +608,8 @@ char *buf;
 	return (atof (buf));
 }
 
-/* just like stdio fopen() but substitutes HOME if name starts with '~'.
+/* just like stdio fopen() but substitutes HOME if name starts with '~'
+ * and we remove any trailing blanks IN PLACE.
  */
 FILE *
 fopenh (name, how)
@@ -589,10 +617,26 @@ char *name;
 char *how;
 {
 	FILE *fp;
+	int i;
 
+	/* get home, if we don't already know it */
 	if (!home)
 	    home = getenv ("HOME");
 
+	/* skip leading blanks */
+	while (*name == ' ')
+	    name++;
+
+	/* move '\0' back past any trailing blanks */
+	for (i = strlen(name)-1; i >= 0; i--)
+	    if (name[i] == ' ')
+		name[i] = '\0';
+	    else
+		break;
+
+	/* prepend home if name starts with ~
+	 * open net filename.
+	 */
 	if (name[0] == '~' && home) {
 	    char *fullname= (char *) malloc (strlen(home) + strlen(name+1) + 1);
 	    (void) sprintf (fullname, "%s%s", home, name+1);
@@ -607,9 +651,10 @@ char *how;
 /* rather like open(2) but substitutes HOME if name starts with '~'.
  */
 int
-openh (name, flags)
+openh (name, flags, perm)
 char *name;
 int flags;
+int perm;
 {
 	int fd;
 
@@ -619,10 +664,10 @@ int flags;
 	if (name[0] == '~' && home) {
 	    char *fullname= (char *) malloc (strlen(home) + strlen(name+1) + 1);
 	    (void) sprintf (fullname, "%s%s", home, name+1);
-	    fd = open (fullname, flags);
+	    fd = open (fullname, flags, perm);
 	    free (fullname);
 	} else
-	    fd = open (name, flags);
+	    fd = open (name, flags, perm);
 
 	return (fd);
 }
@@ -658,25 +703,28 @@ char *name;
 }
 
 /* return a string for whatever is in errno right now.
- * I never would have imagined it would be so hard to turn errno into a string!
+ * I never would have imagined it would be so crazy to turn errno into a string!
  */
 char *
 syserrstr ()
 {
 #if defined(__STDC__) && !defined(__GNUC__)
-/* gcc doesn't have strerror */
+/* some older gcc don't have strerror */
 #include <errno.h>
-#include <string.h>
-	return (strerror (errno));
+return (strerror (errno));
 #else
 #if defined(VMS)
 #include <errno.h>
 #include <perror.h>
 #else
-	extern char *sys_errlist[];
-	extern int errno;
+#if !defined(__FreeBSD__) && !defined(__EMX__)
+/* this is aready in stdio.h on FreeBSD */
+/* this is already in stdlib.h in EMX   M. Goldberg 27 January 1997 for OS/2 */
+extern char *sys_errlist[];
 #endif
-	return (sys_errlist[errno]);
+extern int errno;
+#endif
+return (sys_errlist[errno]);
 #endif
 }
 
@@ -690,7 +738,8 @@ syserrstr ()
  *    B ____________ C
  *           a
  *
- * given A, b, c find B and a in range 0..B..2PI, 0..a..PI
+ * given A, b, c find B and a in range 0..B..2PI and 0..a..PI, respectively..
+ * cap and Bp may be NULL if not interested in either one.
  * N.B. we pass in cos(c) and sin(c) because in many problems one of the sides
  *   remains constant for many values of A and b.
  */
@@ -704,11 +753,15 @@ double *cap, *Bp;
 	double cb = cos(b), sb = sin(b);
 	double ca;
 	double B;
-	double x, y;
 
 	ca = cb*cc + sb*sc*cA;
 	if (ca >  1.0) ca =  1.0;
 	if (ca < -1.0) ca = -1.0;
+	if (cap)
+	    *cap = ca;
+
+	if (!Bp)
+	    return;
 
 	if (cc > .99999) {
 	    /* as c approaches 0, B approaches pi - A */
@@ -720,6 +773,8 @@ double *cap, *Bp;
 	    /* compute cB and sB and remove common factor of sa from quotient.
 	     * be careful where B causes atan to blow.
 	     */
+	    double x, y;
+
 	    y = sA*sb*sc;
 	    x = cb - ca*cc;
 	
@@ -729,7 +784,6 @@ double *cap, *Bp;
 		B = atan2 (y, x);
 	}
 
-	*cap = ca;
 	*Bp = B;
 	range (Bp, 2*PI);
 }
@@ -754,7 +808,7 @@ FILE **fpp;
 	FILE *fp;
 
 	/* get the name of the fifo from the fifores resource */
-	fn = XGetDefault (XtD, myclass, xres);
+	fn = getXRes (xres);
 	if (!fn) {
 	    (void) sprintf (msg, "No `%.200s' resource.\n", xres);
 	    xe_msg (msg, 0);
@@ -824,7 +878,7 @@ struct exception *xp;
 		case 3: /* pow */
 		/* FALLTHRU */
 		case 4: /* exp */
-		    xp->retval = xp->type == OVERFLOW ? 1e308 : 0.0;
+		    xp->retval = xp->o_type == OVERFLOW ? 1e308 : 0.0;
 		    break;
 		case 5: /* log */
 		/* FALLTHRU */
@@ -879,3 +933,39 @@ int mag;	/* limiting magnitude */
 	(void) fclose (fp);
 	return (0);
 }
+
+/* given the difference in two RA's, in rads, return their difference,
+ *   accounting for wrap at 2*PI. caller need *not* first force it into the
+ *   range 0..2*PI.
+ */
+double
+delra (dra)
+double dra;
+{
+	double fdra = fmod(fabs(dra), 2*PI);
+
+	if (fdra > PI)
+	    fdra = 2*PI - fdra;
+	return (fdra);
+}
+
+/* return 1 if op is any of the FIXED star-like classes, else 0.
+ */
+int
+is_starlike (op)
+Obj *op;
+{
+	int starlike = 0;
+
+	if (is_type(op, FIXEDM)) {
+	    switch (op->f_class) {
+	    case 'T': case 'B': case 'D': case 'M': case 'S': case 'V':
+		starlike = 1;
+		break;
+	    default:
+		break;
+	    }
+	}
+
+	return (starlike);
+}
