@@ -22,10 +22,12 @@
 
 
 extern Widget toplevel_w;
+extern Colormap xe_cm;
 
 extern void prompt_map_cb P_((Widget w, XtPointer client, XtPointer call));
 extern void set_something P_((Widget w, char *resource, XtArgVal value));
 extern void set_xmstring P_((Widget w, char *resource, char *txt));
+extern void wtip P_((Widget w, char *tip));
 
 static void msg_on_top P_((void));
 static void msg_create_w P_((void));
@@ -34,7 +36,6 @@ static void msg_close_cb P_((Widget w, XtPointer client, XtPointer call));
 static void msg_add P_((char *msg));
 static void msg_scroll_down P_((void));
 static void xe_msg_modal P_((char *p));
-static void msg_bell P_((void));
 
 
 #define	MINBELLSECS	3	/* min time between bell rings, seconds */
@@ -58,6 +59,21 @@ msg_manage()
 	    XtUnmanageChild (msg_w);
 }
 
+/* ring the bellm but avoid overdoing a lot of them */
+void
+msg_bell()
+{
+	static long lastbellt;
+	long t;
+
+	t = time (NULL);
+
+	if (t - lastbellt >= MINBELLSECS) {
+	    XBell (XtDisplay(toplevel_w), 0);
+	    lastbellt = t;
+	}
+}
+
 /* if app_modal != 0
  *   make an application modal info box,
  * else
@@ -96,7 +112,7 @@ Cursor c;
 {
 	Window win;
 
-	if (msg_w && (win = XtWindow(msg_w))) {
+	if (msg_w && (win = XtWindow(msg_w)) != 0) {
 	    Display *dsp = XtDisplay(msg_w);
 	    if (c)
 		XDefineCursor (dsp, win, c);
@@ -134,9 +150,10 @@ msg_create_w()
 	static struct {
 	    char *name;
 	    void (*cb)();
+	    char *tip;
 	} cb[] = {
-	    {"Erase", msg_erase_cb},
-	    {"Close", msg_close_cb},
+	    {"Erase", msg_erase_cb, "Erase messages"},
+	    {"Close", msg_close_cb, "Close this dialog"},
 	};
 	Widget w;
 	Arg args[20];
@@ -148,9 +165,11 @@ msg_create_w()
 	XtSetArg (args[n], XmNdefaultPosition, False); n++;
 	XtSetArg (args[n], XmNautoUnmanage, False); n++;
 	XtSetArg (args[n], XmNfractionBase, 9); n++;
+	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
 	msg_w = XmCreateFormDialog (toplevel_w, "Message", args, n);
+	set_something (msg_w, XmNcolormap, (XtArgVal)xe_cm);
 	XtAddCallback (msg_w, XmNmapCallback, prompt_map_cb, NULL);
-	set_something (XtParent(msg_w), XmNtitle, (XtArgVal)"xephem Message Box");
+	set_something (XtParent(msg_w),XmNtitle,(XtArgVal)"xephem Message Box");
 
 	/* make the control buttons */
 
@@ -164,6 +183,7 @@ msg_create_w()
 	    XtSetArg (args[n], XmNrightPosition, 3 + 5*i); n++;
 	    w = XmCreatePushButton (msg_w, cb[i].name, args, n);
 	    XtAddCallback (w, XmNactivateCallback, cb[i].cb, NULL);
+	    wtip (w, cb[i].tip);
 	    XtManageChild (w);
 	}
 
@@ -179,7 +199,7 @@ msg_create_w()
 	XtSetArg (args[n], XmNeditMode, XmMULTI_LINE_EDIT); n++;
 	XtSetArg (args[n], XmNeditable, False); n++;
 	XtSetArg (args[n], XmNcursorPositionVisible, False); n++;
-	txt_w = XmCreateScrolledText (msg_w, "ScrolledText", args, n);
+	txt_w = XmCreateScrolledText (msg_w, "Messages", args, n);
 	XtManageChild (txt_w);
 }
 
@@ -260,7 +280,9 @@ char *p;
 	    XtSetArg (args[n], XmNtitle, "xephem Important Message");  n++;
 	    XtSetArg (args[n], XmNokLabelString, button_string);  n++;
 	    XtSetArg (args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); n++;
+	    XtSetArg (args[n], XmNcolormap, xe_cm); n++;
 	    apmsg_w = XmCreateWarningDialog (toplevel_w, "ModalMessage",args,n);
+	    set_something (apmsg_w, XmNcolormap, (XtArgVal)xe_cm);
 	    XtAddCallback (apmsg_w, XmNmapCallback, prompt_map_cb, NULL);
 
 	    XmStringFree (title_string);
@@ -276,18 +298,10 @@ char *p;
 
 	/* Display help window. rely on autoUnmanage to bring back down. */
 	XtManageChild (apmsg_w);
-}
 
-static void
-msg_bell()
-{
-	static long lastbellt;
-	long t;
-
-	t = time (NULL);
-
-	if (t - lastbellt >= MINBELLSECS) {
-	    XBell (XtDisplay(msg_w), 0);
-	    lastbellt = t;
-	}
+	/* make sure it's up because we are occasionally used when the main
+	 * loop might now run for a while
+	 */
+	XmUpdateDisplay (toplevel_w);
+	XSync (XtDisplay(toplevel_w), False);
 }
