@@ -26,6 +26,7 @@
 #include "preferences.h"
 
 extern Widget toplevel_w;
+extern Colormap xe_cm;
 
 extern Now *mm_get_now P_((void));
 extern Obj *db_basic P_((int id));
@@ -35,10 +36,10 @@ extern int db_n P_((void));
 extern int db_set_field P_((char bp[], int id, PrefDateFormat pref, Obj *op));
 extern int obj_cir P_((Now *np, Obj *op));
 extern void all_newobj P_((int dbidx));
-extern void db_scaninit P_((DBScan *sp));
+extern void db_scaninit P_((DBScan *sp, int mask, ObjF *op, int nop));
 extern void db_setuserobj P_((int id, Obj *op));
+extern void f_prdec P_((Widget w, double a));
 extern void f_double P_((Widget w, char *fmt, double f));
-extern void f_pangle P_((Widget w, double a));
 extern void f_ra P_((Widget w, double ra));
 extern void f_string P_((Widget w, char *s));
 extern void fs_date P_((char out[], double jd));
@@ -51,6 +52,8 @@ extern void set_xmstring P_((Widget w, char *resource, char *txt));
 extern void sv_id P_((Obj *op));
 extern void sv_point P_((Obj *op));
 extern void watch_cursor P_((int want));
+extern void wtip P_((Widget w, char *tip));
+extern void xe_msg P_((char *msg, int app_modal));
 extern void zero_mem P_((void *loc, unsigned len));
 
 /* info about each field for a given type of object.
@@ -64,6 +67,7 @@ typedef struct {
     int id;		/* field id */
     char *label;	/* label */
     char *prompt;	/* prompt if changeable, or 0 if it can't be */
+    char *tip;		/* widget tip */
     char *altp[3];	/* alternate prompts for fields effected by prefs */
     Widget pb_w;	/* PushButton if it can be changed, else Label */
 } Field;
@@ -73,13 +77,14 @@ static void obj_unsetmenu P_((void));
 static void obj_set_from_db P_((Obj *op));
 static void obj_set_db P_((void));
 static void obj_setmenu P_((void));
+static void obj_setxyz P_((void));
 static void obj_setobjinfo P_((Field *fp, int nf));
 static void obj_create_form P_((void));
 static void obj_create_type_rc_form P_((Widget fw, Field *fp));
 static void obj_change_cb P_((Widget w, XtPointer client, XtPointer call));
 static void obj_ctl_cb P_((Widget w, XtPointer client, XtPointer call));
 static void do_help P_((void));
-static void obj_xy_cb P_((Widget w, XtPointer client, XtPointer call));
+static void obj_xyz_cb P_((Widget w, XtPointer client, XtPointer call));
 static void obj_type_cb P_((Widget w, XtPointer client, XtPointer call));
 static void obj_select_cb P_((Widget w, XtPointer client, XtPointer call));
 static void obj_scroll_cb P_((Widget w, XtPointer client, XtPointer call));
@@ -99,145 +104,174 @@ static void epoch_as_mdy P_((Widget w, double e));
 static Widget objform_w;	/* overall form Dialog */
 static Widget fldrc_w;		/* overall field info rc */
 static Widget scroll_w;		/* name scroll widget */
+static Widget srch_w;		/* the search text field */
+
+static Widget objx_w;		/* ObjX selection TB */
+static Widget objy_w;		/* ObjY selection TB */
+static Widget objz_w;		/* ObjZ selection TB */
 
 static Field fixed_fields[] = {
-    {O_NAME,  "Name:",  "Object name:"},
-    {F_CLASS, "Class:",	NULL},
-    {F_RA,    "RA:",	"RA (h:m:s):"},
-    {F_DEC,   "Dec:",	"Dec (d:m:s):"},
-    {F_MAG,   "Mag:",	"Magnitude:"},
-    {F_EPOCH, "Epoch:", "dummy", {
+    {O_NAME,  "Name:",  "Object name:", "Name of Object"},
+    {F_CLASS, "Class:",	NULL, "Database type classifcation"},	/* TODO */
+    {F_RA,    "RA:",	"RA (h:m:s):", "Right Ascension (epoch below)"},
+    {F_DEC,   "Dec:",	"Dec (d:m:s):", "Declination (epoch below)"},
+    {F_MAG,   "Mag:",	"Magnitude:", "Nominal magnitude"},
+    {F_EPOCH, "Epoch:", "dummy", "Epoch of RA and Dec", {
 	"Reference epoch\n(UT Date as m/d.d/y or year.d):",
 	"Reference epoch\n(UT Date as y/m/d.d or year.d):",
 	"Reference epoch\n(UT Date as d.d/m/y or year.d):"
 	}
     },
-    {F_SIZE,  "Size:",  "Angular Size (arc secs):"},
+    {F_SIZE,  "Size:",  "Angular Size (arc secs):", "Angular size"},
 };
 
 static Field ellip_fields[] = {
-    {O_NAME, "Name:", "Object name:"},
-    {E_INC, "Inclination:", "Inclination (degs):"},
-    {E_LAN, "Long of Asc Nod:", "Longitude of ascending node (degs):"},
-    {E_AOP, "Arg of Peri:", "Argument of Perihelion (degs):"},
-    {E_A, "Mean Dist:", "Mean distance (AU):"},
-    {E_N, "Daily Motion:", "Daily motion (degs/day):"},
-    {E_E, "Eccentricity:", "Eccentricty (<1):"},
-    {E_M, "Mean Anomaly:", "Mean Anomaly (degs):"},
-    {E_CEPOCH, "C Epoch:", "dummy", {
-	"Epoch date\n(UT Date as m/d.d/y or year.d):",
-	"Epoch date\n(UT Date as y/m/d.d or year.d):",
-	"Epoch date\n(UT Date as d.d/m/y or year.d):"
+    {O_NAME, "Name:", "Object name:", "Name of Object"},
+    {E_INC, "Inclination:", "Inclination (degs):", "Orbit inclination"},
+    {E_LAN, "Long of Asc Nod:", "Longitude of ascending node (degs):",
+    	"Longitude of ascending node"},
+    {E_AOP, "Arg of Peri:", "Argument of Perihelion (degs):",
+    	"Argument of Perihelion"},
+    {E_A, "Semi-Axis:", "Semi-Major axis (AU):", "Semi-major axis of orbit"},
+    {E_E, "Eccentricity:", "Eccentricty (<1):", "Orbital eccentricity"},
+    {E_M, "Mean Anomaly:", "Mean Anomaly (degs):",
+    	"Mean anomaly at moment of MA Epoch"},
+    {E_CEPOCH, "MA Epoch:", "dummy", "Date of Mean Anomaly value", {
+	"Epoch date of MA\n(UT Date as m/d.d/y or year.d):",
+	"Epoch date of MA\n(UT Date as y/m/d.d or year.d):",
+	"Epoch date of MA\n(UT Date as d.d/m/y or year.d):"
 	}
     },
-    {E_EPOCH, "Epoch:", "dummy", {
+    {E_EPOCH, "Equinox:", "dummy", "Reference date of orbital parameters", {
 	"Equinox year\n(UT Date as m/d.d/y or year.d):",
 	"Equinox year\n(UT Date as y/m/d.d or year.d):",
 	"Equinox year\n(UT Date as d.d/m/y or year.d):"
 	}
     },
-    {E_M1, "Mag coeff 1:", "Magnitude coefficient 1 (g#, H# or just #):"},
-    {E_M2, "Mag coeff 2:", "Magnitude coefficient 2 (k#, G# or just #):"},
-    {E_SIZE, "Size:", "Angular Size @ 1 AU (arc secs):"},
+    {E_M1, "Mag coeff 1:", "Magnitude coefficient 1\n(g#, H# or just #):",
+    	"First coefficient in magnitude model: g or H"},
+    {E_M2, "Mag coeff 2:", "Magnitude coefficient 2\n(k#, G# or just #):",
+    	"Second coefficient in magnitude model: k or G"},
+    {E_SIZE, "Size:", "Angular Size @ 1 AU (arc secs):","Angular size at 1 AU"},
 };
 
 static Field hyper_fields[] = {
-    {O_NAME, "Name:", "Object name:"},
-    {H_EP, "Ep of Peri:", "dummy", {
+    {O_NAME, "Name:", "Object name:", "Name of Object"},
+    {H_EP, "Ep of Peri:", "dummy", "Date of Perihelion", {
 	"Epoch of perihelion\n(UT Date as m/d.d/y or year.d):",
 	"Epoch of perihelion\n(UT Date as y/m/d.d or year.d):",
 	"Epoch of perihelion\n(UT Date as d.d/m/y or year.d):"
 	}
     },
-    {H_INC, "Inclination:", "Inclination (degs):"},
-    {H_LAN, "Long of Asc Nod:", "Longitude of ascending node (degs):"},
-    {H_AOP, "Arg of Peri:", "Argument of perihelion (degs):"},
-    {H_E, "Eccentricity:", "Eccentricity (>1):"},
-    {H_QP, "Peri Dist:", "Perihelion distance (AU):"},
-    {H_EPOCH, "Epoch:", "dummy", {
+    {H_INC, "Inclination:", "Inclination (degs):", "Orbital inclination"},
+    {H_LAN, "Long of Asc Nod:", "Longitude of ascending node (degs):",
+    	"Longitude of Ascending Node"},
+    {H_AOP, "Arg of Peri:", "Argument of perihelion (degs):",
+    	"Argument if Perihelion"},
+    {H_E, "Eccentricity:", "Eccentricity (>1):", "Oribital eccentricty"},
+    {H_QP, "Peri Dist:", "Perihelion distance (AU):", "Distance at perihelion"},
+    {H_EPOCH, "Epoch:", "dummy", "Reference date of orbital parameters", {
 	"Reference epoch\n(UT Date as m/d.d/y or year.d):",
 	"Reference epoch\n(UT Date as y/m/d.d or year.d):",
 	"Reference epoch\n(UT Date as d.d/m/y or year.d):"
 	}
     },
-    {H_G, "g:", "g, as in m = g + 5*log(delta) + 2.5*k*log(r):"},
-    {H_K, "k:", "k, as in m = g + 5*log(delta) + 2.5*k*log(r):"},
-    {H_SIZE, "Size:", "Angular Size @ 1 AU (arc secs):"},
+    {H_G, "g:", "g, as in m = g + 5*log(delta) + 2.5*k*log(r):", 
+    	"g magnitude model, as in m = g + 5*log(delta) + 2.5*k*log(r)"},
+    {H_K, "k:", "k, as in m = g + 5*log(delta) + 2.5*k*log(r):",
+    	"k magnitude model, as in m = g + 5*log(delta) + 2.5*k*log(r)"},
+    {H_SIZE, "Size:", "Angular Size @ 1 AU (arc secs):","Angular size at 1 AU"},
 };
 
 static Field para_fields[] = {
-    {O_NAME, "Name:", "Object name:"},
-    {P_EP, "Ep of Peri:", "dummy", {
+    {O_NAME, "Name:", "Object name:", "Name of Object"},
+    {P_EP, "Ep of Peri:", "dummy", "Date of perihelion", {
 	"Epoch of perihelion\n(UT Date as m/d.d/y or year.d):",
 	"Epoch of perihelion\n(UT Date as y/m/d.d or year.d):",
 	"Epoch of perihelion\n(UT Date as d.d/m/y or year.d):"
 	}
     },
-    {P_INC, "Inclination:", "Inclination (degs):"},
-    {P_AOP, "Arg of Peri:", "Argument of perihelion (degs):"},
-    {P_QP, "Peri Dist:", "Perihelion distance (AU):"},
-    {P_LAN, "Long of Asc Nod:", "Longitude of ascending node (degs):"},
-    {P_EPOCH, "Epoch:", "dummy", {
+    {P_INC, "Inclination:", "Inclination (degs):", "Oribital inclination"},
+    {P_AOP, "Arg of Peri:", "Argument of perihelion (degs):",
+    	"Argument of perihelion"},
+    {P_QP, "Peri Dist:", "Perihelion distance (AU):", "Distance at perihelion"},
+    {P_LAN, "Long of Asc Nod:", "Longitude of ascending node (degs):",
+    	"Longitude of ascending node"},
+    {P_EPOCH, "Epoch:", "dummy", "Date of orbital parameters", {
 	"Reference epoch\n(UT Date as m/d.d/y or year.d):",
 	"Reference epoch\n(UT Date as y/m/d.d or year.d):",
 	"Reference epoch\n(UT Date as d.d/m/y or year.d):"
 	}
     },
-    {P_G, "g:", "g, as in m = g + 5*log(delta) + 2.5*k*log(r):"},
-    {P_K, "k:", "k, as in m = g + 5*log(delta) + 2.5*k*log(r):"},
-    {P_SIZE, "Size:", "Angular Size @ 1 AU (arc secs):"},
+    {P_G, "g:", "g, as in m = g + 5*log(delta) + 2.5*k*log(r):",
+    	"g magnitude model, as in m = g + 5*log(delta) + 2.5*k*log(r)"},
+    {P_K, "k:", "k, as in m = g + 5*log(delta) + 2.5*k*log(r):",
+    	"k magnitude model, as in m = g + 5*log(delta) + 2.5*k*log(r)"},
+    {P_SIZE, "Size:", "Angular Size @ 1 AU (arc secs):","Angular size at 1 AU"},
 };
 
 static Field esat_fields[] = {
-    {O_NAME, "Name:", "Object name:"},
-    {ES_EPOCH, "Ep of Perigee:", "dummy", {
+    {O_NAME, "Name:", "Object name:", "Name of Object"},
+    {ES_EPOCH, "Ep of Perigee:", "dummy", "Epoch of perigee", {
 	"Epoch of perigee\n(UT Date as m/d.d/y or year.d):",
 	"Epoch of perigee\n(UT Date as y/m/d.d or year.d):",
 	"Epoch of perigee\n(UT Date as d.d/m/y or year.d):"
 	}
     },
-    {ES_INC, "Inclination:", "Inclination (degs):"},
-    {ES_RAAN, "RA of Node:", "RA of Ascending Node (degs):"},
-    {ES_E, "Eccentricty:", "Eccentricty (<1):"},
-    {ES_AP, "Arg of Perigee:", "Argument of Perigee (degs):"},
-    {ES_M, "Mean Anomaly:", "Mean Anomaly (degs):"},
-    {ES_N, "Daily motion:", "Daily motion (revs/day):"},
-    {ES_DECAY, "Orbit decay:", "Orbital decay rate (revs/day^2):"},
-    {ES_ORBIT, "Orbit reference:", "Orbit number at epoch (integer):"},
+    {ES_INC, "Inclination:", "Inclination (degs):", "Orbital inclination"},
+    {ES_RAAN, "RA of Node:", "RA of Ascending Node (degs):",
+    	"Right Ascension of Ascending Node"},
+    {ES_E, "Eccentricty:", "Eccentricty (<1):", "Orbital eccentricity"},
+    {ES_AP, "Arg of Perigee:", "Argument of Perigee (degs):",
+    	"Argument of perigee"},
+    {ES_M, "Mean Anomaly:", "Mean Anomaly (degs):", "Mean anomaly"},
+    {ES_N, "Daily motion:", "Daily motion (revs/day):", "Revolutions per day"},
+    {ES_DECAY, "Orbit decay:", "Orbital decay rate (revs/day^2):",
+    	"Revolutions per day^2"},
+    {ES_ORBIT, "Orbit reference:", "Orbit number at epoch (integer):",
+    	"Orbit number at epoch"},
 };
 
 typedef struct {
     char *name;		/* name of type toggle button */
     Field *fp;		/* ptr to Fields array defining the info, if any */
     int nfields;	/* number of Fields in fp[] */
+    char *tip;		/* widget tip text */
     Widget tb_w;	/* type toggle button widget */
     Widget frc_w;	/* Field rc widget */
 } TypeInfo;
+
 /* names of each type.
  * N.B. must match order of enum in circum.h.
  */
 static TypeInfo typeinfo[NOBJTYPES-1 /*No PLANET*/] = {
-    {"Undefined",  NULL, 	  0},
-    {"Fixed",	   fixed_fields,  XtNumber(fixed_fields)},
-    {"Elliptical", ellip_fields,  XtNumber(ellip_fields)},
-    {"Hyperbolic", hyper_fields,  XtNumber(hyper_fields)},
-    {"Parabolic",  para_fields,   XtNumber(para_fields)},
-    {"Earth Sat",  esat_fields,   XtNumber(esat_fields)},
+    {"Undefined",  NULL, 	  0,
+    	"Current object is, or force it to be, Undefined"},
+    {"Fixed",	   fixed_fields,  XtNumber(fixed_fields),
+    	"Current object is, or force it to be, of type Fixed"},
+    {"Elliptical", ellip_fields,  XtNumber(ellip_fields),
+    	"Current object is, or force it to be, of type Sol Sys Elliptical"},
+    {"Hyperbolic", hyper_fields,  XtNumber(hyper_fields),
+    	"Current object is, or force it to be, of type Sol Sys Hyperbolic"},
+    {"Parabolic",  para_fields,   XtNumber(para_fields),
+    	"Current object is, or force it to be, of type Sol Sys Parabolic"},
+    {"Earth Sat",  esat_fields,   XtNumber(esat_fields),
+    	"Current object is, or force it to be, of type Earth Satellite"},
 };
 
-/* working storage - set when Reset, made permanent in database with Apply
+/* working storage - set when Reset, made permanent in database with Ok/Apply
  */
-static Obj objx;
-static Obj objy;
+static Obj objx;			/* local working copy */
+static Obj objy;			/* local working copy */
+static Obj objz;			/* local working copy */
 static int objidx = OBJX;		/* which one is on-screen now */
-#define	objp	(objidx == OBJY ? &objy : &objx)
+#define	objp	(objidx == OBJX ? &objx : objidx == OBJY ? &objy : &objz)
 
 /* used to maintain mapping between object selection buttons and database
  * objects. the userData of each object selection button is set to the index of
  * a namelist[].
  */
 typedef struct {
-	char *name;
 	Obj *op;
 } NameList;
 
@@ -253,6 +287,8 @@ enum {
     OK, APPLY, RESET, POINT, IDENT, CANCEL, HELP
 };
 
+static int newdb;		/* set when db is updated while we are down */
+
 /* called by the main menu pick.
  * create the main form, if this is the first time we've been called.
  * then we toggle each time.
@@ -263,51 +299,64 @@ obj_manage()
 	if (!objform_w) {
 	    obj_create_form();
 	    init_working_obj();
-	    new_namelist();
-	    obj_setmenu();
+	    newdb = 1;	/* force a fresh list */
 	}
 	
 	if (XtIsManaged(objform_w))
 	    XtUnmanageChild (objform_w);
-	else
+	else {
+	    if (newdb) {
+		new_namelist();
+		newdb = 0;
+	    }
+	    obj_setxyz();
+	    obj_setmenu();
 	    XtManageChild (objform_w);
+	}
 }
 
-/* update our view if we have been created */
+/* update our view if we are up */
 /* ARGSUSED */
 void
 obj_update (np, howmuch)
 Now *np;
 int howmuch;
 {
-	if (!objform_w)
+	if (!objform_w || !XtIsManaged(objform_w))
 	    return;
 
-	new_namelist();
 	obj_setmenu();
 }
 
-/* make the given arbitrary db object the current objectX/Y:
+/* make the given arbitrary db object the current value of ObjXYZ.
  * update our working copy, the real db object, all the stuff in the menu (if
  * we've been managed!) and inform all others.
  */
 void
-obj_set (op)
+obj_set (op, dbidx)
 Obj *op;
+int dbidx;
 {
 	static char me[] = "obj_set()";
 
 	/* TODO: we don't support planets */
-	if (op->type == PLANET) {
+	if (op->o_type == PLANET) {
 	    printf ("%s: planets not supported\n", me);
 	    exit (1);
 	}
 
+	if (dbidx != OBJX && dbidx != OBJY && dbidx != OBJZ) {
+	    printf ("%s: not one of XYZ: %d\n", me, dbidx);
+	    exit(1);
+	}
+
 	obj_unsetmenu();
+	objidx = dbidx;		/* effectively changes current objp */
 	obj_set_from_db(op);	/* *objp <- *op */
-	obj_setmenu();
+	obj_setxyz();		/* set the correct objxyz_w TB to match */
+	obj_setmenu();		/* load menu items */
 	obj_set_db();		/* db_setuserobj(objp) */
-	all_newobj (objidx);
+	all_newobj (objidx);	/* tell all */
 }
 
 /* called when the db has changed (beyond NOBJ that is).
@@ -319,8 +368,11 @@ void
 obj_newdb(appended)
 int appended;
 {
-	if (!objform_w)
+	if (!objform_w || !XtIsManaged(objform_w)) {
+	    newdb = 1;
 	    return;
+	}
+
 	new_namelist();
 }
 
@@ -331,7 +383,7 @@ Cursor c;
 {
 	Window win;
 
-	if (objform_w && (win = XtWindow(objform_w))) {
+	if (objform_w && (win = XtWindow(objform_w)) != 0) {
 	    Display *dsp = XtDisplay(objform_w);
 	    if (c)
 		XDefineCursor (dsp, win, c);
@@ -341,7 +393,7 @@ Cursor c;
 }
 
 /* this was added to support backward compatability with the ability to
- * set OBJX and OBJY from the ephem.cfg file.
+ * set OBJXYZ from the ephem.cfg file.
  */
 static void
 init_working_obj()
@@ -352,6 +404,8 @@ init_working_obj()
 	objx = *op;
 	op = db_basic(OBJY);
 	objy = *op;
+	op = db_basic(OBJZ);
+	objz = *op;
 }
 
 /* turn off menu info about the current object.
@@ -362,8 +416,27 @@ obj_unsetmenu()
 	if (!objform_w)
 	    return;
 
-	XtUnmanageChild (typeinfo[objp->type].frc_w);
-	XmToggleButtonSetState (typeinfo[objp->type].tb_w, False, False);
+	XtUnmanageChild (typeinfo[objp->o_type].frc_w);
+	XmToggleButtonSetState (typeinfo[objp->o_type].tb_w, False, False);
+}
+
+/* set the xyz toggle buttons to match objidx */
+static void
+obj_setxyz()
+{
+	if (!objform_w)
+	    return;
+
+	XmToggleButtonSetState (objx_w, False, False);
+	XmToggleButtonSetState (objy_w, False, False);
+	XmToggleButtonSetState (objz_w, False, False);
+
+	switch (objidx) {
+	case OBJX: XmToggleButtonSetState (objx_w, True, False); break;
+	case OBJY: XmToggleButtonSetState (objy_w, True, False); break;
+	case OBJZ: XmToggleButtonSetState (objz_w, True, False); break;
+	default: printf ("Objsetxyz: impossible objidx: %d\n", objidx); exit(1);
+	}
 }
 
 /* copy the arbitrary database object, op, into the working copy of the
@@ -376,7 +449,7 @@ Obj *op;
 {
 	static char me[] = "obj_set_from_db()";
 
-	if (op->type == PLANET) {
+	if (op->o_type == PLANET) {
 	    printf ("%s: PLANET not supported\n", me);
 	    exit (1);
 	}
@@ -399,13 +472,11 @@ static void
 obj_setmenu()
 {
 	TypeInfo *tp;
-	int type;
 
 	if (!objform_w)
 	    return;
 
-	type = objp->type;
-	tp = &typeinfo[type];
+	tp = &typeinfo[objp->o_type];
 	XmToggleButtonSetState (tp->tb_w, True, False);
 	obj_setobjinfo(tp->fp, tp->nfields);
 	XtManageChild (tp->frc_w);
@@ -436,21 +507,24 @@ obj_create_form ()
 	struct Btns {	/* to streamline creation of control buttons */
 	    int id;
 	    char *name;
+	    char *tip;
 	};
 	static struct Btns ctlbtns[] = {
-	    {OK, "Ok"},
-	    {APPLY, "Apply"},
-	    {RESET, "Reset"},
-	    {CANCEL, "Close"},
-	    {HELP, "Help"}
+	    {OK,    "Ok",    "Define the current Object and close this dialog"},
+	    {APPLY, "Apply", "Define the current Object and leave dialog up"},
+	    {RESET, "Reset", "Restore current Object"},
+	    {CANCEL,"Close", "Close this dialog without changing anything"},
+	    {HELP,  "Help",  "Display detailed help information"}
 	};
 	static struct Btns skybtns[] = {
-	    {POINT, "Sky Point"},
-	    {IDENT, "Sky Mark"},
+	    {POINT, "Sky Point",
+	    	"Center Sky View on current working Object and mark"},
+	    {IDENT, "Sky Mark",
+		"Mark current working Object on Sky View, but only if visible"},
 	};
 	Widget typrbf_w, typrb_w;
 	Widget fldrcf_w;
-	Widget xyrbf_w, xyrb_w;
+	Widget xyzrbf_w, xyzrb_w;
 	Widget namerc_w, namercf_w;
 	Widget ctlfr_w, ctlf_w;
 	Widget skyfr_w, skyf_w;
@@ -465,14 +539,16 @@ obj_create_form ()
 	XtSetArg (args[n], XmNautoUnmanage, False); n++;
 	XtSetArg (args[n], XmNallowOverlap, False); n++;
 	XtSetArg (args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
+	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
 	objform_w = XmCreateFormDialog (toplevel_w, "Obj", args, n);
+	set_something (objform_w, XmNcolormap, (XtArgVal)xe_cm);
 	XtAddCallback (objform_w, XmNhelpCallback, obj_ctl_cb, (XtPointer)HELP);
 
 	/* set some stuff in the parent DialogShell.
 	 * setting XmNdialogTitle in the Form didn't work..
 	 */
 	n = 0;
-	XtSetArg (args[n], XmNtitle, "xephem ObjX/Y Menu"); n++;
+	XtSetArg (args[n], XmNtitle, "xephem Search and ObjXYZ"); n++;
 	XtSetValues (XtParent(objform_w), args, n);
 
 	/* make the type control radio box in a frame.
@@ -496,45 +572,56 @@ obj_create_form ()
 	    for (i = UNDEFOBJ; i < PLANET; i++) {
 		TypeInfo *tp = &typeinfo[i];
 		n = 0;
-		w= tp->tb_w = XmCreateToggleButton (typrb_w, tp->name, args, n);
+		w = tp->tb_w = XmCreateToggleButton (typrb_w, tp->name, args,n);
 		XtAddCallback (w, XmNvalueChangedCallback, obj_type_cb,
 								(XtPointer)i);
+		if (tp->tip)
+		    wtip (w, tp->tip);
 		XtManageChild (w);
 	    }
 
-	/* make the x/y selection radio box in a frame */
+	/* make the xyz selection radio box in a frame */
 
 	n = 0;
 	XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
 	XtSetArg (args[n], XmNleftWidget, typrbf_w); n++;
 	XtSetArg (args[n], XmNorientation, XmVERTICAL); n++;
-	xyrbf_w = XmCreateFrame (objform_w, "XYRBF", args, n);
-	XtManageChild (xyrbf_w);
+	xyzrbf_w = XmCreateFrame (objform_w, "XYRBF", args, n);
+	XtManageChild (xyzrbf_w);
 
 	n = 0;
-	xyrb_w = XmCreateRadioBox (xyrbf_w, "XYRBox", args, n);
-	XtManageChild (xyrb_w);
+	xyzrb_w = XmCreateRadioBox (xyzrbf_w, "XYRBox", args, n);
+	XtManageChild (xyzrb_w);
 
 	    n = 0;
-	    XtSetArg (args[n], XmNset, True); n++; /* must match objidx init */
-	    w = XmCreateToggleButton (xyrb_w, "ObjX", args, n);
-	    XtAddCallback (w, XmNvalueChangedCallback, obj_xy_cb,
+	    XtSetArg (args[n], XmNset, True); n++; /* N.B. must = objidx init */
+	    objx_w = XmCreateToggleButton (xyzrb_w, "ObjX", args, n);
+	    XtAddCallback (objx_w, XmNvalueChangedCallback, obj_xyz_cb,
 							    (XtPointer)OBJX);
-	    XtManageChild (w);
+	    wtip (objx_w, "Display, or define, user-defined Object X");
+	    XtManageChild (objx_w);
 
 	    n = 0;
-	    w = XmCreateToggleButton (xyrb_w, "ObjY", args, n);
-	    XtAddCallback (w, XmNvalueChangedCallback, obj_xy_cb,
+	    objy_w = XmCreateToggleButton (xyzrb_w, "ObjY", args, n);
+	    XtAddCallback (objy_w, XmNvalueChangedCallback, obj_xyz_cb,
 							    (XtPointer)OBJY);
-	    XtManageChild (w);
+	    wtip (objy_w, "Display, or define, user-defined Object Y");
+	    XtManageChild (objy_w);
+
+	    n = 0;
+	    objz_w = XmCreateToggleButton (xyzrb_w, "ObjZ", args, n);
+	    XtAddCallback (objz_w, XmNvalueChangedCallback, obj_xyz_cb,
+							    (XtPointer)OBJZ);
+	    wtip (objz_w, "Display, or define, user-defined Object Z");
+	    XtManageChild (objz_w);
 
 	/* make a rc in a frame to hold the name buttons and srch field */
 
 	n = 0;
 	XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
-	XtSetArg (args[n], XmNleftWidget, xyrbf_w); n++;
+	XtSetArg (args[n], XmNleftWidget, xyzrbf_w); n++;
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNrightOffset, SCROLLBAR_WIDTH); n++;
 	namercf_w = XmCreateFrame (objform_w, "NameF", args, n);
@@ -550,35 +637,41 @@ obj_create_form ()
 		XtAddCallback (w, XmNactivateCallback, obj_select_cb,
 								(XtPointer)i);
 		XtManageChild (w);
+		wtip (w, "Select to view this Object's definition");
 		namepb_w[i] = w;
 	    }
 
-	/* make the search label and text field in a frame */
+	/* make the search pb and text field in a frame */
 
 	n = 0;
 	XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
 	XtSetArg (args[n], XmNtopWidget, namercf_w); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
-	XtSetArg (args[n], XmNleftWidget, xyrbf_w); n++;
+	XtSetArg (args[n], XmNleftWidget, xyzrbf_w); n++;
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
 	srchf_w = XmCreateFrame (objform_w, "SrchF", args, n);
 	XtManageChild (srchf_w);
 
 
 	    n = 0;
+	    XtSetArg (args[n], XmNisAligned, False); n++;
 	    srchrc_w = XmCreateRowColumn (srchf_w, "SrchRS", args, n);
 	    XtManageChild (srchrc_w);
 
 	    n = 0;
-	    w = XmCreateLabel (srchrc_w, "SrchL", args, n);
+	    XtSetArg (args[n], XmNalignment, XmALIGNMENT_CENTER); n++;
+	    w = XmCreatePushButton (srchrc_w, "SrchL", args, n);
 	    set_xmstring (w, XmNlabelString, "Search:");
+	    XtAddCallback (w, XmNactivateCallback, obj_srch_cb, NULL);
+	    wtip (w,"Scan for next Object in memory with name described below");
 	    XtManageChild (w);
 
 	    n = 0;
 	    XtSetArg (args[n], XmNmaxLength, MAXNM-1); n++;
-	    w = XmCreateTextField (srchrc_w, "SrchTF", args, n);
-	    XtManageChild (w);
-	    XtAddCallback (w, XmNactivateCallback, obj_srch_cb, NULL);
+	    srch_w = XmCreateTextField (srchrc_w, "SrchTF", args, n);
+	    XtManageChild (srch_w);
+	    wtip (srch_w, "Enter all or partial name, hit RETURN to search");
+	    XtAddCallback (srch_w, XmNactivateCallback, obj_srch_cb, NULL);
 
 #if XmVersion >= 1001
 	    /* init kb focus here if possible */
@@ -631,6 +724,8 @@ obj_create_form ()
 		w = XmCreatePushButton (ctlf_w, ctlbtns[i].name, args, n);
 		XtAddCallback (w, XmNactivateCallback, obj_ctl_cb,
 						    (XtPointer)ctlbtns[i].id);
+		if (ctlbtns[i].tip)
+		    wtip (w, ctlbtns[i].tip);
 		XtManageChild (w);
 	    }
 
@@ -680,7 +775,7 @@ obj_create_form ()
 	XtSetArg (args[n], XmNbottomWidget, srchf_w); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
-	XtSetArg (args[n], XmNrightWidget, xyrbf_w); n++;
+	XtSetArg (args[n], XmNrightWidget, xyzrbf_w); n++;
 	skyfr_w = XmCreateFrame (objform_w, "SkyFr", args, n);
 	XtManageChild (skyfr_w);
 
@@ -702,6 +797,8 @@ obj_create_form ()
 		w = XmCreatePushButton (skyf_w, skybtns[i].name, args, n);
 		XtAddCallback (w, XmNactivateCallback, obj_ctl_cb,
 						    (XtPointer)skybtns[i].id);
+		if (skybtns[i].tip)
+		    wtip (w, skybtns[i].tip);
 		XtManageChild (w);
 	    }
 }
@@ -731,8 +828,10 @@ Field *fp;
 								(XtPointer)fp);
 	} else
 	    w = XmCreateLabel (fw, "ObjFldInfo", args, n);
-	XtManageChild (w);
 	fp->pb_w = w;
+	if (fp->tip)
+	    wtip (w, fp->tip);
+	XtManageChild (w);
 
 	/* make the label to the left of w */
 
@@ -807,7 +906,7 @@ XtPointer call;
 	    obj_setmenu();
 	    break;
 	case POINT:
-	    if (objp->type != UNDEFOBJ) {
+	    if (objp->o_type != UNDEFOBJ) {
 		if (obj_cir (mm_get_now(), objp) < 0) {
 		    printf ("obj_ctl_cb: can't get obj_cir for POINT\n");
 		    exit (1);
@@ -816,7 +915,7 @@ XtPointer call;
 	    }
 	    break;
 	case IDENT:
-	    if (objp->type != UNDEFOBJ) {
+	    if (objp->o_type != UNDEFOBJ) {
 		if (obj_cir (mm_get_now(), objp) < 0) {
 		    printf ("obj_ctl_cb: can't get obj_cir for IDENT\n");
 		    exit (1);
@@ -837,15 +936,12 @@ static void
 do_help()
 {
 	static char *msg[] = {
-"Object X and Object Y may be set to any of five types of objects: fixed, in",
-"heliocentric elliptical, hyperbolic or parabolic orbits or an earth",
-"satellite. Give the location or orbital elements as necessary.",
-"",
-"The object may also be loaded from the in-memory database. This database may",
-"be loaded from a file using the Database pulldown on the main menu.",
+"Search for an object by name;",
+"edit it;",
+"or assign it to a User-defined object"
 };
 
-	switch (objp->type) {
+	switch (objp->o_type) {
 	case UNDEFOBJ:
 	    hlp_dialog ("Object", msg, sizeof(msg)/sizeof(msg[0]));
 	    break;
@@ -865,33 +961,33 @@ do_help()
 	    hlp_dialog ("Earth satellite", msg, sizeof(msg)/sizeof(msg[0]));
 	    break;
 	default:
-	    printf ("objmenu: no help for type %d\n", objp->type);
+	    printf ("objmenu: no help for type %d\n", objp->o_type);
 	    exit (1);
 	}
 }
 
-/* callback for which object (x or y) toggle.
- * client is either OBJX or OBJY.
+/* callback for which object (xyz) toggle.
+ * client is one of OBJXYZ.
  */
 /* ARGSUSED */
 static void
-obj_xy_cb (w, client, call)
+obj_xyz_cb (w, client, call)
 Widget w;
 XtPointer client;
 XtPointer call;
 {
 	XmToggleButtonCallbackStruct *t = (XmToggleButtonCallbackStruct *) call;
-	int xy = (int)client;
+	int xyz = (int)client;
 
-	if (t->set && xy != objidx) {
+	if (t->set && xyz != objidx) {
 	    obj_unsetmenu();
-	    objidx = xy;
+	    objidx = xyz;
 	    obj_setmenu();
 	}
 }
 
 /* callback for what type of object toggles.
- * client data is the Obj.type code.
+ * client data is the Obj.o_type code.
  */
 /* ARGSUSED */
 static void
@@ -903,10 +999,10 @@ XtPointer call;
 	XmToggleButtonCallbackStruct *t = (XmToggleButtonCallbackStruct *) call;
 	int type = (int)client;
 
-	if (t->set && objp->type != type) {
+	if (t->set && objp->o_type != type) {
 	    obj_unsetmenu();
 	    zero_mem ((void *)objp, sizeof(Obj));
-	    objp->type = (Objtype_t) type;
+	    objp->o_type = (ObjType_t) type;
 	    obj_setmenu();
 	}
 }
@@ -954,7 +1050,9 @@ XtPointer call;
 	obj_setnames (sp->value);
 }
 
-/* callback when CR is hit in the Srch test field.
+/* callback when CR is hit in the Srch text field or when the Search: PB is
+ * activated.
+ * N.B. Since we are used by two different kinds of widgets don't use w or call.
  */
 /* ARGSUSED */
 static void
@@ -968,7 +1066,7 @@ XtPointer call;
 	char *str;
 	int i;
 	
-	str = XmTextFieldGetString (w);
+	str = XmTextFieldGetString (srch_w);
 
 	/* start from top if new string or smaller list */
 	if (strcmp (str, last_str) || last_idx >= nnamelist) {
@@ -982,13 +1080,15 @@ XtPointer call;
 
 	    i++;
 	    if (last_idx < 0 && i >= nnamelist) {
-		XBell (XtDisplay(w), 0);
+		char msg[1024];
+		(void) sprintf (msg, "%s: not found", str);
+		xe_msg (msg, 1);
 		break;
 	    }
 	    if (i >= nnamelist)
 		i = 0;
 
-	    if (srch_match (str, namelist[i].name)) {
+	    if (srch_match (str, namelist[i].op->o_name)) {
 		int idx, max, slider;
 
 		/* find index to use for setting names and the scroll bar */
@@ -1035,7 +1135,7 @@ char *n1, *n2;
 	char c, *cp;
 	int i;
 
-	for (n1l = 0, cp = n1; (c = *cp++); ) {
+	for (n1l = 0, cp = n1; (c = *cp++) != '\0'; ) {
 	    if (islower(c))
 		c = toupper (c);
 	    if (!isspace(c))
@@ -1043,7 +1143,7 @@ char *n1, *n2;
 	}
 	n1c[n1l] = '\0';
 
-	for (n2l = 0, cp = n2; (c = *cp++); ) {
+	for (n2l = 0, cp = n2; (c = *cp++) != '\0'; ) {
 	    if (islower(c))
 		c = toupper (c);
 	    if (!isspace(c))
@@ -1073,8 +1173,8 @@ const
 #endif
 void *nl1, *nl2;
 {
-	char *s1 = ((NameList *)nl1)->name;
-	char *s2 = ((NameList *)nl2)->name;
+	char *s1 = ((NameList *)nl1)->op->o_name;
+	char *s2 = ((NameList *)nl2)->op->o_name;
 	int p;
 	int q;
 
@@ -1138,6 +1238,7 @@ load_namelist ()
 {
 	DBScan dbs;
 	NameList *nlp;
+	int mask = ALLM & ~PLANETM;
 	Obj *op;
 	int nobj;
 
@@ -1154,10 +1255,7 @@ load_namelist ()
 	nlp = namelist = (NameList *) XtMalloc (nobj*sizeof(NameList));
 
 	/* put everything except the basic objects on the list */
-	for (db_scaninit(&dbs); (op = db_scan (&dbs)); ) {
-	    if (op->type == PLANET || op->type == UNDEFOBJ)
-		continue;
-	    nlp->name = op->o_name;
+	for (db_scaninit(&dbs, mask, NULL, 0); (op = db_scan (&dbs)) != 0; ) {
 	    nlp->op = op;
 	    nlp++;
 	}
@@ -1189,7 +1287,7 @@ int nli;
 	for (bi = 0; bi < NBTNS; bi++) {
 	    int n = 0;
 	    if (bi < nb) {
-		namep = namelist[nli].name;
+		namep = namelist[nli].op->o_name;
 		XtSetArg (args[n], XmNuserData, nli); n++;
 		obj_set_a_button(namepb_w[bi], 1);
 		nli++;
@@ -1326,7 +1424,9 @@ Field *fp;
 	    XtSetArg(args[n], XmNdefaultPosition, False);  n++;
 	    XtSetArg(args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL);  n++;
 	    XtSetArg(args[n], XmNdialogTitle, title);  n++;
+	    XtSetArg (args[n], XmNcolormap, xe_cm); n++;
 	    dw = XmCreatePromptDialog(toplevel_w, "ObjPrompt", args, n);
+	    set_something (dw, XmNcolormap, (XtArgVal)xe_cm);
 	    XtAddCallback (dw, XmNmapCallback, prompt_map_cb, NULL);
 	    XtAddCallback (dw, XmNokCallback, prompt_ok_cb, NULL);
 	    XtAddCallback (dw, XmNcancelCallback, prompt_ok_cb, NULL);
@@ -1338,7 +1438,18 @@ Field *fp;
 
 	set_something (dw, XmNuserData, (XtArgVal)fp);
 	set_xmstring (dw, XmNselectionLabelString, fp->prompt);
-	set_xmstring(dw, XmNtextString, "");
+
+	/* preload with current string -- and skip leading blanks */
+	if (pref_get (PREF_PRE_FILL) == PREF_PREFILL) {
+	    char *txt, *txt0;
+
+	    get_xmstring (fp->pb_w, XmNlabelString, &txt0);
+	    for (txt = txt0; *txt == ' '; txt++)
+		continue;
+	    set_xmstring(dw, XmNtextString, txt);
+	    XtFree (txt0);
+	} else
+	    set_xmstring(dw, XmNtextString, "");
 
 	XtManageChild (dw);
 
@@ -1370,10 +1481,10 @@ int id;
 	    f_ra (w, objp->f_RA);
 	    break;
 	case F_DEC:
-	    f_pangle (w, objp->f_dec);
+	    f_prdec (w, objp->f_dec);
 	    break;
 	case F_MAG:
-	    f_double (w, "%g", objp->f_mag/MAGSCALE);
+	    f_double (w, "%g", get_mag(objp));
 	    break;
 	case F_EPOCH:
 	    epoch_as_decimal (w, objp->f_epoch);
@@ -1395,7 +1506,7 @@ int id;
 	    f_double (w, "%g", objp->e_a);
 	    break;
 	case E_N:
-	    f_double (w, "%g", objp->e_n);
+	    /* retired */
 	    break;
 	case E_E:
 	    f_double (w, "%g", objp->e_e);
