@@ -19,8 +19,7 @@
 
 #include "P_.h"
 #include "astro.h"
-
-extern char *myclass;
+#include "ps.h"
 
 extern int get_color_resource P_((Widget w, char *cname, Pixel *p));
 extern int tickmarks P_((double min, double max, int numdiv, double ticks[]));
@@ -64,7 +63,7 @@ int xasdate;		/* whether to show x axis as dates */
 	int nt;
 	static char fmt[] = "%c,%lf,%lf";
 	double x, y;	/* N.B. be sure these match what scanf's %lf wants*/
-	double minx, maxx, miny, maxy, xscale, yscale;
+	double minx=0, maxx=0, miny=0, maxy=0, xscale, yscale;
 	char buf[128];
 	int lx[MAXPLTLINES], ly[MAXPLTLINES], one[MAXPLTLINES];
 	char c, tags[MAXPLTLINES];
@@ -119,8 +118,12 @@ int xasdate;		/* whether to show x axis as dates */
 	    return(-1);
 	}
 
+	/* build the gcs, fonts etc if this is the first time. */
 	if (!plt_gc)
 	    mk_gsc (widget);
+	
+	/* pick a font to use in case we are printing */
+	XPSRegisterFont (plt_fs->fid, "Helvetica");
 
 	/* add y-axis tick marks.
 	 * first compute length of longest y-axis label and other char stuff.
@@ -156,10 +159,10 @@ int xasdate;		/* whether to show x axis as dates */
 	    (void) sprintf (buf, "%g", ticks[i]);
 	    l = strlen(buf);
 	    iy = YCORD(ticks[i]);
-	    XDrawLine (dsp, win, plt_gc, x0-TL, iy, x0, iy);
-	    XDrawString (dsp, win, plt_gc, 1, iy+(asc-desc)/2, buf, l);
+	    XPSDrawLine (dsp, win, plt_gc, x0-TL, iy, x0, iy);
+	    XPSDrawString (dsp, win, plt_gc, 1, iy+(asc-desc)/2, buf, l);
 	    if (grid)
-		XDrawLine (dsp, win, plt_gc, x0, iy, x0+w-1, iy);
+		XPSDrawLine (dsp, win, plt_gc, x0, iy, x0+w-1, iy);
 	}
 
 	/* now draw x axis and label it's first and last tick mark.
@@ -172,9 +175,9 @@ int xasdate;		/* whether to show x axis as dates */
 	    if (ticks[i] < minx || ticks[i] > maxx)
 		continue;
 	    ix = XCORD(ticks[i]);
-	    XDrawLine (dsp, win, plt_gc, ix, y0+TL, ix, y0);
+	    XPSDrawLine (dsp, win, plt_gc, ix, y0+TL, ix, y0);
 	    if (grid)
-		XDrawLine (dsp, win, plt_gc, ix, y0, ix, y0-h-1);
+		XPSDrawLine (dsp, win, plt_gc, ix, y0, ix, y0-h-1);
 	}
 	for (i = 0; i < nt; i++)
 	    if (ticks[i] >= minx) {
@@ -193,10 +196,10 @@ int xasdate;		/* whether to show x axis as dates */
 	    						XCORD(ticks[nt/2]), y0);
 
 	/* draw border of actual plotting area */
-	XDrawLine (dsp, win, plt_gc, x0, y0-h, x0, y0);
-	XDrawLine (dsp, win, plt_gc, x0, y0, x0+w, y0);
-	XDrawLine (dsp, win, plt_gc, x0+w, y0, x0+w, y0-h);
-	XDrawLine (dsp, win, plt_gc, x0+w, y0-h, x0, y0-h);
+	XPSDrawLine (dsp, win, plt_gc, x0, y0-h, x0, y0);
+	XPSDrawLine (dsp, win, plt_gc, x0, y0, x0+w, y0);
+	XPSDrawLine (dsp, win, plt_gc, x0+w, y0, x0+w, y0-h);
+	XPSDrawLine (dsp, win, plt_gc, x0+w, y0-h, x0, y0-h);
 
 	/* read file again, this time plotting the data (finally!).
 	 * also, the first line we see that doesn't look like a point
@@ -215,7 +218,7 @@ int xasdate;		/* whether to show x axis as dates */
 		    int di, as, de;
 		    XCharStruct ovl;
 		    XTextExtents (plt_fs, buf+2, l-2, &di, &as, &de, &ovl);
-		    XDrawString (dsp, win, plt_gc, x0+(w-ovl.width)/2, asc+1,
+		    XPSDrawString (dsp, win, plt_gc, x0+(w-ovl.width)/2, asc+1,
 								buf+2, l-3);
 		    sawtitle = 1;
 		}
@@ -227,10 +230,10 @@ int xasdate;		/* whether to show x axis as dates */
 	    ix = XCORD(x);
 	    iy = YCORD(y);
 	    if (one[i]++ > 0)
-		XDrawLine (dsp, win, plt_gcc[i], ix, iy, lx[i], ly[i]);
+		XPSDrawLine (dsp, win, plt_gcc[i], ix, iy, lx[i], ly[i]);
 	    else {
 		int ytop = y0 - h + asc;
-		XDrawString(dsp,win,plt_gcc[i], ix+2, (iy<ytop ? ytop : iy)-2,
+		XPSDrawString(dsp,win,plt_gcc[i], ix+2, (iy<ytop ? ytop : iy)-2,
 									&c, 1);
 	    }
 	    lx[i] = ix;
@@ -247,7 +250,7 @@ Widget w;
 	Window win = XtWindow(w);
 	unsigned long da_p;	/* the foreground color of the drawing area */
 	unsigned long da_bgp;/* the background color of the drawing area */
-	unsigned gcm = GCForeground;
+	unsigned long gcm = GCForeground;
 	XGCValues gcv;
 	int i;
 
@@ -367,8 +370,8 @@ int x, y;
 	    (void) sprintf (buf, "%g", v);
 	l = strlen(buf);
 	XTextExtents (plt_fs, buf, l, &dir, &asc, &des, &ovl);
-	XDrawLine (dsp, win, plt_gc, x, y+TL, x, y+2*TL);
-	XDrawString (dsp, win, plt_gc, x-ovl.width/2, y+2*TL+asc+1, buf, l);
+	XPSDrawLine (dsp, win, plt_gc, x, y+TL, x, y+2*TL);
+	XPSDrawString (dsp, win, plt_gc, x-ovl.width/2, y+2*TL+asc+1, buf, l);
 
 	return (ovl.width);
 }
