@@ -23,9 +23,11 @@
 #include <Xm/Text.h>
 
 #include "P_.h"
+#include "ps.h"
 
 extern Widget	toplevel_w;
 #define	XtD	XtDisplay(toplevel_w)
+extern Colormap xe_cm;
 
 extern FILE *fopenh P_((char *name, char *how));
 extern char *syserrstr P_((void));
@@ -38,9 +40,12 @@ extern void get_something P_((Widget w, char *resource, XtArgVal value));
 extern void hlp_dialog P_((char *tag, char *deflt[], int ndeflt));
 extern void prompt_map_cb P_((Widget w, XtPointer client, XtPointer call));
 extern void query P_((Widget tw, char *msg, char *label1, char *label2,
-    char *label3, void (*func1)(), void (*func2)(), void (*func3)()));
+    char *label3, void (*func1)(void), void (*func2)(void),
+    void (*func3)(void)));
 extern void set_something P_((Widget w, char *resource, XtArgVal value));
 extern void set_xmstring P_((Widget w, char *resource, char *txt));
+extern void watch_cursor P_((int want));
+extern void wtip P_((Widget w, char *string));
 extern void xe_msg P_((char *msg, int app_modal));
 
 static void plt_select P_((int whether));
@@ -51,20 +56,24 @@ static void plt_help_cb P_((Widget w, XtPointer client, XtPointer call));
 static void plt_reset P_((void));
 static void plt_stop_selecting P_((void));
 static void plt_turn_off P_((void));
-static void init_next_tag P_((void));
+static void init_next_row P_((void));
 static void plt_try_append P_((void));
 static void plt_try_overwrite P_((void));
 static void plt_try_cancel P_((void));
 static void plt_try_turn_on P_((void));
 static void plt_turn_on P_((char *how));
 static void plt_da_manage P_((void));
-static void plt_da_destroy P_((Widget daform_w));
+static void plt_da_destroy P_((Widget da_w));
+static void plt_print P_((void));
+static void plt_da_print_cb P_((Widget w, XtPointer client, XtPointer call));
 static void plt_da_close_cb P_((Widget w, XtPointer client, XtPointer call));
+static void plt_da_unmap_cb P_((Widget w, XtPointer client, XtPointer call));
 static void plt_da_flipx_cb P_((Widget w, XtPointer client, XtPointer call));
 static void plt_da_flipy_cb P_((Widget w, XtPointer client, XtPointer call));
 static void plt_da_grid_cb P_((Widget w, XtPointer client, XtPointer call));
 static void plt_da_xasdate_cb P_((Widget w, XtPointer client, XtPointer call));
 static void plt_da_exp_cb P_((Widget da_w, XtPointer client, XtPointer call));
+static void plt_drawall P_((Widget da_w));
 
 
 /* max number of things we can keep track of at once to plot */
@@ -74,15 +83,17 @@ static void plt_da_exp_cb P_((Widget da_w, XtPointer client, XtPointer call));
 static Widget plotform_w;
 static Widget select_w, active_w, prompt_w;
 static Widget title_w, filename_w;
-static Widget table_w[MAXPLTLINES][3];	/* column indeces follow.. */
-static int selecting_xy;		/* also one of ... */
-#define	T	0
-#define	X	1
-#define	Y	2
+static Widget table_w[MAXPLTLINES][4];	/* column indeces follow.. */
+#define	T	0		/* tag label text field */
+#define	X	1		/* X label */
+#define	Y	2		/* Y label */
+#define	FORM	3		/* the form that holds the three above */
+static int selecting_xy;	/* one of X or Y */
 
 #define	DEF_PLTFN	"ephem.plt"	/* default plot file name */
 static FILE *plt_fp;            /* the plot file; == 0 means don't plot */
 
+static Widget da_w_save;	/* used to set up for a print action */
 
 /* plt_activate_cb client values. */
 enum { SELECT, ACTIVE, SHOW};
@@ -158,7 +169,7 @@ char *name;
 		plt_stop_selecting();
 	    else {
 		selecting_xy = X;
-		init_next_tag();
+		init_next_row();
 	    }
 	}
 }
@@ -214,7 +225,7 @@ Cursor c;
 {
 	Window win;
 
-	if (plotform_w && (win = XtWindow(plotform_w))) {
+	if (plotform_w && (win = XtWindow(plotform_w)) != 0) {
 	    Display *dsp = XtDisplay(plotform_w);
 	    if (c)
 		XDefineCursor (dsp, win, c);
@@ -240,10 +251,13 @@ plot_create_form()
 	    char *title;
 	    int cb_data;
 	    Widget *wp;
+	    char *tip;
 	} tbs[] = {
-	    {"Select fields", SELECT, &select_w},
-	    {"Plot to file", ACTIVE, &active_w},
-	    {"Show plot file", SHOW, NULL},
+	    {"Select fields", SELECT, &select_w,
+	    	"When on, data fields eligible for plotting are selectable buttons"},
+	    {"Plot to file", ACTIVE, &active_w,
+	    	"When on, selected fields are written to the named file at each main Update"},
+	    {"Show plot file", SHOW, NULL, "Display the named plot file"},
 	};
 	XmString str;
 	Widget f_w, rc_w;
@@ -257,7 +271,13 @@ plot_create_form()
 	n = 0;
 	XtSetArg (args[n], XmNautoUnmanage, False); n++;
 	XtSetArg (args[n], XmNdefaultPosition, False); n++;
+	XtSetArg (args[n], XmNallowShellResize, True); n++;
+	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
+	XtSetArg (args[n], XmNmarginHeight, 10); n++;
+	XtSetArg (args[n], XmNmarginWidth, 10); n++;
+	XtSetArg (args[n], XmNverticalSpacing, 10); n++;
 	plotform_w = XmCreateFormDialog (toplevel_w, "Plot", args, n);
+	set_something (plotform_w, XmNcolormap, (XtArgVal)xe_cm);
 	XtAddCallback (plotform_w, XmNhelpCallback, plt_help_cb, 0);
 
 	n = 0;
@@ -271,6 +291,7 @@ plot_create_form()
 	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
+	XtSetArg (args[n], XmNspacing, 8); n++;
 	XtSetArg (args[n], XmNisAligned, False); n++;
 	XtSetArg (args[n], XmNadjustMargin, False); n++;
 	rc_w = XmCreateRowColumn(plotform_w, "PlotRC", args, n);
@@ -285,11 +306,13 @@ plot_create_form()
 	    XtSetArg (args[n], XmNlabelString, str); n++;
 	    w = XmCreateToggleButton(rc_w, "PlotTB", args, n);
 	    XmStringFree (str);
-	    XtManageChild (w);
 	    XtAddCallback(w, XmNvalueChangedCallback, plt_activate_cb,
 						    (XtPointer)tbs[i].cb_data);
 	    if (tbs[i].wp)
 		*tbs[i].wp = w;
+	    if (tbs[i].tip)
+		wtip (w, tbs[i].tip);
+	    XtManageChild (w);
 	}
 
 	/* create filename text area and its label */
@@ -308,6 +331,7 @@ plot_create_form()
 	if (strlen(deffn) == 0)
 	    XmTextSetString (filename_w, DEF_PLTFN);
 	XtFree (deffn);
+	wtip (filename_w, "Enter name of file to write");
 	XtManageChild (filename_w);
 
 	/* create title text area and its label */
@@ -322,6 +346,7 @@ plot_create_form()
 
 	n = 0;
 	title_w = XmCreateText (rc_w, "PlotTitle", args, n);
+	wtip (title_w, "Enter a title to be written to the file");
 	XtManageChild (title_w);
 
 	/* create prompt line -- it will be managed as necessary */
@@ -333,14 +358,19 @@ plot_create_form()
 	/* create the table.
 	 * each row is in a form to control its shape.
 	 * loop index of -1 is used to make the column headings.
-	 * the table entries are not managed at this time.
+	 * the table entries are all managed but the forms are not at this time.
 	 */
 
 	for (i = -1; i < MAXPLTLINES; i++) {
 	    n = 0;
 	    XtSetArg (args[n], XmNfractionBase, 9); n++;
 	    f_w = XmCreateForm (rc_w, "PlotTF", args, n);
-	    XtManageChild (f_w);
+
+	    /* save the form unless we are just making the column headings */
+	    if (i == -1)
+		XtManageChild (f_w);
+	    else
+		table_w[i][FORM] = f_w;
 
 	    n = 0;
 	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
@@ -350,12 +380,15 @@ plot_create_form()
 	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
 	    XtSetArg (args[n], XmNrightPosition, 1); n++;
 	    if (i == -1) {
+		XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
 		w = XmCreateLabel (f_w, "Tag", args, n);
+		wtip (w, "Column of tags for each X/Y pair to plot");
 		XtManageChild (w);
 	    } else {
 		XtSetArg (args[n], XmNmaxLength, 1); n++;
 		XtSetArg (args[n], XmNcolumns, 1); n++;
 		table_w[i][T] = XmCreateText (f_w, "PlotTag", args, n);
+		XtManageChild (table_w[i][T]);
 	    }
 
 	    n = 0;
@@ -365,11 +398,14 @@ plot_create_form()
 	    XtSetArg (args[n], XmNleftPosition, 2); n++;
 	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
 	    XtSetArg (args[n], XmNrightPosition, 5); n++;
+	    XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
 	    if (i == -1) {
-		w = XmCreateLabel (f_w, "X", args, n);
+		w = XmCreateLabel (f_w, "_X_", args, n);
+		wtip (w, "Column of X fields for each X/Y pair to plot");
 		XtManageChild (w);
 	    } else {
 		table_w[i][X] = XmCreateLabel (f_w, "PlotX", args, n);
+		XtManageChild (table_w[i][X]);
 	    }
 
 	    n = 0;
@@ -379,11 +415,14 @@ plot_create_form()
 	    XtSetArg (args[n], XmNleftPosition, 6); n++;
 	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
 	    XtSetArg (args[n], XmNrightPosition, 9); n++;
+	    XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
 	    if (i == -1) {
-		w = XmCreateLabel (f_w, "Y", args, n);
+		w = XmCreateLabel (f_w, "_Y_", args, n);
+		wtip (w, "Column of Y fields for each X/Y pair to plot");
 		XtManageChild (w);
 	    } else {
 		table_w[i][Y] = XmCreateLabel (f_w, "PlotY", args, n);
+		XtManageChild (table_w[i][Y]);
 	    }
 	}
 
@@ -408,6 +447,7 @@ plot_create_form()
 	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
 	    XtSetArg (args[n], XmNrightPosition, 3); n++;
 	    w = XmCreatePushButton (f_w, "Close", args, n);
+	    wtip (w, "Close this dialog ((but continue plotting if active)");
 	    XtAddCallback (w, XmNactivateCallback, plt_close_cb, 0);
 	    XtManageChild (w);
 
@@ -419,6 +459,7 @@ plot_create_form()
 	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
 	    XtSetArg (args[n], XmNrightPosition, 6); n++;
 	    w = XmCreatePushButton (f_w, "Help", args, n);
+	    wtip (w, "More detailed usage information");
 	    XtAddCallback (w, XmNactivateCallback, plt_help_cb, 0);
 	    XtManageChild (w);
 }
@@ -443,7 +484,7 @@ XtPointer call;
 		    XmToggleButtonSetState(active_w, False, True);
 		plt_reset();	/* reset pltlines array and unmanage the table*/
 		plt_select(1);	/* inform other menus to inform us of fields */
-		init_next_tag();/* set first tag to something and show it */
+		init_next_row();/* set first tag to something and show it */
 		selecting_xy = X;
 	    } else
 		plt_stop_selecting();
@@ -504,11 +545,8 @@ plt_reset()
 {
 	int i;
 
-	for (i = 0; i < npltlines; i++) {
-	    XtUnmanageChild (table_w[i][T]);
-	    XtUnmanageChild (table_w[i][X]);
-	    XtUnmanageChild (table_w[i][Y]);
-	}
+	for (i = 0; i < npltlines; i++)
+	    XtUnmanageChild (table_w[i][FORM]);
 
 	npltlines = 0;
 }
@@ -519,12 +557,8 @@ plt_reset()
 static void
 plt_stop_selecting()
 {
-	/* harmless to unmanage something already unmanaged so do them all */
-	if (npltlines < MAXPLTLINES) {
-	    XtUnmanageChild (table_w[npltlines][T]);
-	    XtUnmanageChild (table_w[npltlines][X]);
-	    XtUnmanageChild (table_w[npltlines][Y]);
-	}
+	if (npltlines < MAXPLTLINES)
+	    XtUnmanageChild (table_w[npltlines][FORM]);
 
 	XmToggleButtonSetState (select_w, False, False);
 	plt_select(0);
@@ -541,16 +575,20 @@ plt_turn_off ()
 }
 
 static void
-init_next_tag()
+init_next_row()
 {
 	char buf[100];
-	Widget w = table_w[npltlines][T];
 
 	XtManageChild (prompt_w);
 	f_string (prompt_w, "Select quantity for X..");
+
 	(void) sprintf (buf, "%c", 'A' + npltlines);
-	XmTextSetString (w, buf);
-	XtManageChild (w);
+	XmTextSetString (table_w[npltlines][T], buf);
+
+	f_string (table_w[npltlines][X], " ");
+	f_string (table_w[npltlines][Y], " ");
+
+	XtManageChild (table_w[npltlines][FORM]);
 }
 
 /* called from the query routine when want to append to an existing plot file.*/
@@ -666,7 +704,9 @@ plt_da_manage()
 
 	/* create the form dialog parent. */
 	n = 0;
+	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
 	daform_w = XmCreateFormDialog (toplevel_w, "PlotDisplay", args, n);
+	set_something (daform_w, XmNcolormap, (XtArgVal)xe_cm);
 	XtAddCallback (daform_w, XmNmapCallback, prompt_map_cb, NULL);
 
 	/* set some stuff in the parent DialogShell.
@@ -716,6 +756,10 @@ plt_da_manage()
 	XtAddCallback (da_w, XmNexposeCallback, plt_da_exp_cb, NULL);
 	XtManageChild (da_w);
 
+	/* when unmap the form, pass the da to the unmap callback */
+	XtAddCallback (daform_w, XmNunmapCallback, plt_da_unmap_cb,
+							    (XtPointer)da_w);
+
 	/* make the Control pulldown */
 
 	n = 0;
@@ -728,11 +772,25 @@ plt_da_manage()
 	    set_xmstring (cb_w, XmNlabelString, "Control");
 	    XtManageChild (cb_w);
 
+	    /* make the print button */
+	    n = 0;
+	    w = XmCreatePushButton (pd_w, "PltPrint", args, n);
+	    XtAddCallback (w, XmNactivateCallback, plt_da_print_cb, da_w);
+	    set_xmstring (w, XmNlabelString, "Print...");
+	    wtip (w, "Print this plot");
+	    XtManageChild (w);
+
+	    /* add a separator */
+	    n = 0;
+	    w = XmCreateSeparator (pd_w, "Sep", args, n);
+	    XtManageChild (w);
+
 	    /* make the close button.
 	     * it destroys the dialog and frees the DrawInfo struct.
 	     */
 	    n = 0;
 	    w = XmCreatePushButton (pd_w, "Close", args, n);
+	    wtip (w, "Close this plot display");
 	    XtAddCallback (w, XmNactivateCallback, plt_da_close_cb, daform_w);
 	    XtManageChild (w);
 
@@ -757,6 +815,7 @@ plt_da_manage()
 	    di->flipx = XmToggleButtonGetState (w);
 	    XmStringFree (str);
 	    XtAddCallback (w, XmNvalueChangedCallback, plt_da_flipx_cb, da_w);
+	    wtip (w, "Flip left-to-right");
 	    XtManageChild (w);
 
 	    str = XmStringCreate("Flip Y", XmSTRING_DEFAULT_CHARSET);
@@ -768,6 +827,7 @@ plt_da_manage()
 	    di->flipy = XmToggleButtonGetState (w);
 	    XmStringFree (str);
 	    XtAddCallback (w, XmNvalueChangedCallback, plt_da_flipy_cb, da_w);
+	    wtip (w, "Flip top-to-bottom");
 	    XtManageChild (w);
 
 	    n = 0;
@@ -776,6 +836,7 @@ plt_da_manage()
 	    w = XmCreateToggleButton(pd_w, "Grid", args, n);
 	    di->grid = XmToggleButtonGetState (w);
 	    XtAddCallback (w, XmNvalueChangedCallback, plt_da_grid_cb, da_w);
+	    wtip (w, "Overlay plot with calibrated grid");
 	    XtManageChild (w);
 
 	    str = XmStringCreate("Show X-Axis as Dates",
@@ -788,20 +849,22 @@ plt_da_manage()
 	    di->xasdate = XmToggleButtonGetState (w);
 	    XmStringFree (str);
 	    XtAddCallback (w, XmNvalueChangedCallback, plt_da_xasdate_cb, da_w);
+	    wtip (w, "Assume ordinate values are decimal years and display as dates");
 	    XtManageChild (w);
 
 	/* go. the expose will do the actual plotting */
 	XtManageChild (daform_w);
 }
 
-/* called when finished with a plot with its FormDialog widget.
- * reclaim all memory in the userData DrawInfo and destroy it.
- * N.B. caller's daform_w is useless when this returns.
+/* called with the DrawingArea when finished with a plot.
+ * use it to get the FormDialog parent and reclaim all memory in its userData
+ *   DrawInfo and destroy the Form and hence everything in it.
  */
 static void
-plt_da_destroy (daform_w)
-Widget daform_w;
+plt_da_destroy (da_w)
+Widget da_w;
 {
+	Widget daform_w = XtParent(da_w);
 	DrawInfo *di;
 
 	get_something (daform_w, XmNuserData, (XtArgVal)&di);
@@ -813,10 +876,65 @@ Widget daform_w;
 	while (!XtIsWMShell (daform_w))
 	    daform_w = XtParent(daform_w);
 	XtDestroyWidget(daform_w);
+
+	/* invalidate the saved print widget if it matches */
+	if (da_w_save == da_w)
+	    da_w_save = 0;
+}
+
+/* proceed to generate a postscript file from da_w_save if it is still up.
+ * reset da_w_save and call XPSClose() when finished.
+ */
+static void
+plt_print()
+{
+	if (da_w_save) {
+	    Dimension w, h;
+	    int iw, ih;
+
+	    get_something (da_w_save, XmNwidth, (XtArgVal)&w);
+	    get_something (da_w_save, XmNheight, (XtArgVal)&h);
+	    iw = (int)w;
+	    ih = (int)h;
+
+	    /* draw in an area 6.5w x 8h centered 1in down from top */
+	    if (16*iw >= 13*ih)
+		XPSXBegin (XtWindow(da_w_save), 0, 0, iw, ih, 1*72, 10*72,
+							    (int)(6.5*72), 12);
+	    else {
+		int pw = 72*8*iw/ih;	/* width on paper when 8 hi */
+		XPSXBegin (XtWindow(da_w_save), 0, 0, iw, ih,
+					(int)((8.5*72-pw)/2), 10*72, pw, 12);
+	    }
+
+	    plt_drawall (da_w_save);
+	    XPSXEnd();
+	    XPSClose();
+	    da_w_save = 0;
+	} else {
+	    xe_msg ("Can not print after plot has been dismissed.", 1);
+	    XPSClose();
+	}
+}
+
+/* called when the Print button is pushed on a plot.
+ * store the da_w (passed in client) and ask for print setup info.
+ */
+/* ARGSUSED */
+static void
+plt_da_print_cb (w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
+{
+	Widget da_w = (Widget)client;
+
+	da_w_save = da_w;
+	XPSAsk ("Plot", plt_print);
 }
 
 /* called when the Close button is pushed on a plot.
- * free the DrawInfo and destroy the dialog (passed as client).
+ * just unmap the dialog (passed as client).
  */
 /* ARGSUSED */
 static void
@@ -824,6 +942,19 @@ plt_da_close_cb (w, client, call)
 Widget w;
 XtPointer client;
 XtPointer call;
+{
+	XtUnmanageChild ((Widget)client);
+}
+
+/* called when a plot is unmapped, either via a WM command or the Close button.
+ * free the DrawInfo and destroy the DrawingArea (passed as client).
+ */
+/* ARGSUSED */
+static void
+plt_da_unmap_cb (w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
 {
 	plt_da_destroy ((Widget)client);
 }
@@ -996,29 +1127,24 @@ XtPointer client;
 XtPointer call;
 {
 	XmDrawingAreaCallbackStruct *c = (XmDrawingAreaCallbackStruct *)call;
-	DrawInfo *di;
-	Widget daform_w;
-	XWindowAttributes xwa;
 
 	switch (c->reason) {
 	case XmCR_EXPOSE: {
-	    /* turn off gravity so we get expose events for either shrink or
-	     * expand if we have never seen this window before.
-	     */
 	    XExposeEvent *e = &c->event->xexpose;
+	    XSetWindowAttributes swa;
+	    unsigned long mask = CWBitGravity;
+
+	    /* turn off gravity so we get fresh exposes when grow and shrink.
+	     * do it every time because this expose handler is used for all
+	     * the windows -- yes, it is over kill but not worth remembering
+	     * which windows have already been set.
+	     */
+	    swa.bit_gravity = ForgetGravity;
+	    XChangeWindowAttributes (e->display, e->window, mask, &swa);
 
 	    /* wait for the last in the series */
 	    if (e->count != 0)
 		return;
-
-	    XGetWindowAttributes (e->display, e->window, &xwa);
-	    if (xwa.bit_gravity != ForgetGravity) {
-		/* first expose for this window -- set no gravity */
-		XSetWindowAttributes swa;
-		swa.bit_gravity = ForgetGravity;
-		XChangeWindowAttributes (e->display, e->window,
-							    CWBitGravity, &swa);
-	    }
 	    break;
 	    }
 	default:
@@ -1026,22 +1152,40 @@ XtPointer call;
 	    exit(1);
 	}
 
-	/* use xwa for the drawing area window size.
-	 * get di from the FormDiaglog parent and plot fresh.
-	 */
+	/* draw the plot from scratch */
+	plt_drawall (da_w);
+}
+
+/* given a drawing area, draw the plot.
+ * remember: the parent of the da is the form and it's userData is the di info.
+ */
+static void
+plt_drawall (da_w)
+Widget da_w;
+{
+	Dimension w, h;
+	Widget daform_w;
+	DrawInfo *di;
+
+	watch_cursor (1);
+
+	get_something (da_w, XmNwidth, (XtArgVal)&w);
+	get_something (da_w, XmNheight, (XtArgVal)&h);
 
 	daform_w = XtParent(da_w);
 	get_something (daform_w, XmNuserData, (XtArgVal)&di);
 	XClearWindow (XtDisplay(da_w), XtWindow(da_w));
 	rewind (di->fp);
-	if (plot_cartesian (di->fp, da_w, xwa.width, xwa.height,
-			    di->flipx, di->flipy, di->grid, di->xasdate) < 0) {
+	if (plot_cartesian (di->fp, da_w, w, h, di->flipx, di->flipy, di->grid,
+							    di->xasdate) < 0) {
 	    /* had trouble, so done with this FormDialog.
 	     */
 	    char buf[128];
 	    (void) sprintf (buf, "Error plotting `%.*s'\n", (int)sizeof(buf)-20,
 								di->filename);
 	    xe_msg (buf, 0);
-	    plt_da_destroy (daform_w);
+	    plt_da_destroy (da_w);
 	}
+
+	watch_cursor (0);
 }
