@@ -6,18 +6,19 @@
 #define _PREFERENCES_H
 
 typedef enum {
-    PREF_ALGO, PREF_UNITS, PREF_DATE_FORMAT, PREF_ZONE,
-    PREF_DPYPREC, PREF_MSG_BELL
+    PREF_EQUATORIAL, PREF_UNITS, PREF_DATE_FORMAT, PREF_ZONE,
+    PREF_DPYPREC, PREF_MSG_BELL, PREF_PRE_FILL, PREF_TIPS,
+    NPREFS
 } Preferences;
 
-#define	NPREFS	6	/* number of entries in Preferences */
-
-typedef enum {PREF_ACCURATE, PREF_FAST} PrefAlgo;
+typedef enum {PREF_GEO, PREF_TOPO} PrefEquatorial;
 typedef enum {PREF_ENGLISH, PREF_METRIC} PrefUnits;
 typedef enum {PREF_MDY, PREF_YMD, PREF_DMY} PrefDateFormat;
 typedef enum {PREF_LOCALTZ, PREF_UTCTZ} PrefStampZone;
 typedef enum {PREF_LOPREC, PREF_HIPREC} PrefDpyPrec;
 typedef enum {PREF_NOMSGBELL, PREF_MSGBELL} PrefMsgBell;
+typedef enum {PREF_PREFILL, PREF_NOPREFILL} PrefPreFill;
+typedef enum {PREF_TIPSON, PREF_NOTIPS} PrefTips;
 
 int pref_get();
 
