@@ -23,25 +23,32 @@
 #include "P_.h"
 #include "astro.h"
 #include "circum.h"
+#include "preferences.h"
+#include "ps.h"
 
 
 extern Widget	toplevel_w;
 #define	XtD	XtDisplay(toplevel_w)
+extern Colormap xe_cm;
 
 extern Now *mm_get_now P_((void));
 extern Obj *db_basic P_((int id));
 extern Obj *db_scan P_((DBScan *sp));
+extern double delra P_((double dra));
 extern int any_ison P_((void));
-extern int magdiam P_((int fmag, double scale, double mag, double size));
-extern int obj_cir P_((Now *np, Obj *op));
+extern int fs_fetch P_((Now *np, double ra, double dec, double fov,
+    double mag, ObjF **opp));
+extern int magdiam P_((int fmag, int magstp, double scale, double mag,
+    double size));
 extern int skyloc_fifo P_((int test, double ra, double dec, double yr,
     double fov, int mag));
-extern void db_scaninit P_((DBScan *sp));
+extern void db_scaninit P_((DBScan *sp, int mask, ObjF *op, int nop));
 extern void db_update P_((Obj *op));
+extern void f_prdec P_((Widget w, double a));
 extern void f_double P_((Widget w, char *fmt, double f));
-extern void f_pangle P_((Widget w, double a));
 extern void f_ra P_((Widget w, double ra));
-extern void fs_pangle P_((char out[], double a));
+extern void fs_manage P_((void));
+extern void fs_prdec P_((char out[], double a));
 extern void fs_ra P_((char out[], double ra));
 extern void get_something P_((Widget w, char *resource, XtArgVal value));
 extern void hlp_dialog P_((char *tag, char *deflt[], int ndeflt));
@@ -55,6 +62,7 @@ extern void sv_draw_obj P_((Display *dsp, Drawable win, GC gc, Obj *op, int x,
     int y, int diam, int dotsonly));
 extern void timestamp P_((Now *np, Widget w));
 extern void watch_cursor P_((int want));
+extern void wtip P_((Widget w, char *string));
 extern void xe_msg P_((char *buf, int app_modal));
 
 #define	MINR	10	/* min distance for picking, pixels */
@@ -84,6 +92,7 @@ static void sm_sstats_cb P_((Widget w, XtPointer client, XtPointer call));
 static void sm_option_cb P_((Widget w, XtPointer client, XtPointer call));
 static void sm_scale_cb P_((Widget w, XtPointer client, XtPointer call));
 static void sm_activate_cb P_((Widget w, XtPointer client, XtPointer call));
+static void sm_unmap_cb P_((Widget w, XtPointer client, XtPointer call));
 static void sm_close_cb P_((Widget w, XtPointer client, XtPointer call));
 static void sm_movie_cb P_((Widget w, XtPointer client, XtPointer call));
 static void sm_da_exp_cb P_((Widget w, XtPointer client, XtPointer call));
@@ -93,6 +102,8 @@ static void sm_fill_popup P_((ScreenObj *sop));
 static void sm_help_cb P_((Widget w, XtPointer client, XtPointer call));
 static void sm_helpon_cb P_((Widget w, XtPointer client, XtPointer call));
 static void sm_locfifo_cb P_((Widget w, XtPointer client, XtPointer call));
+static void sm_calibline P_((Display *dsp, Drawable win, GC gc, int xc, int yc,
+    char *tag, int tw, int th, int l));
 
 static void add_screenobj P_((Obj *op, int x, int y));
 static void reset_screenobj P_((void));
@@ -111,6 +122,10 @@ static Widget limmag_w;		/* limiting magnitude scale */
 static Widget dt_w;		/* main date/time stamp widget */
 static Widget sdt_w;		/* statistics date/time stamp widget */
 static Widget skybkg_w;		/* toggle for controlling sky background */
+static Pixmap sm_pm;		/* main pixmap */
+static GC s_fgc, s_bgc, s_xgc;  /* various colors and operators */
+static XFontStruct *s_fs;       /* font for labels */
+static int s_cw, s_ch;          /* size of label font */
 static int sm_selecting;	/* set while our fields are being selected */
 static int brmoons;		/* whether we want to brightten the moons */
 static int tags;		/* whether we want tags on the drawing */
@@ -128,6 +143,15 @@ static Widget smpu_mag_w;	/* popup Mag label */
 #define	MAXSCALE	10.0	/* max scale mag factor */
 #define	MOVIE_STEPSZ	0.25	/* movie step size, hours */
 
+/* field star support */
+static ObjF *fstars;            /* malloced list of field stars, or NULL */
+static int nfstars;             /* number of entries in fstars[] */
+static double fsdec, fsra;      /* location when field stars were loaded */
+#define FSFOV   degrad(1.0)     /* size of FOV to fetch, rads */
+#define FSMAG   20.0            /* limiting mag for fetch */
+#define FSMOVE  degrad(.2)      /* reload when jup has moved this far, rads */
+static void sm_loadfs P_((double ra, double dec));
+
 /* arrays that use NM use [0] for the planet itself, and [1..NM-1] for moons.
  */
 #define	NM	9		/* number of moons + 1 for planet */
@@ -168,7 +192,7 @@ sm_manage ()
 	
 	if (XtIsManaged(satform_w)) {
 	    XtUnmanageChild (satform_w);
-	    if ((sswasman = XtIsManaged(ssform_w)))
+	    if ((sswasman = XtIsManaged(ssform_w)) != 0)
 		XtUnmanageChild(ssform_w);
 	} else {
 	    XtManageChild (satform_w);
@@ -221,7 +245,7 @@ Cursor c;
 {
 	Window win;
 
-	if (satform_w && (win = XtWindow(satform_w))) {
+	if (satform_w && (win = XtWindow(satform_w)) != 0) {
 	    Display *dsp = XtDisplay(satform_w);
 	    if (c)
 		XDefineCursor (dsp, win, c);
@@ -229,7 +253,7 @@ Cursor c;
 		XUndefineCursor (dsp, win);
 	}
 
-	if (ssform_w && (win = XtWindow(ssform_w))) {
+	if (ssform_w && (win = XtWindow(ssform_w)) != 0) {
 	    Display *dsp = XtDisplay(ssform_w);
 	    if (c)
 		XDefineCursor (dsp, win, c);
@@ -247,13 +271,19 @@ sm_create_form_w()
 	    char *label;	/* label */
 	    int *flagp;		/* pointer to global flag, or NULL if none */
 	    Widget *wp;		/* widget to save, or NULL */
+	    char *tip;          /* widget tip */
 	} Option;
 	static Option options[] = {
-	    {"SkyBkg",	   "Sky background",	&skybkg,  &skybkg_w},
-	    {"BrightMoons","Bright moons",	&brmoons},
-	    {"Tags",	   "Tags",		&tags},
-	    {"FlipTB",	   "Flip T/B",		&flip_tb},
-	    {"FlipLR",	   "Flip L/R",		&flip_lr},
+	    {"SkyBkg",	   "Sky background",	&skybkg,  &skybkg_w,
+	    	"When on, sky will include database objects and Field Stars"},
+	    {"BrightMoons","Bright moons",	&brmoons, NULL,
+		"Display moons disproportionately bright, even if below limit"},
+	    {"Tags",	   "Tags",		&tags,    NULL,
+		"Label each moon with its Roman Numeral sequence number"},
+	    {"FlipTB",	   "Flip T/B",		&flip_tb, NULL,
+		"Flip the display top-to-bottom"},
+	    {"FlipLR",	   "Flip L/R",		&flip_lr, NULL,
+		"Flip the display left-to-right"},
 	};
 	typedef struct {
 	    char *label;	/* what goes on the help label */
@@ -278,8 +308,11 @@ sm_create_form_w()
 	XtSetArg (args[n], XmNautoUnmanage, False); n++;
 	XtSetArg (args[n], XmNdefaultPosition, False); n++;
 	XtSetArg (args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
+	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
 	satform_w = XmCreateFormDialog (toplevel_w, "Saturn", args, n);
+	set_something (satform_w, XmNcolormap, (XtArgVal)xe_cm);
 	XtAddCallback (satform_w, XmNhelpCallback, sm_help_cb, 0);
+	XtAddCallback (satform_w, XmNunmapCallback, sm_unmap_cb, 0);
 
 	/* set some stuff in the parent DialogShell.
 	 * setting XmNdialogTitle in the Form didn't work..
@@ -309,12 +342,22 @@ sm_create_form_w()
 	    set_xmstring (cb_w, XmNlabelString, "Control");
 	    XtManageChild (cb_w);
 
+	    /* add the "Field stars" push button */
+
+	    n = 0;
+	    w = XmCreatePushButton (pd_w, "FS", args, n);
+	    set_xmstring (w, XmNlabelString, "Setup Field Stars...");
+	    XtAddCallback (w, XmNactivateCallback, (XtCallbackProc)fs_manage,0);
+	    wtip (w, "Define where GSC and PPM catalogs are to be found");
+	    XtManageChild (w);
+
 	    /* add the "LOCFIFO" push button */
 
 	    n = 0;
 	    w = XmCreatePushButton (pd_w, "LOCFIFO", args, n);
 	    set_xmstring (w, XmNlabelString, "Send to LOCFIFO");
 	    XtAddCallback (w, XmNactivateCallback, sm_locfifo_cb, 0);
+	    wtip (w, "Open export connection");
 	    XtManageChild (w);
 
 	    /* add the "movie" push button */
@@ -323,6 +366,7 @@ sm_create_form_w()
 	    w = XmCreatePushButton (pd_w, "Movie", args, n);
 	    set_xmstring (w, XmNlabelString, "Movie Demo");
 	    XtAddCallback (w, XmNactivateCallback, sm_movie_cb, 0);
+	    wtip (w, "Start/Stop a demonstration animation sequence");
 	    XtManageChild (w);
 
 	    /* add the "close" push button beneath a separator */
@@ -334,6 +378,7 @@ sm_create_form_w()
 	    n = 0;
 	    w = XmCreatePushButton (pd_w, "Close", args, n);
 	    XtAddCallback (w, XmNactivateCallback, sm_close_cb, 0);
+	    wtip (w, "Close this and all supporting dialogs");
 	    XtManageChild (w);
 
 	/* make the View pulldown */
@@ -359,11 +404,13 @@ sm_create_form_w()
 		XtAddCallback (w, XmNvalueChangedCallback, sm_option_cb, 
 						    (XtPointer)(op->flagp));
 		set_xmstring (w, XmNlabelString, op->label);
-		XtManageChild (w);
 		if (op->flagp)
 		    *(op->flagp) = XmToggleButtonGetState(w);
 		if (op->wp)
 		    *op->wp = w;
+                if (op->tip)
+		    wtip (w, op->tip);
+		XtManageChild (w);
 	    }
 
 	    /* add a separator */
@@ -378,6 +425,7 @@ sm_create_form_w()
 	    w = XmCreatePushButton (pd_w, "Stats", args, n);
 	    set_xmstring (w, XmNlabelString, "More info...");
 	    XtAddCallback (w, XmNactivateCallback, sm_sstats_cb, NULL);
+	    wtip (w, "Display additional details");
 	    XtManageChild (w);
 
 	/* make the help pulldown */
@@ -415,6 +463,7 @@ sm_create_form_w()
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
 	dt_w = XmCreateLabel (satform_w, "DateStamp", args, n);
 	timestamp (mm_get_now(), dt_w);	/* establishes size */
+	wtip (dt_w, "Date and Time for which map is computed");
 	XtManageChild (dt_w);
 
 	/* make the scale widget.
@@ -435,6 +484,7 @@ sm_create_form_w()
 	scale_w = XmCreateScale (satform_w, "Scale", args, n);
 	XtAddCallback (scale_w, XmNdragCallback, sm_scale_cb, 0);
 	XtAddCallback (scale_w, XmNvalueChangedCallback, sm_scale_cb, 0);
+	wtip (scale_w, "Zoom in and out");
 	XtManageChild (scale_w);
 
 	/* make the limiting mag scale widget.
@@ -456,6 +506,7 @@ sm_create_form_w()
 	limmag_w = XmCreateScale (satform_w, "LimMag", args, n);
 	XtAddCallback (limmag_w, XmNdragCallback, sm_scale_cb, 0);
 	XtAddCallback (limmag_w, XmNvalueChangedCallback, sm_scale_cb, 0);
+	wtip (limmag_w, "Adjust the brightness and limiting magnitude");
 	XtManageChild (limmag_w);
 
 	/* make a frame for the drawing area.
@@ -493,14 +544,21 @@ sm_create_ssform_w()
 	    char *rcname;	/* name of rowcolumn */
 	    char *collabel;	/* column label */
 	    char *suffix;	/* suffix for plot/list/search name */
+	    char *tip;          /* widget tip */
 	} MoonColumn;
 	static MoonColumn mc[] = {
-	    {CX,   "SatX",   "X (+E)",     "X"},
-	    {CY,   "SatY",   "Y (+S)",     "Y"},
-	    {CZ,   "SatZ",   "Z (+front)", "Z"},
-	    {CRA,  "SatRA",  "RA",         "RA"},
-	    {CDEC, "SatDec", "Dec",        "Dec"},
-	    {CMAG, "SatMag", "Mag",        "Mag"},
+	    {CX,   "SatX",   "X (+E)",     "X",
+	    	"Apparent displacement east of planetary center"},
+	    {CY,   "SatY",   "Y (+S)",     "Y",
+	    	"Apparent displacement south of planetary center"},
+	    {CZ,   "SatZ",   "Z (+front)", "Z",
+	    	"Saturn radii towards Earth from planetary center"},
+	    {CRA,  "SatRA",  "RA",         "RA",
+	    	"Right Ascension (to Main's settings)"},
+	    {CDEC, "SatDec", "Dec",        "Dec",
+	    	"Declination (to Main's settings)"},
+	    {CMAG, "SatMag", "Mag",        "Mag",
+	    	"Apparent visual magnitude"},
 	};
 	Widget w;
 	Widget rc_w, title_w, sep_w;
@@ -513,7 +571,9 @@ sm_create_ssform_w()
 	XtSetArg (args[n], XmNautoUnmanage, False); n++;
 	XtSetArg (args[n], XmNresizePolicy, XmRESIZE_ANY); n++;
 	XtSetArg (args[n], XmNverticalSpacing, 5); n++;
+	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
 	ssform_w = XmCreateFormDialog (toplevel_w, "SaturnStats", args, n);
+	set_something (ssform_w, XmNcolormap, (XtArgVal)xe_cm);
 
 	/* set some stuff in the parent DialogShell.
 	 * setting XmNdialogTitle in the Form didn't work..
@@ -553,6 +613,7 @@ sm_create_ssform_w()
 	    XtSetArg (args[n], XmNuserData, "Saturn.ETilt"); n++;
 	    ringet_w = XmCreatePushButton (rc_w, "SatET", args, n);
 	    XtAddCallback (ringet_w, XmNactivateCallback, sm_activate_cb, 0);
+	    wtip (ringet_w, "Ring tilt as seen from Earth");
 	    XtManageChild (ringet_w);
 
 	/* make the sun tilt r/c */
@@ -576,6 +637,7 @@ sm_create_ssform_w()
 	    XtSetArg (args[n], XmNuserData, "Saturn.STilt"); n++;
 	    ringst_w = XmCreatePushButton (rc_w, "SatST", args, n);
 	    XtAddCallback (ringst_w, XmNactivateCallback, sm_activate_cb, 0);
+	    wtip (ringst_w, "Ring tilt as seen from Sun");
 	    XtManageChild (ringst_w);
 
 	/* make table title label */
@@ -605,8 +667,9 @@ sm_create_ssform_w()
 
 	    n = 0;
 	    w = XmCreateLabel (rc_w, "TL", args, n);
-	    XtManageChild (w);
 	    set_xmstring (w, XmNlabelString, "Tag");
+	    wtip (w, "Roman Numeral sequence designation of moon");
+	    XtManageChild (w);
 
 	    for (i = 0; i < NM; i++) {
 		char *tag = mnames[i].tag;
@@ -632,8 +695,9 @@ sm_create_ssform_w()
 
 	    n = 0;
 	    w = XmCreateLabel (rc_w, "NL", args, n);
-	    XtManageChild (w);
 	    set_xmstring (w, XmNlabelString, "Name");
+	    wtip (w, "Common name of body");
+	    XtManageChild (w);
 
 	    for (i = 0; i < NM; i++) {
 		n = 0;
@@ -661,8 +725,10 @@ sm_create_ssform_w()
 
 	    n = 0;
 	    w = XmCreateLabel (rc_w, "SatLab", args, n);
-	    XtManageChild (w);
 	    set_xmstring (w, XmNlabelString, mp->collabel);
+            if (mp->tip)
+		wtip (w, mp->tip);
+	    XtManageChild (w);
 
 	    for (j = 0; j < NM; j++) {
 		char *sel;
@@ -697,6 +763,7 @@ sm_create_ssform_w()
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
 	sdt_w = XmCreateLabel (ssform_w, "JDateStamp", args, n);
+	wtip (sdt_w, "Date and Time for which data are computed");
 	XtManageChild (sdt_w);
 
 	/* make a separator */
@@ -715,12 +782,14 @@ sm_create_ssform_w()
 	XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
 	XtSetArg (args[n], XmNtopWidget, sep_w); n++;
 	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
+	XtSetArg (args[n], XmNbottomOffset, 5); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
 	XtSetArg (args[n], XmNleftPosition, 30); n++;
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
 	XtSetArg (args[n], XmNrightPosition, 70); n++;
 	w = XmCreatePushButton (ssform_w, "Close", args, n);
 	XtAddCallback (w, XmNactivateCallback, sm_sstats_cb, 0);
+	wtip (w, "Close this dialog");
 	XtManageChild (w);
 }
 
@@ -823,7 +892,7 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
-	if ((sswasman = XtIsManaged(ssform_w)))
+	if ((sswasman = XtIsManaged(ssform_w)) != 0)
 	    XtUnmanageChild (ssform_w);
 	else {
 	    XtManageChild (ssform_w);
@@ -877,6 +946,32 @@ XtPointer call;
 	}
 }
 
+/* callback when main dialog is unmapped */
+/* ARGSUSED */
+static void
+sm_unmap_cb (w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
+{
+	if ((sswasman = (ssform_w && XtIsManaged(ssform_w))) != 0)
+	    XtUnmanageChild (ssform_w);
+        if (sm_pm) {
+	    XFreePixmap (XtDisplay(sda_w), sm_pm);
+	    sm_pm = (Pixmap) NULL;
+	}
+
+	/* free any field stars */
+	if (fstars) {
+	    free ((void *)fstars);
+	    fstars = NULL;
+	    nfstars = 0;
+	}
+
+	/* stop movie that might be running */
+	mm_movie (0.0);
+}
+
 /* callback from the main Close button */
 /* ARGSUSED */
 static void
@@ -885,12 +980,8 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
+	/* let unmap do all the work */
 	XtUnmanageChild (satform_w);
-	if ((sswasman = XtIsManaged(ssform_w)))
-	    XtUnmanageChild (ssform_w);
-
-	/* stop movie that might be running */
-	mm_movie (0.0);
 }
 
 /* callback from the Movie button
@@ -902,7 +993,7 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
-	/* best effect if turn off worrying about the sky background */
+	/* smoother if turn off the sky background */
 	skybkg = 0;
 	XmToggleButtonSetState (skybkg_w, False, False);
 
@@ -918,7 +1009,13 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
+	static last_nx, last_ny;
 	XmDrawingAreaCallbackStruct *c = (XmDrawingAreaCallbackStruct *)call;
+        Window win = XtWindow(w);
+	Display *dsp = XtDisplay(w);
+	unsigned int nx, ny, bw, d;
+	Window root;
+	int x, y;
 
 	/* filter out a few oddball cases */
 	switch (c->reason) {
@@ -931,11 +1028,14 @@ XtPointer call;
 
 	    if (!before) {
 		XSetWindowAttributes swa;
+                unsigned long mask = CWBitGravity | CWBackingStore;
+
 		swa.bit_gravity = ForgetGravity;
-		XChangeWindowAttributes (e->display, e->window, 
-							    CWBitGravity, &swa);
+		swa.backing_store = NotUseful; /* we use a pixmap */
+		XChangeWindowAttributes (dsp, win, mask, &swa);
 		before = 1;
 	    }
+
 	    /* wait for the last in the series */
 	    if (e->count != 0)
 		return;
@@ -946,7 +1046,14 @@ XtPointer call;
 	    exit(1);
 	}
 
-	sm_update (mm_get_now(), 1);
+        XGetGeometry(dsp, win, &root, &x, &y, &nx, &ny, &bw, &d);
+	if (!sm_pm || nx != last_nx || ny != last_ny) {
+	    sm_pm = XCreatePixmap (dsp, win, nx, ny, d);
+	    last_nx = nx;
+	    last_ny = ny;
+	    sm_update (mm_get_now(), 1);
+	} else
+	    XCopyArea (dsp, sm_pm, win, s_fgc, 0, 0, nx, ny, 0, 0);
 }
 
 /* callback from mouse or keyboard input over drawing area */
@@ -1025,7 +1132,7 @@ ScreenObj *sop;
 	char buf[64], buf2[64];
 
 	name = sop->o.o_name;
-	mag = sop->o.s_mag/MAGSCALE;
+	mag = get_mag(&sop->o);
 	ra = sop->o.s_ra;
 	dec = sop->o.s_dec;
 
@@ -1036,7 +1143,7 @@ ScreenObj *sop;
 	(void) sprintf (buf2, "  RA: %s", buf);
 	set_xmstring (smpu_ra_w, XmNlabelString, buf2);
 
-	fs_pangle (buf, dec);
+	fs_prdec (buf, dec);
 	(void) sprintf (buf2, " Dec: %s", buf);
 	set_xmstring (smpu_dec_w, XmNlabelString, buf2);
 
@@ -1141,7 +1248,7 @@ close_screenobj (x, y)
 int x, y;
 {
 	int minr;
-	ScreenObj *sop, *minsop;
+	ScreenObj *sop, *minsop = NULL;
 
 	minr = 2*MINR;
 	for (sop = &screenobj[nscreenobj]; --sop >= screenobj; ) {
@@ -1167,6 +1274,7 @@ typedef struct {
 static void saturn_data();
 static void moonradec();
 static void sm_draw_map ();
+static void make_gcs ();
 
 /* called to recompute and fill in values for the saturn menu.
  * don't bother if it doesn't exist or is unmanaged now or no one is logging.
@@ -1216,7 +1324,7 @@ int how_much;
 		f_double (s_w[i][CZ], fmt, md[i].z);
 		f_double (s_w[i][CMAG], "%4.1f", md[i].mag);
 		f_ra (s_w[i][CRA], md[i].ra);
-		f_pangle (s_w[i][CDEC], md[i].dec);
+		f_prdec (s_w[i][CDEC], md[i].dec);
 	    }
 
 	    f_double (ringet_w, fmt, raddeg(etilt));
@@ -1250,11 +1358,6 @@ double satsize;
 double salpha, calpha;
 MoonData md[NM];
 {
-	static GC s_fgc, s_bgc, s_xgc;
-	static XFontStruct *s_fs;
-	static last_nx, last_ny;
-	static int cw, ch;
-	static Pixmap pm;
 	Display *dsp = XtDisplay(w);
 	Window win = XtWindow(w);
 	Window root;
@@ -1275,62 +1378,35 @@ MoonData md[NM];
 #define	XCORD(x)	((int)(((int)nx)/2.0 + ew*MAPSCALE(x) + 0.5))
 #define	YCORD(y)	((int)(((int)ny)/2.0 - ns*MAPSCALE(y) + 0.5))
 
-	if (!s_fgc) {
-	    XGCValues gcv;
-	    unsigned int gcm;
-	    Pixel fg, bg;
-
-	    gcm = GCForeground;
-	    get_something (w, XmNforeground, (XtArgVal)&fg);
-	    gcv.foreground = fg;
-	    s_fgc = XCreateGC (dsp, win, gcm, &gcv);
-	    s_fs = XQueryFont (dsp, XGContextFromGC (s_fgc));
-	    cw = s_fs->max_bounds.width;
-	    ch = s_fs->max_bounds.ascent + s_fs->max_bounds.descent;
-
-	    gcm = GCForeground;
-	    get_something (w, XmNbackground, (XtArgVal)&bg);
-	    gcv.foreground = bg;
-	    s_bgc = XCreateGC (dsp, win, gcm, &gcv);
-
-	    gcm = GCForeground | GCFunction;
-	    gcv.foreground = fg ^ bg;
-	    gcv.function = GXxor;
-	    s_xgc = XCreateGC (dsp, win, gcm, &gcv);
-	}
+	if (!s_fgc) 
+	    make_gcs(w);
 
 	XmScaleGetValue (limmag_w, &fmag);
 	XmScaleGetValue (scale_w, &sv);
 	scale = pow(MAXSCALE, sv/100.0);
 
 	XGetGeometry(dsp, win, &root, &x, &y, &nx, &ny, &bw, &d);
-	if (!pm || nx != last_nx || ny != last_ny) {
-	    if (pm)
-		XFreePixmap (dsp, pm);
-	    pm = XCreatePixmap (dsp, win, nx, ny, d);
-	    last_nx = nx;
-	    last_ny = ny;
-	}
 
-	XFillRectangle (dsp, pm, s_bgc, 0, 0, nx, ny);
+	XFillRectangle (dsp, sm_pm, s_bgc, 0, 0, nx, ny);
 
 	/* prepare for a new list of things on the screen */
 	reset_screenobj();
 
 	/* draw background objects, if desired */
 	if (skybkg)
-	    sky_background(pm, nx, ny, fmag, md[0].ra, md[0].dec, salpha,calpha,
-			satsize/MAPSCALE(2), satsize*NORM, flip_tb, flip_lr);
+	    sky_background(sm_pm, nx, ny, fmag, md[0].ra, md[0].dec, salpha,
+		calpha, satsize/MAPSCALE(2), satsize*NORM, flip_tb, flip_lr);
 
 	/* draw labels */
 	c = flip_lr ? 'W' : 'E';
-	XDrawString(dsp, pm, s_fgc, nx-cw-1, ny/2-2, &c, 1);
+	XDrawString(dsp, sm_pm, s_fgc, nx-s_cw-1, ny/2-2, &c, 1);
 	c = flip_tb ? 'N' : 'S';
-	XDrawString(dsp, pm, s_fgc, (nx-cw)/2-1, s_fs->ascent, &c, 1);
+	XDrawString(dsp, sm_pm, s_fgc, (nx-s_cw)/2-1, s_fs->ascent, &c, 1);
 
 	/* draw saturn in center with unit radius */
-	XFillArc (dsp, pm, s_fgc, nx/2-(int)MAPSCALE(1), ny/2-(int)MAPSCALE(1),
-			    (int)MAPSCALE(2), (int)MAPSCALE(2), 0, 360*64);
+	XPSFillArc(dsp,sm_pm, s_fgc, nx/2-(int)MAPSCALE(1),
+					ny/2-(int)MAPSCALE(1), (int)MAPSCALE(2),
+					(int)MAPSCALE(2), 0, 360*64);
 	add_screenobj (db_basic(SATURN), nx/2, ny/2);
 
 	/* rings of radius IRR and ORR.
@@ -1350,23 +1426,28 @@ MoonData md[NM];
 	ory = ny/2 - MAPSCALE(ORR*fabs(sin(etilt)));
 	if (irh < RLW || orh < RLW) {
 	    /* too near the ring plane to draw a fill ellipse */
-	    XDrawLine (dsp, pm, s_fgc, orx, ny/2, nx-orx, ny/2);
+	    XDrawLine (dsp, sm_pm, s_fgc, orx, ny/2, nx-orx, ny/2);
 	} else {
 	    /* near rings are up if tilt is positive and we are not flipping n/s
 	     * or if tilt is negative and we are flipping n/s.
 	     */
 	    int nearup = ((etilt>0.0) == !flip_tb);
 
-	    XDrawArc (dsp, pm, s_xgc, irx, iry, irw, irh,
+	    XDrawArc (dsp, sm_pm, s_xgc, irx, iry, irw, irh,
 					     nearup ? 0 : 180*64, 180*64-1);
-	    XDrawArc (dsp, pm, s_xgc, orx, ory, orw, orh,
+	    XDrawArc (dsp, sm_pm, s_xgc, orx, ory, orw, orh,
 					     nearup ? 0 : 180*64, 180*64-1);
-	    XDrawArc (dsp, pm, s_fgc, irx, iry, irw, irh,
+	    XDrawArc (dsp, sm_pm, s_fgc, irx, iry, irw, irh,
 					     nearup ? 180*64 : 0, 180*64-1);
-	    XDrawArc (dsp, pm, s_fgc, orx, ory, orw, orh,
+	    XDrawArc (dsp, sm_pm, s_fgc, orx, ory, orw, orh,
 					     nearup ? 180*64 : 0, 180*64-1);
 	}
 
+	/* draw 1' calibration line */
+	if (tags)
+	    sm_calibline (dsp, sm_pm, s_fgc, nx/2, 5*ny/6, "1'", s_cw*2, s_ch+4,
+				(int)MAPSCALE(degrad(1.0/60.0)/(satsize/2)));
+
 	/* draw each moon
 	 */
 	for (i = 1; i < NM; i++) {
@@ -1381,7 +1462,7 @@ MoonData md[NM];
 
 	    if (!outside && !infront)
 		continue;	/* behind saturn */
-	    diam = magdiam (fmag, satsize/MAPSCALE(2), mag, 0.0);
+	    diam = magdiam (fmag, 1, satsize/MAPSCALE(2), mag, 0.0);
 	    if (brmoons)
 		diam += 3;
 	    if (diam <= 0)
@@ -1390,28 +1471,80 @@ MoonData md[NM];
 	    x = XCORD(mx);
 	    y = YCORD(my);
 	    if (diam == 1)
-		XDrawPoint(dsp, pm, s_xgc, x, y);
+		XDrawPoint(dsp, sm_pm, s_xgc, x, y);
 	    else
-		XFillArc (dsp, pm, s_xgc, x-diam/2, y-diam/2, diam, diam,
+		XPSFillArc (dsp, sm_pm, s_xgc, x-diam/2, y-diam/2, diam, diam,
 								    0, 360*64);
 
 	    /* add object to list of screen objects drawn */
 	    (void) strcpy (o.o_name, mnames[i].full);
-	    o.s_mag = mag*MAGSCALE;
+	    set_smag (&o, mag);
 	    o.s_ra = md[i].ra;
 	    o.s_dec = md[i].dec;
 	    add_screenobj (&o, x, y);
 
 	    if (tags && mnames[i].tag)
-		XDrawString(dsp, pm, s_xgc, x-cw/2, y+2*ch,
+		XDrawString(dsp, sm_pm, s_xgc, x-s_cw/2, y+2*s_ch,
 					mnames[i].tag, strlen(mnames[i].tag));
 	}
 
-	XCopyArea (dsp, pm, win, s_fgc, 0, 0, nx, ny, 0, 0);
+	XCopyArea (dsp, sm_pm, win, s_fgc, 0, 0, nx, ny, 0, 0);
+}
+
+static void
+make_gcs (w)
+Widget w;
+{
+	Display *dsp = XtDisplay(w);
+	Window win = XtWindow(w);
+	XGCValues gcv;
+	unsigned int gcm;
+	Pixel fg, bg;
+
+	gcm = GCForeground;
+	get_something (w, XmNforeground, (XtArgVal)&fg);
+	gcv.foreground = fg;
+	s_fgc = XCreateGC (dsp, win, gcm, &gcv);
+	s_fs = XQueryFont (dsp, XGContextFromGC (s_fgc));
+	s_cw = s_fs->max_bounds.width;
+	s_ch = s_fs->max_bounds.ascent + s_fs->max_bounds.descent;
+
+	gcm = GCForeground;
+	get_something (w, XmNbackground, (XtArgVal)&bg);
+	gcv.foreground = bg;
+	s_bgc = XCreateGC (dsp, win, gcm, &gcv);
+
+	gcm = GCForeground | GCFunction;
+	gcv.foreground = fg ^ bg;
+	gcv.function = GXxor;
+	s_xgc = XCreateGC (dsp, win, gcm, &gcv);
+}
+
+/* draw a calibration line l pixels long centered at [xc,yc] marked with tag
+ * with is in a bounding box tw x th.
+ */
+static void
+sm_calibline (dsp, win, gc, xc, yc, tag, tw, th, l)
+Display *dsp;
+Drawable win;
+GC gc;
+int xc, yc;
+char *tag;
+int tw, th;
+int l;
+{
+	int lx = xc - l/2;
+	int rx = lx + l;
+
+	XDrawLine (dsp, win, gc, lx, yc, rx, yc);
+	XDrawLine (dsp, win, gc, lx, yc-3, lx, yc+3);
+	XDrawLine (dsp, win, gc, rx, yc-3, rx, yc+3);
+	XDrawString (dsp, win, gc, xc-tw/2, yc+th, tag, strlen(tag));
 }
 
 /* draw all database objects in a small sky patch with the given orientation.
- * also save objects and screen locs in the global screenobj array.
+ * get field stars too if enabled and we have moved enough.
+ * save objects and screen locs in the global screenobj array for picking.
  * this is used to draw the backgrounds for the planet closeups.
  * Based on work by: Dan Bruton <WDB3926@acs.tamu.edu>
  */
@@ -1426,29 +1559,36 @@ double scale;		/* rads per pixel */
 double rad;		/* maximum radius to draw away from ra0/dec0, rads */
 int fliptb, fliplr;	/* flip direction; default is S up E right */
 {
+	static before;
 	double cdec0 = cos(dec0);
 	DBScan dbs;
 	Obj *op;
 
+	if (!before && pref_get(PREF_EQUATORIAL) == PREF_GEO) {
+	    xe_msg ("Equatorial preference should probably be set to Topocentric", 1);
+	    before = 1;
+	}
+
+	/* update field star list */
+	sm_loadfs (ra0, dec0);
+
 	/* scan the database and draw whatever is near */
-	for (db_scaninit(&dbs); (op = db_scan (&dbs)) != NULL; ) {
+	for (db_scaninit(&dbs, ALLM, fstars, nfstars);
+					    (op = db_scan (&dbs)) != NULL; ) {
 	    double dra, ddec;
 	    GC gc;
 	    int dx, dy, x, y;
 	    int diam;
 
-	    if (op->type == UNDEFOBJ)
-		continue;
-
 	    if (is_planet (op, SATURN)) {
-		/* we draw it elsewhere */
+		/* we draw it elsewhere :-) */
 		continue;
 	    }
 
 	    db_update (op);
 
 	    /* find size, in pixels. */
-	    diam = magdiam (fmag, scale, op->s_mag/MAGSCALE,
+	    diam = magdiam (fmag, 1, scale, get_mag(op),
 					    degrad((double)op->s_size/3600.0));
 	    if (diam <= 0)
 		continue;
@@ -1456,7 +1596,7 @@ int fliptb, fliplr;	/* flip direction; default is S up E right */
 	    /* find x/y location if it's in the general area */
 	    dra = op->s_ra - ra0;	/* + E */
 	    ddec = dec0 - op->s_dec;	/* + S */
-	    if (fabs(ddec) > rad || (PI-fabs(dra-PI))*cdec0 > rad)
+	    if (fabs(ddec) > rad || delra(dra)*cdec0 > rad)
 		continue;
 	    dx = (dra*calpha - ddec*salpha)/scale;
 	    dy = (dra*salpha + ddec*calpha)/scale;
@@ -1476,6 +1616,35 @@ int fliptb, fliplr;	/* flip direction; default is S up E right */
 	sv_draw_obj (XtD, win, (GC)0, NULL, 0, 0, 0, 0);	/* flush */
 }
 
+/* load field stars around the given location, unless current set is
+ * already close enough.
+ */
+static void
+sm_loadfs (ra, dec)
+double ra, dec;
+{
+	Now *np = mm_get_now();
+
+        if (fstars && fabs(dec-fsdec)<FSMOVE && cos(dec)*delra(ra-fsra)<FSMOVE)
+	    return;
+
+	if (fstars) {
+	    free ((void *)fstars);
+	    fstars = NULL;
+	    nfstars = 0;
+	}
+
+        nfstars = fs_fetch (np, ra, dec, FSFOV, FSMAG, &fstars);
+
+	if (nfstars > 0) {
+	    char msg[128];
+            (void) sprintf (msg, "Saturn View added %d field stars", nfstars);
+	    xe_msg (msg, 0);
+	    fsdec = dec;
+	    fsra = ra;
+	}
+}
+
 /* given saturn loc in md[0].ra/dec and size, and location of each moon in 
  * md[1..NM-1].x/y in sat radii, find ra/dec of each moon in md[1..NM-1].ra/dec.
  */
@@ -1656,10 +1825,11 @@ MoonData md[NM];        /* fill in md[0].ra/dec/mag/x/y/z for jupiter,
      /* ECD: and mag and x/y/z */
      {
 	Obj *op = db_basic(SATURN);
+	db_update (op);
 
 	md[0].ra = RA = op->s_ra;
 	md[0].dec = DECL = op->s_dec;
-	md[0].mag = op->s_mag/MAGSCALE;
+	md[0].mag = get_mag(op);
 
 	md[0].x = 0;
 	md[0].y = 0;
