@@ -7,6 +7,7 @@
 #if defined(__STDC__)
 #include <stdlib.h>
 #endif
+
 #include <X11/Xlib.h>
 #include <Xm/Xm.h>
 #include <Xm/Form.h>
@@ -26,21 +27,24 @@
 
 extern Widget	toplevel_w;
 #define	XtD	XtDisplay(toplevel_w)
+extern Colormap xe_cm;
 
 extern Now *mm_get_now P_((void));
 extern void get_something P_((Widget w, char *resource, XtArgVal value));
 extern void gk_mag P_((double g, double k, double rp, double rho, double *mp));
-extern void prompt_map_cb P_((Widget w, XtPointer client, XtPointer call));
+extern void set_something P_((Widget w, char *resource, XtArgVal value));
 extern void sv_all P_((Now *np));
 extern void sv_draw_obj P_((Display *dsp, Drawable win, GC gc, Obj *op, int x,
     int y, int diam, int dotsonly));
+extern void zero_mem P_((void *loc, unsigned len));
+
 
 /* info for each filter option toggle */
 typedef struct {
     char *label;	/* label on the toggle button */
     char *name;		/* name of toggle button, for resource setting */
-    int type;		/* as in Obj.type */
-    char class;		/* as in Obj.f.f_class */
+    int type;		/* as in Obj.o_type */
+    char class;		/* if type==FIXED then as in Obj.f.f_class */
     char state;		/* 0 or 1 depending on whether this class is on */
     Widget tbw;		/* the toggle button widget */
 } FilterTB;
@@ -80,8 +84,7 @@ static FilterTB other_f[] = {
     {"Q Quasars", 	"Quasars",	FIXED,		'Q'},
     {"L Pulsars", 	"Pulsars",	FIXED,		'L'},
     {"J Radio", 	"Radio",	FIXED,		'J'},
-    {"R S/N Remnants", 	"SNRemnants",	FIXED,		'R'},
-    {"T Stellar",	"Stellar",	FIXED,		'T'},
+    {"R S/N Rem", 	"SNRemnants",	FIXED,		'R'},
     {"Undefined",	"Undefined",	FIXED,		'\0'}
 };
 
@@ -90,7 +93,8 @@ static FilterTB stars_f[] = {
     {"B Binary",	"Binary",	FIXED,		'B'},
     {"D Double",	"Double",	FIXED,		'D'},
     {"M Multiple",	"Multiple",	FIXED,		'M'},
-    {"V Variable",	"Variable",	FIXED,		'V'}
+    {"V Variable",	"Variable",	FIXED,		'V'},
+    {"T Star-like",	"Stellar",	FIXED,		'T'}
 };
 
 static FilterTB nebulae_f[] = {
@@ -115,22 +119,25 @@ static FilterTB clusters_f[] = {
 static FilterCat filters[] = {
     {"Solar System:",	solsys_f,	XtNumber(solsys_f)},
     {"Clusters:",	clusters_f,	XtNumber(clusters_f)},
+    {"Other:",		other_f,	XtNumber(other_f)},
     {"Galaxies:",	galaxies_f,	XtNumber(galaxies_f)},
     {"Nebulae:",	nebulae_f,	XtNumber(nebulae_f)},
     {"Stars:",		stars_f,	XtNumber(stars_f)},
-    {"Other:",		other_f,	XtNumber(other_f)}
 };
 
 /* these form rapid lookup tables for the state of an object.
- * they can be directly indexed by Obj.type and Obj.f.f_class, respectively.
+ * type_table can be directly indexed by Obj.o_type and, if type is FIXED,
+ *   fclass_table by Obj.f.f_class.
  */
-static char fclass_table[128];
-static char type_table[10];
+static char type_table[NOBJTYPES];
+static char fclass_table[NCLASSES];
 
-#define BMAGLIMIT       (-28)   /* brightest setting for the Mag scales */
-#define FMAGLIMIT       20      /* faintest setting for the Mag scales */
-static int fmag, bmag;		/* current values of faintest/brightest mag */
-static Widget fmag_w, bmag_w;	/* faintest and brightest mag scales */
+#define BMAGLIMIT       (-28)   /* brightest setting for the Mag scale */
+#define FMAGLIMIT       20      /* faintest setting for the Mag scale */
+#define	MINSTEP		1	/* min scale step size */
+#define	MAXSTEP		4	/* max scale step size */
+static int fmag, magstp;	/* current values of faintest/step mag */
+static Widget fmag_w, magstp_w;	/* faintest and step mag scales */
 
 static Widget filter_w;		/* main filter dialog */
 
@@ -146,20 +153,18 @@ svf_create()
 	    char *name;
 	    void (*cb)();
 	} ctls[] = {
+	    /* balance in half */
 	    {"Ok", svf_ok_cb},
 	    {"Apply", svf_apply_cb},
-	    {"All", svf_all_cb},
-	    {"Toggle All", svf_toggle_cb},
+	    {"Toggle\nAll", svf_toggle_cb},
+	    {"All\nOn", svf_all_cb},
 	    {"Reset", svf_reset_cb},
 	    {"Close", svf_close_cb}
 	};
 	Arg args[20];
 	XmString str;
-	Widget w;
-	Widget f_w;
-	Widget ctlf_w;
-	Widget topsep_w, botsep_w;
-	Widget hrc_w;
+	Widget w, f_w, s_w;
+	Widget rc_w, lrc_w, rrc_w, mrc_w;
 	int n;
 	int i;
 
@@ -167,8 +172,10 @@ svf_create()
 	n = 0;
 	XtSetArg (args[n], XmNautoUnmanage, False); n++;
 	XtSetArg (args[n], XmNverticalSpacing, 4); n++;
+	XtSetArg (args[n], XmNdefaultPosition, False); n++;
+	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
 	filter_w = XmCreateFormDialog (toplevel_w, "SkyFilter", args, n);
-	XtAddCallback (filter_w, XmNmapCallback, prompt_map_cb, NULL);
+	set_something (filter_w, XmNcolormap, (XtArgVal)xe_cm);
 
 	/* set some stuff in the parent DialogShell.
 	 * setting XmNdialogTitle in the Form didn't work..
@@ -177,26 +184,95 @@ svf_create()
 	XtSetArg (args[n], XmNtitle, "xephem Sky view Filter"); n++;
 	XtSetValues (XtParent(filter_w), args, n);
 
+	/* put the control buttons across the bottom
+	 * use two rowcols to center the bunch
+	 */
 
-	/* put the mag limits in a form to center them across the top */
+	n = 0;
+	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
+	XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
+	XtSetArg (args[n], XmNrightPosition, 50); n++;
+	XtSetArg (args[n], XmNisAligned, False); n++;
+	XtSetArg (args[n], XmNorientation, XmHORIZONTAL); n++;
+	XtSetArg (args[n], XmNspacing, 2); n++;
+	rc_w = XmCreateRowColumn (filter_w, "CRC", args, n);
+	XtManageChild (rc_w);
 
+	for (i = 0; i < XtNumber(ctls)/2; i++) {
+	    str = XmStringCreateLtoR(ctls[i].name,XmSTRING_DEFAULT_CHARSET);
+	    n = 0;
+	    XtSetArg (args[n], XmNalignment, XmALIGNMENT_CENTER); n++;
+	    XtSetArg (args[n], XmNmarginLeft, 2); n++;
+	    XtSetArg (args[n], XmNmarginRight, 2); n++;
+	    XtSetArg (args[n], XmNlabelString, str); n++;
+	    w = XmCreatePushButton (rc_w, "FilterCB", args, n);
+	    XtManageChild(w);
+	    XmStringFree (str);
+	    XtAddCallback (w, XmNactivateCallback, ctls[i].cb, NULL);
+	}
+
+	n = 0;
+	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
+	XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
+	XtSetArg (args[n], XmNleftPosition, 50); n++;
+	XtSetArg (args[n], XmNisAligned, False); n++;
+	XtSetArg (args[n], XmNorientation, XmHORIZONTAL); n++;
+	XtSetArg (args[n], XmNspacing, 2); n++;
+	rc_w = XmCreateRowColumn (filter_w, "CRC", args, n);
+	XtManageChild (rc_w);
+
+	for (i = XtNumber(ctls)/2; i < XtNumber(ctls); i++) {
+	    str = XmStringCreateLtoR(ctls[i].name,XmSTRING_DEFAULT_CHARSET);
+	    n = 0;
+	    XtSetArg (args[n], XmNalignment, XmALIGNMENT_CENTER); n++;
+	    XtSetArg (args[n], XmNmarginLeft, 2); n++;
+	    XtSetArg (args[n], XmNmarginRight, 2); n++;
+	    XtSetArg (args[n], XmNlabelString, str); n++;
+	    w = XmCreatePushButton (rc_w, "FilterCB", args, n);
+	    XtManageChild(w);
+	    XmStringFree (str);
+	    XtAddCallback (w, XmNactivateCallback, ctls[i].cb, NULL);
+	}
+
+	/* a sep */
+	n = 0;
+	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_WIDGET); n++;
+	XtSetArg (args[n], XmNbottomWidget, rc_w); n++;
+	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
+	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
+	s_w = XmCreateSeparator (filter_w, "Sep", args, n);
+	XtManageChild (s_w);
+
+	/* pile all but the control buttons in a main rowcolumn */
 	n = 0;
 	XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
-	XtSetArg (args[n], XmNfractionBase, 9); n++;
-	f_w = XmCreateForm (filter_w, "MagF", args, n);
+	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_WIDGET); n++;
+	XtSetArg (args[n], XmNbottomWidget, s_w); n++;
+	XtSetArg (args[n], XmNpacking, XmPACK_COLUMN); n++;
+	XtSetArg (args[n], XmNnumColumns, 2); n++;
+	mrc_w = XmCreateRowColumn (filter_w, "MRC", args, n);
+	XtManageChild (mrc_w);
+
+	/* make the left rc */
+	n = 0;
+	lrc_w = XmCreateRowColumn (mrc_w, "LRC", args, n);
+	XtManageChild (lrc_w);
+
+	/* add half the FilterCats */
+
+	for (i = 0; i < XtNumber(filters)/2; i++)
+	    svf_create_filter (lrc_w, &filters[i]);
+
+	/* add faint limit scale in a frame */
+
+	n = 0;
+	f_w = XmCreateFrame (lrc_w, "FMF", args, n);
 	XtManageChild (f_w);
 
-	    str = XmStringCreate ("Faintest Magnitude",
-						XmSTRING_DEFAULT_CHARSET);
+	    str = XmStringCreate ("Faintest Mag", XmSTRING_DEFAULT_CHARSET);
 	    n = 0;
-	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
-	    XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
-	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
-	    XtSetArg (args[n], XmNleftPosition, 1); n++;
-	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
-	    XtSetArg (args[n], XmNrightPosition, 4); n++;
 	    XtSetArg (args[n], XmNorientation, XmHORIZONTAL); n++;
 	    XtSetArg (args[n], XmNminimum, BMAGLIMIT); n++;
 	    XtSetArg (args[n], XmNmaximum, FMAGLIMIT); n++;
@@ -208,96 +284,35 @@ svf_create()
 	    XtManageChild (fmag_w);
 	    XmStringFree (str);
 
-	    str = XmStringCreate ("Brightest Magnitude",
-						XmSTRING_DEFAULT_CHARSET);
+	/* make the right rc */
+	n = 0;
+	XtSetArg (args[n], XmNisAligned, False); n++;
+	rrc_w = XmCreateRowColumn (mrc_w, "RRC", args, n);
+	XtManageChild (rrc_w);
+
+	/* add the other half of the FilterCat */
+
+	for (i = XtNumber(filters)/2; i < XtNumber(filters); i++)
+	    svf_create_filter (rrc_w, &filters[i]);
+
+	/* add mag-scale in a frame */
+
+	n = 0;
+	f_w = XmCreateFrame (rrc_w, "FMF", args, n);
+	XtManageChild (f_w);
+
+	    str = XmStringCreate ("Magnitude Step", XmSTRING_DEFAULT_CHARSET);
 	    n = 0;
-	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
-	    XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
-	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
-	    XtSetArg (args[n], XmNleftPosition, 5); n++;
-	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
-	    XtSetArg (args[n], XmNrightPosition, 8); n++;
 	    XtSetArg (args[n], XmNorientation, XmHORIZONTAL); n++;
-	    XtSetArg (args[n], XmNminimum, BMAGLIMIT); n++;
-	    XtSetArg (args[n], XmNmaximum, FMAGLIMIT); n++;
-	    XtSetArg (args[n], XmNprocessingDirection, XmMAX_ON_LEFT); n++;
+	    XtSetArg (args[n], XmNminimum, MINSTEP); n++;
+	    XtSetArg (args[n], XmNmaximum, MAXSTEP); n++;
 	    XtSetArg (args[n], XmNscaleMultiple, 1); n++;
 	    XtSetArg (args[n], XmNshowValue, True); n++;
 	    XtSetArg (args[n], XmNtitleString, str); n++;
-	    bmag_w = XmCreateScale (f_w, "BrightMagScale", args, n);
-	    XtManageChild (bmag_w);
+	    magstp_w = XmCreateScale (f_w, "MagStep", args, n);
+	    XtManageChild (magstp_w);
 	    XmStringFree (str);
 
-	/* make a separator below the mag scales */
-
-	n = 0;
-	XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
-	XtSetArg (args[n], XmNtopWidget, f_w); n++;
-	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
-	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
-	topsep_w = XmCreateSeparator (filter_w, "TopSep", args, n);
-	XtManageChild (topsep_w);
-
-	/* make a form for bottom control panel */
-
-	n = 0;
-	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
-	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
-	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
-	XtSetArg (args[n], XmNfractionBase, 25); n++;
-	XtSetArg (args[n], XmNverticalSpacing, 5); n++;
-	ctlf_w = XmCreateForm (filter_w, "SVControlF", args, n);
-	XtManageChild (ctlf_w);
-
-	    /* add the control buttons */
-
-	    for (i = 0; i < XtNumber(ctls); i++) {
-		str = XmStringCreateLtoR(ctls[i].name,XmSTRING_DEFAULT_CHARSET);
-		n = 0;
-		XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
-		XtSetArg (args[n], XmNleftPosition, 1+i*4); n++;
-		XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
-		XtSetArg (args[n], XmNrightPosition, 4+i*4); n++;
-		XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
-		XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
-		XtSetArg (args[n], XmNlabelString, str); n++;
-		w = XmCreatePushButton (ctlf_w, "FilterCB", args, n);
-		XtManageChild(w);
-		XmStringFree (str);
-		XtAddCallback (w, XmNactivateCallback, ctls[i].cb, NULL);
-	    }
-
-	/* put a separator on top of the control panel */
-
-	n = 0;
-	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
-	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
-	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_WIDGET); n++;
-	XtSetArg (args[n], XmNbottomWidget, ctlf_w); n++;
-	botsep_w = XmCreateSeparator (filter_w, "BotSep", args, n);
-	XtManageChild (botsep_w);
-
-	/* make a horizontal rowcol to contain each column */
-
-	n = 0;
-	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
-	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
-	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_WIDGET); n++;
-	XtSetArg (args[n], XmNbottomWidget, botsep_w); n++;
-	XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
-	XtSetArg (args[n], XmNtopWidget, topsep_w); n++;
-	XtSetArg (args[n], XmNorientation, XmHORIZONTAL); n++;
-	XtSetArg (args[n], XmNpacking, XmPACK_TIGHT); n++;
-	XtSetArg (args[n], XmNspacing, 0); n++;
-	XtSetArg (args[n], XmNmarginWidth, 0); n++;
-	hrc_w = XmCreateRowColumn (filter_w, "CategoryTableRC", args, n);
-	XtManageChild (hrc_w);
-
-	/* fill the horizontal rc with each FilterCat */
-
-	for (i = 0; i < XtNumber(filters); i++)
-	    svf_create_filter (hrc_w, &filters[i]);
-
 	/* set the real filter according to the state of the toggle buttons */
 	svf_apply();
 }
@@ -308,7 +323,7 @@ int
 svf_ismanaged()
 {
 
-	return (XtIsManaged(filter_w));
+	return (filter_w && XtIsManaged(filter_w));
 }
 
 /* called to manage the filter dialog.
@@ -330,13 +345,51 @@ svf_unmanage()
 	XtUnmanageChild(filter_w);
 }
 
-/* return the current faint and bright mag limits */
+/* return a copy of the currently enforced type and class tables.
+ * this is to support sky history.
+ */
 void
-svf_maglimits (fmagp, bmagp)
-int *fmagp, *bmagp;
+svf_gettables (tt, ct)
+char tt[NOBJTYPES];
+char ct[NCLASSES];
+{
+	(void) memcpy (tt, type_table, NOBJTYPES);
+	(void) memcpy (ct, fclass_table, NCLASSES);
+}
+
+/* install a new copy of the type and class tables and update the buttons
+ *   to match.
+ * this is to support sky history.
+ */
+void
+svf_settables (tt, ct)
+char tt[NOBJTYPES];
+char ct[NCLASSES];
+{
+	(void) memcpy (type_table, tt, NOBJTYPES);
+	(void) memcpy (fclass_table, ct, NCLASSES);
+
+	svf_reset();
+}
+
+/* return the current faint limit and mag step */
+void
+svf_getmaglimits (fmagp, magstpp)
+int *fmagp, *magstpp;
 {
 	*fmagp = fmag;
-	*bmagp = bmag;
+	*magstpp = magstp;
+}
+
+/* set the current faint limit and mag step */
+void
+svf_setmaglimits (f, s)
+int f, s;
+{
+	fmag = f;
+	magstp = s;
+	XmScaleSetValue (fmag_w, fmag);
+	XmScaleSetValue (magstp_w, magstp);
 }
 
 /* return 1 if p satisfies the stuff in the filter dialog, else 0.
@@ -347,16 +400,16 @@ int
 svf_filter_ok (op)
 Obj *op;
 {
-	int t = op->type;
 	int tok;
 
-	if (t == FIXED)
-	    tok = fclass_table[(int)op->f_class];
+	if (is_type(op,FIXEDM))
+	    tok = fclass_table[((int)op->f_class) & (NCLASSES-1)];
 	else
-	    tok = type_table[t];
+	    tok = type_table[op->o_type];
 
 	if (tok)
 	    return (svf_bright_ok (op));
+
 	return (0);
 }
 
@@ -367,7 +420,7 @@ Cursor c;
 {
 	Window win;
 
-	if (filter_w && (win = XtWindow(filter_w))) {
+	if (filter_w && (win = XtWindow(filter_w)) != 0) {
 	    Display *dsp = XtDisplay(filter_w);
 	    if (c)
 		XDefineCursor (dsp, win, c);
@@ -462,8 +515,9 @@ FilterCat *fcp;
 	}
 }
 
-/* set the toggle button states according to the real filter states.
- * set the mag scales from the real mag limits.
+/* set the real filter states and the toggle button states according to the
+ *   rapid-access tables.
+ * also, set the mag scales from the real mag limits.
  */
 static void
 svf_reset()
@@ -474,15 +528,17 @@ svf_reset()
 	    FilterCat *fcp = &filters[i];
 	    for (j = 0; j < fcp->nftb; j++) {
 		FilterTB *ftbp = &fcp->ftb[j];
+		ftbp->state = (ftbp->type == FIXED)
+		    ? fclass_table[(int)ftbp->class] : type_table[ftbp->type];
 		XmToggleButtonSetState (ftbp->tbw, ftbp->state, True);
 	    }
 	}
 
-	XmScaleSetValue (fmag_w, fmag);
-	XmScaleSetValue (bmag_w, bmag);
+	svf_setmaglimits (fmag, magstp);
 }
 
-/* set the filter according to the present state of the toggle buttons.
+/* set the real filter and rapid-access tables according to the present state
+ *   of the toggle buttons.
  * also set the mag limits from the scales.
  */
 static void
@@ -500,8 +556,8 @@ svf_apply()
 		    exit (1);
 		}
 		ftbp->state = XmToggleButtonGetState (ftbp->tbw);
-		if (ftbp->type == FIXED) {
-		    int c = ftbp->class;
+		if (t == FIXED) {
+		    int c = (int)ftbp->class;
 		    if (c < 0 || c >= XtNumber(fclass_table)) {
 			printf ("svf_apply: FIXED class out of range: %d\n", c);
 			exit (1);
@@ -513,16 +569,16 @@ svf_apply()
 	}
 
 	XmScaleGetValue (fmag_w, &fmag);
-	XmScaleGetValue (bmag_w, &bmag);
+	XmScaleGetValue (magstp_w, &magstp);
 }
 
-/* return 1 if db object is ever possibly within the fmag/bmag range; else 0.
+/* return 1 if db object is ever possibly brighter than fmag range; else 0.
  */
 static int
 svf_bright_ok(op)
 Obj *op;
 {
-	switch (op->type) {
+	switch (op->o_type) {
 	case PLANET:
 	    /* always go for the planets for now but ..
 	     * TODO: work up a table of extreme planet magnitudes.
@@ -544,17 +600,17 @@ Obj *op;
 	    else
 		 gk_mag(op->e_mag.m1, op->e_mag.m2,
 						per, fabs(per-1.0), &mag);
-	    return (mag <= fmag && mag >= bmag);
+	    return (mag <= fmag);
 	    /* break; */
 	    }
 	case FIXED:
-	    return (op->f_mag/MAGSCALE <= fmag && op->f_mag/MAGSCALE >= bmag);
+	    return (get_mag(op) <= fmag);
 	    /* break; */
 	case EARTHSAT:
 	    /* TODO: work on satellite magnitudes someday */
 	    return (1);
 	default: 
-	    printf ("sv_bright_ok(): bad type: %d\n", op->type);
+	    printf ("sv_bright_ok(): bad type: %d\n", op->o_type);
 	    exit (1);
 	    return (0);	/* for lint */
 	}
@@ -611,13 +667,17 @@ Widget daw;
 	static GC gc;
 	Display *dsp = XtDisplay(daw);
 	Window win = XtWindow(daw);
-	Obj obj;
+	Obj o;
 	Dimension w, h;
 	int diam;
 
-	obj.type = ftbp->type;
-	if (ftbp->type == FIXED)
-	    obj.f_class = ftbp->class;
+	zero_mem ((void *)&o, sizeof(Obj));
+	o.o_type = ftbp->type;
+	if (ftbp->type == FIXED) {
+	    o.f_class = ftbp->class;
+	    set_ratio (&o, 2, 1);
+	    set_pa (&o, PI/2);
+	}
 
 	get_something (daw, XmNwidth, (XtArgVal)&w);
 	get_something (daw, XmNheight, (XtArgVal)&h);
@@ -625,7 +685,7 @@ Widget daw;
 
 	if (!gc) {
 	    XGCValues gcv;
-	    unsigned gcm;
+	    unsigned long gcm;
 	    Pixel p;
 
 	    gcm = GCForeground;
@@ -634,7 +694,7 @@ Widget daw;
 	    gc = XCreateGC (dsp, win, gcm, &gcv);
 	}
 
-	sv_draw_obj (dsp, win, gc, &obj, (int)w/2-1, (int)h/2-1, diam, 0);
+	sv_draw_obj (dsp, win, gc, &o, (int)w/2-1, (int)h/2-1, diam, 0);
 	sv_draw_obj (dsp, win, gc, NULL, 0, 0, 0, 0);	/* flush */
 }
 
