@@ -27,6 +27,7 @@
 
 extern Widget	toplevel_w;
 #define	XtD	XtDisplay(toplevel_w)
+extern Colormap xe_cm;
 
 extern Now *mm_get_now P_((void));
 extern int any_ison P_((void));
@@ -46,6 +47,7 @@ extern void prompt_map_cb P_((Widget w, XtPointer client, XtPointer call));
 extern void redraw_screen P_((int how_much));
 extern void set_something P_((Widget w, char *resource, XtArgVal value));
 extern void set_xmstring P_((Widget w, char *resource, char *txt));
+extern void wtip P_((Widget w, char *string));
 extern void xe_msg P_((char *msg, int app_modal));
 
 static void srch_create_form P_((void));
@@ -234,7 +236,7 @@ Cursor c;
 {
 	Window win;
 
-	if (srchform_w && (win = XtWindow(srchform_w))) {
+	if (srchform_w && (win = XtWindow(srchform_w)) != 0) {
 	    Display *dsp = XtDisplay(srchform_w);
 	    if (c)
 		XDefineCursor (dsp, win, c);
@@ -249,10 +251,14 @@ srch_create_form()
 	static struct {
 	    char *title;
 	    XtPointer cb_data;
+	    char *tip;
 	} rbtb[] = {
-	    {"Find Extreme", (XtPointer)srch_minmax},
-	    {"Find 0", (XtPointer)srch_solve0},
-	    {"Binary", (XtPointer)srch_binary},
+	    {"Find Extreme", (XtPointer)srch_minmax,
+	    	"Solve for a time at which Function experiences a local minima or maxima"},
+	    {"Find 0", (XtPointer)srch_solve0,
+	    	"Solve for a time at which the Function evaluates to 0"},
+	    {"Binary", (XtPointer)srch_binary,
+	    	"Step time along until the truth value of the Function changes"},
 	};
 	XmString str;
 	Widget w, rc_w, f_w, rb_w;
@@ -263,14 +269,18 @@ srch_create_form()
 	n = 0;
 	XtSetArg (args[n], XmNautoUnmanage, False); n++;
 	XtSetArg (args[n], XmNdefaultPosition, False); n++;
+	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
+	XtSetArg (args[n], XmNmarginWidth, 10); n++;
+	XtSetArg (args[n], XmNmarginHeight, 10); n++;
 	srchform_w = XmCreateFormDialog (toplevel_w, "Srch", args, n);
+	set_something (srchform_w, XmNcolormap, (XtArgVal)xe_cm);
 	XtAddCallback (srchform_w, XmNhelpCallback, srch_help_cb, 0);
 
 	/* set some stuff in the parent DialogShell.
 	 * setting XmNdialogTitle in the Form didn't work..
 	 */
 	n = 0;
-	XtSetArg (args[n], XmNtitle, "xephem Search Control"); n++;
+	XtSetArg (args[n], XmNtitle, "xephem Solver Control"); n++;
 	XtSetValues (XtParent(srchform_w), args, n);
 
 	/* create a RowColumn to hold stuff */
@@ -279,6 +289,7 @@ srch_create_form()
 	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
+	XtSetArg (args[n], XmNspacing, 8); n++;
 	XtSetArg (args[n], XmNisAligned, False); n++;
 	XtSetArg (args[n], XmNadjustMargin, False); n++;
 	rc_w = XmCreateRowColumn (srchform_w, "SrchRC", args, n);
@@ -287,8 +298,10 @@ srch_create_form()
 	/* searching on/off toggle button */
 
 	n = 0;
-	on_w = XmCreateToggleButton (rc_w, "Searching is Active", args, n);
+	on_w = XmCreateToggleButton (rc_w, "SAct", args, n);
 	XtAddCallback (on_w, XmNvalueChangedCallback, srch_on_off_cb, 0);
+	set_xmstring (on_w, XmNlabelString, "Solver is Active");
+	wtip (on_w, "When on, solver controls Main's step size and searches for goal");
 	XtManageChild (on_w);
 
 	/* create a separator */
@@ -299,8 +312,7 @@ srch_create_form()
 
 	/* compiler area title */
 
-	str = XmStringCreate ("Search Function:",
-						    XmSTRING_DEFAULT_CHARSET);
+	str = XmStringCreate ("Function:", XmSTRING_DEFAULT_CHARSET);
 	n = 0;
 	XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
 	XtSetArg (args[n], XmNlabelString, str); n++;
@@ -314,6 +326,7 @@ srch_create_form()
 	n = 0;
 	func_w = XmCreateText (rc_w, "SrchFunction", args, n);
 	XtAddCallback (func_w, XmNactivateCallback, srch_funcactivate_cb, NULL);
+	wtip (func_w, "Function to solve: enter fields manually, or via buttons");
 	XtManageChild (func_w);
 
 	/* compiler message label */
@@ -321,8 +334,9 @@ srch_create_form()
 	n = 0;
 	XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
 	err_w = XmCreateLabel (rc_w, "SrchErrsL", args, n);
-	XtManageChild (err_w);
 	set_xmstring (err_w, XmNlabelString, " ");
+	wtip (err_w, "Result from compiling the search Function");
+	XtManageChild (err_w);
 
 	/* use-fields button */
 
@@ -331,6 +345,7 @@ srch_create_form()
 	XtSetArg (args[n], XmNlabelString, str); n++;
 	field_w = XmCreateToggleButton (rc_w, "SrchFEnable", args, n);
 	XtAddCallback (field_w, XmNvalueChangedCallback, srch_fields_cb, 0);
+	wtip (field_w, "When on, data fields eligible for use in Function are selectable buttons");
 	XtManageChild (field_w);
 	XmStringFree(str);
 
@@ -340,8 +355,9 @@ srch_create_form()
 	XtSetArg (args[n], XmNshowAsDefault, True); n++;
 	compile_w = XmCreatePushButton (rc_w, "Compile", args, n);
 	XtAddCallback (compile_w, XmNactivateCallback, srch_compile_cb, 0);
-	XtManageChild (compile_w);
 	set_something (srchform_w, XmNdefaultButton, (XtArgVal)w);
+	wtip (compile_w, "Press to compile the Function");
+	XtManageChild (compile_w);
 
 	/* create a separator */
 
@@ -351,7 +367,7 @@ srch_create_form()
 
 	/* create goal radio box and its toggle buttons */
 
-	str = XmStringCreate ("Search goal:", XmSTRING_DEFAULT_CHARSET);
+	str = XmStringCreate ("Goal:", XmSTRING_DEFAULT_CHARSET);
 	n = 0;
 	XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
 	XtSetArg (args[n], XmNlabelString, str); n++;
@@ -371,6 +387,8 @@ srch_create_form()
 		w = XmCreateToggleButton (rb_w, "SrchRBTB", args, n);
 		XtAddCallback (w, XmNvalueChangedCallback, srch_goal_cb,
 						    (XtPointer)rbtb[i].cb_data);
+		if (rbtb[i].tip)
+		    wtip (w, rbtb[i].tip);
 		XtManageChild (w);
 		XmStringFree(str);
 	    }
@@ -394,6 +412,7 @@ srch_create_form()
 	    acc_w = XmCreatePushButton (f_w, "SrchAcc", args, n);
 	    XtAddCallback (acc_w, XmNactivateCallback, srch_acc_cb, 0);
 	    f_time (acc_w, tmlimit);
+	    wtip (acc_w, "Set desired time accuracy of solution");
 	    XtManageChild (acc_w);
 
 	    str = XmStringCreate("Desired accuracy:", XmSTRING_DEFAULT_CHARSET);
@@ -403,17 +422,12 @@ srch_create_form()
 	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
 	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_WIDGET); n++;
 	    XtSetArg (args[n], XmNrightWidget, acc_w); n++;
+	    XtSetArg (args[n], XmNrightOffset, 5); n++;
 	    XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
 	    XtSetArg (args[n], XmNlabelString, str); n++;
 	    w = XmCreateLabel (f_w, "SrchAccL", args, n);
-	    XmStringFree (str);
 	    XtManageChild (w);
-
-	/* create a separator */
-
-	n = 0;
-	w = XmCreateSeparator (rc_w, "Sep4", args, n);
-	XtManageChild (w);
+	    XmStringFree (str);
 
 	/* current search value in a form */
 
@@ -426,8 +440,9 @@ srch_create_form()
 	    XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
 	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
 	    valu_w = XmCreateLabel (f_w, "SrchValu", args, n);
-	    XtManageChild (valu_w);
 	    set_xmstring (valu_w, XmNlabelString, "");
+	    wtip (valu_w, "Value of Function evaluated at current time");
+	    XtManageChild (valu_w);
 
 	    str = XmStringCreate ("Current value: ", XmSTRING_DEFAULT_CHARSET);
 	    n = 0;
@@ -448,13 +463,14 @@ srch_create_form()
 	    set_something (valu_w, XmNuserData, (XtArgVal)srchvname);
 
 	/* button to select using for plotting. 
-	 * ONLY MANAGED WHEN WE ARE TOLD TO SELECT FOR PLOTTING.
+	 * Only managed when we are told to select for plotting or listing.
 	 */
 
 	n = 0;
 	use_w = XmCreatePushButton (rc_w, "SrchUse", args, n);
 	XtAddCallback (use_w, XmNactivateCallback, srch_use_cb, 0);
 	set_xmstring (use_w, XmNlabelString, "Use for plotting");
+	wtip (use_w, "Press to use each Function evaluation in a plot or listing");
 
 	/* create a separator */
 
@@ -477,8 +493,9 @@ srch_create_form()
 	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
 	    XtSetArg (args[n], XmNrightPosition, 3); n++;
 	    w = XmCreatePushButton(f_w, "Close", args,n);
-	    XtManageChild (w);
 	    XtAddCallback(w, XmNactivateCallback, srch_close_cb, 0);
+	    wtip (w, "Close this dialog (but continue solving if active)");
+	    XtManageChild (w);
 
 	    n = 0;
 	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
@@ -488,8 +505,9 @@ srch_create_form()
 	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
 	    XtSetArg (args[n], XmNrightPosition, 6); n++;
 	    help_w = XmCreatePushButton(f_w, "Help", args, n);
-	    XtManageChild (help_w);
 	    XtAddCallback(help_w, XmNactivateCallback, srch_help_cb, 0);
+	    wtip (help_w, "More detailed usage information");
+	    XtManageChild (help_w);
 }
 
 /* return True whenever the srch control menu is up */
@@ -687,15 +705,17 @@ XtPointer call;
 	    Arg args[20];
 	    int n;
 
-	    str = XmStringCreate ("search accuracy (h:m:s):",
+	    str = XmStringCreate ("Accuracy (h:m:s):",
 						    XmSTRING_DEFAULT_CHARSET);
-	    title = XmStringCreate ("xephem Search Accuracy",
+	    title = XmStringCreate ("xephem Solver Accuracy",
 						    XmSTRING_DEFAULT_CHARSET);
 	    n = 0;
 	    XtSetArg(args[n], XmNdefaultPosition, False);  n++;
 	    XtSetArg(args[n], XmNselectionLabelString, str);  n++;
 	    XtSetArg(args[n], XmNdialogTitle, title);  n++;
+	    XtSetArg (args[n], XmNcolormap, xe_cm); n++;
 	    dw = XmCreatePromptDialog(toplevel_w, "Accuracy", args, n);
+	    set_something (dw, XmNcolormap, (XtArgVal)xe_cm);
 	    XtAddCallback (dw, XmNokCallback, srch_acc_ok_cb, NULL);
 	    XtAddCallback (dw, XmNmapCallback, prompt_map_cb, NULL);
 	    XmStringFree (str);
@@ -747,7 +767,7 @@ XtPointer client;
 XtPointer call;
 {
 	static char *msg =
-	    "You must first successfully compile a search function and\n select a goal algorithm before searching may be enabled.";
+	    "You must first successfully compile a search Function and\n select a Goal algorithm before searching may be enabled.";
 
 	if (XmToggleButtonGetState(w)) {
 	    /* insure there is a valid function and goal algorithm selected
@@ -755,11 +775,11 @@ XtPointer call;
 	     */
 	    if (!prog_isgood()) {
 		XmToggleButtonSetState(on_w, False, True/* invoke cb */);
-		xe_msg (msg, 0);
+		xe_msg (msg, 1);
 		 
 	    } else if (srch_f == 0) {
 		XmToggleButtonSetState(on_w, False, True/* invoke cb */);
-		xe_msg (msg, 0);
+		xe_msg (msg, 1);
 	    } else {
 		srch_tmscalled = 0;
 		/* turning on searhing so as a courtesy turn off the
