@@ -1,23 +1,8 @@
+#include "P_.h"
 #include "astro.h"
 
-#if defined(__STDC__) || defined(__cplusplus)
-#define P_(s) s
-#else
-#define P_(s) ()
-#endif
-
-extern void cal_mjd P_((int mn, double dy, int yr, double *Mjd));
-extern void mjd_cal P_((double Mjd, int *mn, double *dy, int *yr));
-extern void range P_((double *v, double r));
-
-void utc_gst P_((double mjd, double utc, double *gst));
-void gst_utc P_((double mjd, double gst, double *utc));
 static double tnaught P_((double mjd));
 
-#undef P_
-
-
-
 /* given a modified julian date, mjd, and a universally coordinated time, utc,
  * return greenwich mean siderial time, *gst.
  * N.B. mjd must be at the beginning of the day.
