@@ -1,5 +1,5 @@
 /* main() for xephem.
- * Copyright (c) 1990,1991,1992,1993,1994,1995 by Elwood Charles Downey
+ * Copyright (c) 1990-1997 by Elwood Charles Downey
  * Permission is granted to make and distribute copies of this program free of
  * charge, provided the copyright notice and this permission notice are
  * preserved on all copies.  All other rights reserved.  No representation is
@@ -35,31 +35,45 @@
 #include <Xm/CascadeB.h>
 #include <Xm/Form.h>
 #include <Xm/Separator.h>
-#include <Xm/MainW.h>
 #include <Xm/RowColumn.h>
 #include <Xm/ToggleB.h>
 
+#if XmVersion >= 1002
+#include <Xm/RepType.h>
+#endif /* XmVersion >= 1002 */
+
 
 #include "P_.h"
+#include "patchlevel.h"
 
+#define	NMINCOL	100	/* min colors we want for the main colormap */
+
+extern Colormap checkCM P_((Colormap cm, int nwant));
+extern char *getXRes P_((char *name));
 extern void c_manage P_((void));
 extern void db_manage P_((void));
+extern void dm_create_form P_((void));
 extern void dm_manage P_((void));
 extern void e_manage P_((void));
+extern void fs_manage P_((void));
 extern void hlp_dialog P_((char *tag, char *deflt[], int ndeflt));
 extern void jm_manage P_((void));
 extern void lst_manage P_((void));
 extern void m_manage P_((void));
 extern void mars_manage P_((void));
-extern void mm_manage P_((Widget main_window));
+extern void mm_external P_((void));
+extern void mm_create P_((Widget mainrc));
+extern void mm_go_cb P_((Widget w, XtPointer client, XtPointer call));
 extern void mm_reset P_((void));
 extern void msg_manage P_((void));
 extern void obj_manage P_((void));
 extern void plot_manage P_((void));
-extern void pref_create_pulldown P_((Widget menu_bar));
+extern void pref_create_pulldown P_((Widget mb_w));
 extern void query P_((Widget tw, char *msg, char *label1, char *label2,
-    char *label3, void (*func1)(), void (*func2)(), void (*func3)()));
+    char *label3, void (*func1)(void), void (*func2)(void),
+    void (*func3)(void)));
 extern void pm_manage P_((void));
+extern void set_something P_((Widget w, char *resource, XtArgVal value));
 extern void set_xmstring P_((Widget w, char *resource, char *txt));
 extern void sm_manage P_((void));
 extern void srch_manage P_((void));
@@ -67,10 +81,12 @@ extern void ss_manage P_((void));
 extern void sv_manage P_((void));
 extern void version P_((void));
 extern void watch_cursor P_((int want));
+extern void wtip P_((Widget w, char *tip));
 
 /* these are used to describe and semi-automate making the main pulldown menus
  */
 typedef struct {
+    char *tip;		/* tip text, if any */
     char *name;		/* button name, or separator name if !cb */
     			/* N.B. watch for a few special cases */
     char *label;	/* button label (use name if 0) */
@@ -81,6 +97,7 @@ typedef struct {
     XtPointer client;	/* button callback client data */
 } ButtonInfo;
 typedef struct {
+    char *tip;		/* tip text, if any */
     char *cb_name;	/* cascade button name */
     char *cb_label;	/* cascade button label (use name if 0) */
     char cb_mne;	/* cascade button mnemonic */
@@ -93,28 +110,32 @@ typedef struct {
 #define	SEPARATOR_N	"MainSep"	/* special ButtonInfo->name */
 
 
-static Widget make_main_window P_((void));
+static void set_title P_((void));
+static void make_main_window P_((void));
 static Widget make_pulldown P_((Widget mb_w, PullDownMenu *pdmp));
-static void setup_window_properties P_((Widget w));
+static void setup_icon P_((void));
 static void m_activate_cb P_((Widget w, XtPointer client, XtPointer call));
 static void make_examples_pullright P_((Widget pulldown_w, Widget cascade_w));
 static void examples_cb P_((Widget w, XtPointer client, XtPointer call));
 static void alldone P_((void));
+static void x_quit P_((void));
 
 
 /* client arg to m_activate_cb().
  */
 enum {
-    PROGRESS, QUIT, XRESET, MSGTXT,
+    PROGRESS, QUIT, XRESET, MSGTXT, EXTIN,
     DATA, MOON, EARTH, MARS, JUPMOON, SATMOON, SKYVIEW, SOLARSYS,
     PLOT, LIST, SEARCH,
-    DB, OBJS, CLOSEOBJS,
-    ABOUT, REFERENCES, INTRO, MAINMENU, CONFIGFILE, OPERATION, DATETIME, NOTES
+    DB, OBJS, FIELDSTARS, CLOSEOBJS, 
+    ABOUT, REFERENCES, INTRO, MAINMENU, OPERATION, DATETIME, NOTES
 };
 
 Widget toplevel_w;
+#define	XtD	XtDisplay(toplevel_w)
+Colormap xe_cm;
 XtAppContext xe_app;
-char *myclass = "XEphem";
+static char *myclass = "XEphem";
 
 #define xephem_width 50
 #define xephem_height 50
@@ -153,26 +174,23 @@ static unsigned char xephem_bits[] = {
 static String fallbacks[] = {
     "XEphem*AdpDispl.columns: 5",
     "XEphem*AdpDispl.value: 0.0",
-    "XEphem*Algorithms.Accurate.set: True",
     "XEphem*CloseList*textString: work/closelist.txt",
     "XEphem*CloseObjs*List.doubleClickInterval: 500",
     "XEphem*CloseObjs*List.visibleItemCount: 10",
     "XEphem*CloseObjs*Mag.value: 20.0",
     "XEphem*CloseObjs*OmitFixedPairs.set: True",
     "XEphem*CloseObjs*Sep.value: 2.0",
-    "XEphem*CloseObjs*Topo.set: True",
     "XEphem*DBManage*NoDups.set: True",
     "XEphem*DataSelMiscCols.Alt.set: True",
     "XEphem*DataSelMiscCols.Az.set: True",
     "XEphem*DataSelMiscCols.Cns.set: True",
     "XEphem*DataSelMiscCols.Dec.set: True",
-    "XEphem*DataSelMiscCols.R_A.set: True",
+    "XEphem*DataSelMiscCols.RA.set: True",
     "XEphem*DataSelRisetCols*Limb.set: True",
     "XEphem*DataSelRisetCols.RiseTm.set: True",
     "XEphem*DataSelRisetCols.SetTm.set: True",
     "XEphem*DataSelRows.Moon.set: True",
     "XEphem*DataSelRows.Sun.set: True",
-    "XEphem*DateFormat.MDY.set: True",
     "XEphem*Earth*Map.height: 300",
     "XEphem*Earth*Map.width: 500",
     "XEphem*Earth*cylindrical.set: True",
@@ -181,10 +199,15 @@ static String fallbacks[] = {
     "XEphem*Earth*sites.set: True",
     "XEphem*Earth*sunlight.set: True",
     "XEphem*Earth*trail.set: True",
-    "XEphem*Help*ScrolledText.columns: 80",
-    "XEphem*Help*ScrolledText.rows: 24",
+    "XEphem*ExtFile*textString: work/external.txt",
+    "XEphem*FieldStars*GSCCDDirectory.value: /cdrom",
+    "XEphem*FieldStars*GSCCacheDirectory.value: auxil/gsccache",
+    "XEphem*FieldStars*GSCNetHost.value: gastro23.physics.uiowa.edu",
+    "XEphem*FieldStars*NoDups.set: True",
+    "XEphem*FieldStars*PPMFilename.value: auxil/ppm.xe",
+    "XEphem*Help*HelpText.columns: 80",
+    "XEphem*Help*HelpText.rows: 24",
     "XEphem*Help.verticalSpacing: 20",
-    "XEphem*Jupiter*BrightMoons.set: True",
     "XEphem*Jupiter*CtlForm.verticalSpacing: 5",
     "XEphem*Jupiter*LimMag.value: 10",
     "XEphem*Jupiter*Map.height: 200",
@@ -193,28 +216,59 @@ static String fallbacks[] = {
     "XEphem*Jupiter*Tags.set: True",
     "XEphem*List*Filename.value: work/ephem.lst",
     "XEphem*ListRC.spacing: 5",
-    "XEphem*Mars*Map.height: 300",
-    "XEphem*Message*ScrolledText.columns: 80",
-    "XEphem*Message*ScrolledText.rows: 10",
+    "XEphem*Mars*LowPrec.set: True",
+    "XEphem*Mars.height: 400",
+    "XEphem*Mars.width: 400",
+    "XEphem*MarsStats*Seeing.maximum: 5",
+    "XEphem*MarsStats*Seeing.minimum: 0",
+    "XEphem*MarsStats*Seeing.value: 0",
+    "XEphem*Message*Messages.columns: 80",
+    "XEphem*Message*Messages.rows: 10",
     "XEphem*Message.verticalSpacing: 20",
-    "XEphem*MessageBell.On.set: True",
+    "XEphem*Moon*FakeStars.set: True",
+    "XEphem*Moon*Scale3X.set: True",
+    "XEphem*Moon*Umbra.set: True",
+    "XEphem*Moon.height: 400",
+    "XEphem*Moon.width: 400",
+    "XEphem*MoonES*Earthshine.value: 4",
     "XEphem*Plot*Filename.value: work/ephem.plt",
+    "XEphem*PlotDA.foreground: white",
     "XEphem*PlotDA.height: 400",
     "XEphem*PlotDA.width: 400",
     "XEphem*PlotRC.spacing: 5",
-    "XEphem*Precision.Low.set: True",
-    "XEphem*Precision.Low.set: True",
+    "XEphem*Precision.Hi.set: True",
+    "XEphem*Print*Filename.value: work/xephem.ps",
+    "XEphem*Print*Grayscale.set: True",
+    "XEphem*Print*PrintCmd.value: lpr",
+    "XEphem*Print*Save.set: True",
+    "XEphem*Progress*DA.foreground: white",
     "XEphem*Progress*DA.height: 50",
     "XEphem*Progress*DA.width: 200",
-    "XEphem*Saturn*BrightMoons.set: True",
     "XEphem*Saturn*CtlForm.verticalSpacing: 5",
     "XEphem*Saturn*LimMag.value: 10",
     "XEphem*Saturn*Map.height: 200",
     "XEphem*Saturn*Map.width: 300",
     "XEphem*Saturn*Scale.value: 50",
     "XEphem*Saturn*Tags.set: True",
+    "XEphem*SiteSL.visibleItemCount: 10",
+    "XEphem*SkyEyep*Elliptical.set: True",
+    "XEphem*SkyEyep*EyepH.decimalPoints: 1",
+    "XEphem*SkyEyep*EyepH.maximum: 50",
+    "XEphem*SkyEyep*EyepH.minimum: 1",
+    "XEphem*SkyEyep*EyepH.value: 7",
+    "XEphem*SkyEyep*EyepW.decimalPoints: 1",
+    "XEphem*SkyEyep*EyepW.maximum: 50",
+    "XEphem*SkyEyep*EyepW.minimum: 1",
+    "XEphem*SkyEyep*EyepW.value: 7",
+    "XEphem*SkyEyep*Lock.set: True",
+    "XEphem*SkyEyep*Rectangular.set: False",
+    "XEphem*SkyFITS*AutoName.set: True",
+    "XEphem*SkyFITS*Header.columns: 40",
+    "XEphem*SkyFITS*Header.rows: 8",
+    "XEphem*SkyFITS*SaveFN.value: work/xxx.fts",
+    "XEphem*SkyFITS.x: 600",
+    "XEphem*SkyFITS.y: 10",
     "XEphem*SkyFilter*Binary.set: True",
-    "XEphem*SkyFilter*BrightMagScale.value: -28",
     "XEphem*SkyFilter*BrightNeb.set: True",
     "XEphem*SkyFilter*ClInNeb.set: True",
     "XEphem*SkyFilter*DarkNeb.set: True",
@@ -226,6 +280,7 @@ static String fallbacks[] = {
     "XEphem*SkyFilter*GalClusters.set: True",
     "XEphem*SkyFilter*GlobularCl.set: True",
     "XEphem*SkyFilter*Hyperbolic.set: True",
+    "XEphem*SkyFilter*MagStep.value: 1",
     "XEphem*SkyFilter*Multiple.set: True",
     "XEphem*SkyFilter*OpenCl.set: True",
     "XEphem*SkyFilter*Parabolic.set: True",
@@ -242,22 +297,30 @@ static String fallbacks[] = {
     "XEphem*SkyFilter*SymbolDA.width: 25",
     "XEphem*SkyFilter*Undefined.set: True",
     "XEphem*SkyFilter*Variable.set: True",
+    "XEphem*SkyFilter.x: 400",
+    "XEphem*SkyFilter.y: 10",
     "XEphem*SkyList*textString: work/skylist.txt",
-    "XEphem*SkyView*AltAzMode.set: True",
-    "XEphem*SkyView*AltDecScale.value: 90",
-    "XEphem*SkyView*AzRAScale.maximum: 360",
-    "XEphem*SkyView*AzRAScale.value: 180",
-    "XEphem*SkyView*CnsFigures.set: True",
-    "XEphem*SkyView*CnsNames.set: True",
-    "XEphem*SkyView*FOVScale.scaleMultiple: 20",
-    "XEphem*SkyView*FOVScale.value: 180",
-    "XEphem*SkyView*JustDots.set: True",
+    "XEphem*SkyOps*AltAzMode.set: True",
+    "XEphem*SkyOps*AltDecScale.value: 900",
+    "XEphem*SkyOps*AutoMag.set: True",
+    "XEphem*SkyOps*CnsFigures.set: True",
+    "XEphem*SkyOps*CnsNames.set: True",
+    "XEphem*SkyOps*JustDots.set: True",
+    "XEphem*SkyOps*LblFldStars.set: True",
+    "XEphem*SkyOps*LblNames.set: True",
+    "XEphem*SkyOps*MagScale.set: True",
+    "XEphem*SkyOps.x: 500",
+    "XEphem*SkyOps.y: 10",
+    "XEphem*SkyView*AzRAScale.value: 1800",
+    "XEphem*SkyView*FOVScale.scaleMultiple: 1",
+    "XEphem*SkyView*FOVScale.value: 1800",
     "XEphem*SkyView*Map.height: 500",
     "XEphem*SkyView*Map.width: 500",
     "XEphem*SolarSystem*Ecliptic.set: True",
     "XEphem*SolarSystem*HLatScale.value: 90",
     "XEphem*SolarSystem*Legs.set: True",
     "XEphem*SolarSystem*Names.set: True",
+    "XEphem*SolarSystem*SolarDA.foreground: white",
     "XEphem*SolarSystem*SolarDA.height: 300",
     "XEphem*SolarSystem*SolarDA.width: 300",
     "XEphem*SolarSystem*Trails.set: True",
@@ -265,10 +328,8 @@ static String fallbacks[] = {
     "XEphem*SolarSystem.HLatScale.value: 90",
     "XEphem*SrchRC.spacing: 5",
     "XEphem*StdRefr.set: True",
-    "XEphem*TZone.Local.set: True",
-    "XEphem*Topocentric.set: True",
-    "XEphem*TrailPrompt*CustomInterval.value: 0:30",
-    "XEphem*Units.English.set: True",
+    "XEphem*Trails*CustomInterval.value: 0:30",
+    "XEphem*Trails*NTicks.maximum: 200",
     "XEphem*XmFrame.marginHeight: 0",
     "XEphem*XmFrame.marginWidth: 0",
     "XEphem*background: black",
@@ -279,42 +340,65 @@ static String fallbacks[] = {
     "XEphem*marginHeight: 1",
     "XEphem*marginWidth: 1",
     "XEphem*spacing: 1",
+    "XEphem.CnsFont: -*-times-medium-i-*-*-10-*-*-*-*-*-*-*",
     "XEphem.DBFIFO: fifos/xephem_db_fifo",
     "XEphem.DBdirectory: edb",
-    "XEphem.DBinitialFiles: basic.edb",
+    "XEphem.DBinitialFiles: Messier.edb YBS.edb",
     "XEphem.DBpattern:  *.edb",
     "XEphem.Elevation: 800",
     "XEphem.Epoch: 2000",
+    "XEphem.FITSdirectory: work",
+    "XEphem.FITSpattern:  *.f*ts",
+    "XEphem.GlassBorderColor: white",
     "XEphem.HELPFILE: auxil/xephem.hlp",
-    "XEphem.LT: Now",
+    "XEphem.JupiterGRSColor: black",
+    "XEphem.JupiterGRSLongitude: 42",
     "XEphem.Lat: 40:00:00",
     "XEphem.Long: 90:00:00",
+    "XEphem.MarsAnnotColor: white",
+    "XEphem.MarsDB: auxil/mars_db",
+    "XEphem.MarsFile: auxil/marsmap.fts",
+    "XEphem.MoonAnnotColor: white",
+    "XEphem.MoonDB: auxil/moon_db",
+    "XEphem.MoonFile: auxil/moon.fts",
+    "XEphem.MoonShadowDim: 4",
     "XEphem.NSteps: 1",
     "XEphem.Pause: 0",
     "XEphem.Pressure: 29.5",
+    "XEphem.QuickQuit: False",
     "XEphem.SITESFILE: auxil/xephem_sites",
     "XEphem.SKYINFIFO: fifos/xephem_in_fifo",
     "XEphem.SKYLOCFIFO: fifos/xephem_loc_fifo",
     "XEphem.SKYOUTFIFO: fifos/xephem_out_fifo",
+    "XEphem.SkyEyePColor: black",
+    "XEphem.SkyEyePDiam: 1",
     "XEphem.StepSize: RTC",
     "XEphem.Temp: 60",
     "XEphem.TwilightDip: 18",
     "XEphem.allowShellResize: True",
-    "XEphem.viewsFont: fixed",
-    "XEphen.JupiterGRSColor: Black",
-    "XEphen.JupiterGRSLongitude: 42",
+    "XEphem.install: guess",
+    "XEphem.trailsFont: -*-helvetica-bold-r-*-*-10-*-*-*-*-*-*-*",
+    "XEphem.viewsFont: 6x12",
+    "XEphem.viewsGreekFont: -*-symbol-*-*-*-*-12-*-*-*-*-*-*-*",
     NULL
 };
 
+static XrmOptionDescRec options[] = {
+    {"-install", ".install", XrmoptionSepArg, NULL},
+};
+
 int
 main(argc, argv)
 int argc;
 char *argv[];
 {
-	Widget main_window_w;
+	toplevel_w = XtAppInitialize (&xe_app, myclass, options,
+			    XtNumber(options), &argc, argv, fallbacks, NULL, 0);
+	set_title();
 
-	toplevel_w = XtAppInitialize (&xe_app, myclass, NULL, 0,
-					    &argc, argv, fallbacks, NULL, 0);
+	/* load xe_cm and toplevel_w with default or private colormap */
+	xe_cm = checkCM (DefaultColormap(XtD,DefaultScreen(XtD)), NMINCOL);
+	set_something (toplevel_w, XmNcolormap, (XtArgVal)xe_cm);
 
 #ifdef DO_EDITRES
 	XtAddEventHandler (toplevel_w, (EventMask)0, True,
@@ -322,6 +406,13 @@ char *argv[];
 	xe_msg ("Can editres!\n", 0);
 #endif
 
+#if XmVersion >= 1002
+	/* install converter so tearOffModel can be set in resource files
+	 * to TEAR_OFF_{EN,DIS}ABLED.
+	 */
+	XmRepTypeInstallTearOffModelConverter();
+#endif
+
 	/* ignore FPE, though we do have a matherr() handler in misc.c. */
 	(void) signal (SIGFPE, SIG_IGN);
 
@@ -330,15 +421,19 @@ char *argv[];
 	(void) signal (SIGPIPE, SIG_IGN);
 #endif
 
-	/* make the main menu bar and form (other stuff is in mainmenu.c) */
-	main_window_w = make_main_window ();
-
-	XtManageChild(main_window_w);
-	XtRealizeWidget(toplevel_w);
-
 	/* connect up the icon pixmap */
-	setup_window_properties (toplevel_w);
+	setup_icon ();
 
+	/* make the main menu bar and form (other stuff is in mainmenu.c) */
+	make_main_window ();
+
+	/* define the data table, but don't manage it.
+	 * we do this up front to avoid the delay on its first use later.
+	 */
+	dm_create_form();
+
+	/* here we go */
+	XtRealizeWidget(toplevel_w);
 	XtAppMainLoop(xe_app);
 
 	printf ("XtAppMainLoop returned :-)\n");
@@ -352,8 +447,8 @@ Cursor c;
 {
 	Window win;
 
-	if (toplevel_w && (win = XtWindow(toplevel_w))) {
-	    Display *dsp = XtDisplay(toplevel_w);
+	if (toplevel_w && (win = XtWindow(toplevel_w)) != 0) {
+	    Display *dsp = XtD;
 	    if (c)
 		XDefineCursor (dsp, win, c);
 	    else
@@ -361,83 +456,131 @@ Cursor c;
 	}
 }
 
+static void
+set_title()
+{
+	char title[100];
+
+	(void) sprintf (title, "xephem %s", PATCHLEVEL);
+	set_something (toplevel_w, XmNtitle, (XtArgVal) title);
+}
+
 /* put together the menu bar, the main form, and fill in the form with the
  * initial xephem buttons.
  */
-static Widget
+static void
 make_main_window ()
 {
 	static ButtonInfo file_buttons[] = {
-	    {"Reset", 0, 0, 0, 'R', m_activate_cb, (XtPointer)XRESET},
-	    {"Progress Meter", 0, "Ctrl<Key>p:", "Ctrl+p", 'P', m_activate_cb,
-							(XtPointer)PROGRESS},
-	    {"Messages", 0, "Ctrl<Key>m:", "Ctrl+m", 'M', m_activate_cb,
+	    {"Restore many Main values to original settings",
+		"Reset", "Reset", 0, 0, 'R', m_activate_cb, (XtPointer)XRESET},
+	    {NULL, SEPARATOR_N},
+	    {"Display a dialog containing supporting informational messages",
+	        "Messages", "Messages...", 0, 0, 'M', m_activate_cb,
 							    (XtPointer)MSGTXT},
-	    {SEPARATOR_N},
-	    {"Quit", 0, "Ctrl<Key>d:", "Ctrl+d", 'Q', m_activate_cb,
+	    {"Set up to run xephem Time and Location from an external file",
+		"ExtIn", "External Time/Loc...", 0, 0, 'E', m_activate_cb,
+							    (XtPointer)EXTIN},
+	    {"Display a simple Progress meter",
+		"Progress", "Progress Meter...", 0, 0, 'P', m_activate_cb,
+							   (XtPointer)PROGRESS},
+	    {NULL, SEPARATOR_N},
+	    {"Start and stop the main execution loop",
+		"Update", "Update", 0, 0, 'U', mm_go_cb, (XtPointer)NULL},
+	    {NULL, SEPARATOR_N},
+	    {"Exit xephem",
+		"Quit", "Quit...", "Ctrl<Key>d", "Ctrl+d", 'Q', m_activate_cb,
 							    (XtPointer)QUIT}
 	};
 	static ButtonInfo view_buttons[] = {
-	    {"GenData", "Data Table...", 0, 0, 'D', m_activate_cb,
+	    {"Display many statistics for any objects",
+		"GenData", "Data Table...", 0, 0, 'D', m_activate_cb,
 							    (XtPointer)DATA},
-	    {SEPARATOR_N},
-	    {"Moon", "Moon...", 0, 0, 'M', m_activate_cb, (XtPointer)MOON},
-	    {"Earth", "Earth...", 0, 0, 'E', m_activate_cb, (XtPointer)EARTH},
-	    {"Mars", "Mars...", 0, 0, 'r', m_activate_cb, (XtPointer)MARS},
-	    {"Jupiter", "Jupiter...", 0, 0, 'J', m_activate_cb,
+	    {NULL, SEPARATOR_N},
+	    {"Display an image of Moon and supporting information",
+		"Moon", "Moon...", 0, 0, 'M', m_activate_cb, (XtPointer)MOON},
+	    {"Display a map of Earth and supporting information",
+		"Earth", "Earth...", 0, 0, 'E', m_activate_cb,(XtPointer)EARTH},
+	    {"Display an image of Mars and supporting information",
+		"Mars", "Mars...", 0, 0, 'r', m_activate_cb, (XtPointer)MARS},
+	    {"Display a schematic of Jupiter, GRS, its moons, and other info",
+		"Jupiter", "Jupiter...", 0, 0, 'J', m_activate_cb,
 							(XtPointer)JUPMOON},
-	    {"Saturn", "Saturn...",0,0, 'a', m_activate_cb, (XtPointer)SATMOON},
-	    {SEPARATOR_N},
-	    {"SkyV", "Sky View...",0,0, 'V', m_activate_cb, (XtPointer)SKYVIEW},
-	    {"SolSys", "Solar system...", 0, 0, 'S', m_activate_cb,
-							(XtPointer)SOLARSYS}
+	    {"Display schematic of Saturn, its rings and moons, and other info",
+		"Saturn", "Saturn...", 0, 0, 'a', m_activate_cb,
+							(XtPointer)SATMOON},
+	    {NULL, SEPARATOR_N},
+	    {"Display a full-featured map of the night sky",
+		"SkyV", "Sky View...", 0, 0, 'V', m_activate_cb,
+							    (XtPointer)SKYVIEW},
+	    {"Display a map of the solar system",
+		"SolSys", "Solar System...", 0, 0, 'S', m_activate_cb,
+	    						(XtPointer)SOLARSYS}
 	};
 	static ButtonInfo ctrl_buttons[] = {
-	    {"Plot", "Plot...", 0, 0, 'P', m_activate_cb, (XtPointer)PLOT},
-	    {"Listing", "Listing...", 0,0, 'L', m_activate_cb, (XtPointer)LIST},
-	    {"Search", "Search...", 0,0, 'S', m_activate_cb, (XtPointer)SEARCH}
+	    {"Capture any xephem values for making and displaying plots",
+		"Plot", "Plot...", 0, 0, 'P', m_activate_cb, (XtPointer)PLOT},
+	    {"Capture any xephem values in a tabular text file",
+		"List", "List...", 0, 0, 'L', m_activate_cb, (XtPointer)LIST},
+	    {"Build any equation of xephem fields and solve for min, max or 0",
+		"Solve", "Solve...", 0, 0, 'S', m_activate_cb,(XtPointer)SEARCH}
 	};
 	static ButtonInfo objs_buttons[] = {
-	    {"DataBase", "Data Base...",0, 0, 'D', m_activate_cb,
+	    {"Load .edb files into memory, list current memory, or delete",
+		"DataBase", "Load and Delete...",0, 0, 'D', m_activate_cb,
 								(XtPointer)DB},
-	    {"ObjX_Y", "ObjX, ObjY...", 0, 0, 'O', m_activate_cb,
+	    {"Search memory for an object, view definitions, assign ObjXYZ",
+		"ObjX_Y", "Search and ObjXYZ...", 0, 0,'S', m_activate_cb,
 							    (XtPointer)OBJS},
-	    {"CloseObjs", "Close Objs...", 0, 0, 'C', m_activate_cb,
+	    {"Define where GSC and PPM catalogs are to be found",
+		"FieldStars", "Setup Field Stars...", 0, 0,'F', m_activate_cb,
+	    						(XtPointer)FIELDSTARS},
+	    {"Find all pairs of close objects in memory",
+		"CloseObjs", "Find Close Pairs...", 0, 0, 'C', m_activate_cb,
 							(XtPointer)CLOSEOBJS}
 	};
 	static ButtonInfo help_buttons[] = {
-	    {"Introduction", "Introduction...", 0, 0, 'I', m_activate_cb,
+	    {"Overall features of xephem",
+		"Introduction", "Introduction...", 0, 0, 'I', m_activate_cb,
 							    (XtPointer)INTRO},
-	    {SEPARATOR_N},
-	    {"onInitialization", "on Initialization...", 0, 0, 'z', 
-					m_activate_cb, (XtPointer)CONFIGFILE},
-	    {"onMainMenu", "on Main Menu...", 0, 0, 'M', m_activate_cb,
+	    {NULL, SEPARATOR_N},
+	    {"Description of fields within this Main xephem menu",
+		"onMainMenu", "on Main Menu...", 0, 0, 'M', m_activate_cb,
 							(XtPointer)MAINMENU},
-	    {"onOperation", "on Operation...", 0, 0, 'O', m_activate_cb,
+	    {"How to control xephem's running behavior, including looping",
+		"onOperation", "on Operation...", 0, 0, 'O', m_activate_cb,
 							(XtPointer)OPERATION},
-	    {"onTriad", "on Triad formats...", 0, 0, 'T', m_activate_cb,
+	    {"Shortcuts to setting date and time formats",
+		"onTriad", "on Triad formats...", 0, 0, 'T', m_activate_cb,
 							(XtPointer)DATETIME},
-	    {SEPARATOR_N},
-	    {EXAMPLES_N, "Examples...", 0, 0, 'E'},
-	    {"onReferences", "Credits...",  0, 0, 'C', m_activate_cb,
+	    {NULL, SEPARATOR_N},
+	    {NULL, EXAMPLES_N, "Examples", 0, 0, 'E'},
+	    {"Credits, references and other kudos.",
+		"onReferences", "Credits...",  0, 0, 'C', m_activate_cb,
 							(XtPointer)REFERENCES},
-	    {"Notes", "Notes...", 0, 0, 'N', m_activate_cb, (XtPointer)NOTES},
-	    {"About", "About...", 0, 0, 'A', m_activate_cb,
-							    (XtPointer)ABOUT}
+	    {"A few supporting issues",
+		"Notes", "Notes...", 0, 0, 'N', m_activate_cb,(XtPointer)NOTES},
+	    {"Version, copyright, logo",
+		"About", "About...", 0, 0, 'A', m_activate_cb, (XtPointer)ABOUT}
 	};
 	static PullDownMenu file_pd =
-	    {"File", 0, 'F', "file_pd", file_buttons, XtNumber(file_buttons)};
+	    {"Overall control functions",
+		"File", 0, 'F', "file_pd", file_buttons,XtNumber(file_buttons)};
 	static PullDownMenu view_pd =
-	    {"View", 0, 'V', "view_pd", view_buttons, XtNumber(view_buttons)};
+	    {"Major display options",
+		"View", 0, 'V', "view_pd", view_buttons,XtNumber(view_buttons)};
 	static PullDownMenu ctrl_pd =
-	    {"Control", 0,'C', "ctrl_pd", ctrl_buttons, XtNumber(ctrl_buttons)};
+	    {"Supporting analysis tools",
+		"Tools", 0,'T',"ctrl_pd",ctrl_buttons,XtNumber(ctrl_buttons)};
 	static PullDownMenu objs_pd =
-	    {"Objects", 0,'O', "objs_pd", objs_buttons, XtNumber(objs_buttons)};
+	    {"Add, delete and inspect objects in memory",
+		"Objects", 0,'O',"objs_pd",objs_buttons,XtNumber(objs_buttons)};
 	static PullDownMenu help_pd =
-	    {"Help", 0, 'H', "help_pd", help_buttons, XtNumber(help_buttons)};
+	    {"Additional information about xephem and the Main display",
+		"Help", 0, 'H', "help_pd", help_buttons,XtNumber(help_buttons)};
 	    
-	Widget main_window;
-	Widget menu_bar;
+	Widget mainrc;
+	Widget mb_w;
 	Widget cb_w;
 	Arg args[20];
 	int n;
@@ -446,39 +589,38 @@ make_main_window ()
 	n = 0;
 	XtSetArg (args[n], XmNmarginHeight, 0); n++;
 	XtSetArg (args[n], XmNmarginWidth, 0); n++;
-	main_window = XmCreateRowColumn (toplevel_w, "xephem_main", args, n);
-	XtAddCallback (main_window, XmNhelpCallback, m_activate_cb,
+	mainrc = XmCreateRowColumn (toplevel_w, "XephemMain", args, n);
+	XtAddCallback (mainrc, XmNhelpCallback, m_activate_cb,
 							(XtPointer)MAINMENU);
+	XtManageChild (mainrc);
 
-	/*	Create MenuBar in MainWindow.  */
+	/*	Create MenuBar in mainrc  */
 
 	n = 0;
-	menu_bar = XmCreateMenuBar (main_window, "menu_bar", args, n); 
-	XtManageChild (menu_bar);
+	mb_w = XmCreateMenuBar (mainrc, "MainMB", args, n); 
+	XtManageChild (mb_w);
 
 	    /* create each pulldown */
 
-	    (void) make_pulldown (menu_bar, &file_pd);
-	    (void) make_pulldown (menu_bar, &view_pd);
-	    (void) make_pulldown (menu_bar, &ctrl_pd);
-	    (void) make_pulldown (menu_bar, &objs_pd);
-	    pref_create_pulldown (menu_bar);
-	    cb_w = make_pulldown (menu_bar, &help_pd);
+	    (void) make_pulldown (mb_w, &file_pd);
+	    (void) make_pulldown (mb_w, &view_pd);
+	    (void) make_pulldown (mb_w, &ctrl_pd);
+	    (void) make_pulldown (mb_w, &objs_pd);
+	    pref_create_pulldown (mb_w);
+	    cb_w = make_pulldown (mb_w, &help_pd);
 
 	    n = 0;
 	    XtSetArg (args[n], XmNmenuHelpWidget, cb_w);  n++;
-	    XtSetValues (menu_bar, args, n);
+	    XtSetValues (mb_w, args, n);
 
-	/* create and manage the main form */
+	/* add the remainder of the main window */
 
-	mm_manage(main_window);
-
-	return (main_window);
+	mm_create (mainrc);
 }
 
 /* create/manage a cascade button with a pulldown menu off a menu bar.
  * return the cascade button.
- * N.B. watch for special bip->names.
+ * N.B. watch for special bip->name.
  */
 static Widget
 make_pulldown (mb_w, pdmp)
@@ -540,6 +682,8 @@ PullDownMenu *pdmp;
 
 	    if (examples)
 		make_examples_pullright (pulldown_w, button);
+	    if (bip->tip)
+		wtip (button, bip->tip);
 	}
 
 	/* create a cascade button and glue them together */
@@ -557,29 +701,21 @@ PullDownMenu *pdmp;
 	if (labstr)
 	    XmStringFree (labstr);
 	XtManageChild (cascade);
+	if (pdmp->tip)
+	    wtip (cascade, pdmp->tip);
 
 	return (cascade);
 }
 
 static void
-setup_window_properties (w)
-Widget w;
+setup_icon ()
 {
-	Pixmap icon_pm;
-	XSizeHints xsh;
-	Display *dsp = XtDisplay(w);
-	Window win = XtWindow (w);
-	
-	icon_pm = XCreateBitmapFromData (dsp, win, (char *)xephem_bits,
+	Display *dsp = XtDisplay (toplevel_w);
+	Window win = RootWindow (dsp, DefaultScreen(dsp));
+	Pixmap pm = XCreateBitmapFromData (dsp, win, (char *)xephem_bits,
 						xephem_width, xephem_height);
 
-	/* can set window size hints here if we like.
-	 * N.B. without this, xephem starts in upper left corner.
-	 */
-	xsh.flags = 0;
-	
-	XSetStandardProperties (dsp, win, "xephem", "xephem", icon_pm,
-							(char **)0, 0, &xsh);
+	set_something (toplevel_w, XmNiconPixmap, (XtArgVal)pm);
 }
 
 
@@ -597,11 +733,9 @@ XtPointer call;
 	switch (code) {
 	case XRESET:	mm_reset(); break;
 	case MSGTXT:	msg_manage(); break;
+	case EXTIN:	mm_external(); break;
 	case PROGRESS:	pm_manage(); break;
-	case QUIT:	query (toplevel_w, "Quit xephem?", "Yes -- quit",
-					    "No -- resume", 0, alldone, 0, 0);
-			/* if this query returns, we just resume */
-			break; 
+	case QUIT:	x_quit(); break;
 	case DATA:	dm_manage(); break;
 	case EARTH:	e_manage(); break;
 	case MOON:	m_manage(); break;
@@ -616,10 +750,10 @@ XtPointer call;
 	case OBJS:	obj_manage(); break;
 	case DB:	db_manage(); break;
 	case CLOSEOBJS:	c_manage(); break;
+	case FIELDSTARS:fs_manage(); break;
 	case ABOUT:	version(); break;
 	case REFERENCES:hlp_dialog ("Credits", NULL, 0); break;
 	case INTRO:	hlp_dialog ("Intro", NULL, 0); break;
-	case CONFIGFILE:hlp_dialog ("Initialization", NULL, 0); break;
 	case MAINMENU:	hlp_dialog ("MainMenu", NULL, 0); break;
 	case OPERATION:	hlp_dialog ("Operation", NULL, 0); break;
 	case DATETIME:	hlp_dialog ("Date/time", NULL, 0); break;
@@ -638,15 +772,25 @@ Widget pulldown_w;
 Widget cascade_w;
 {
 	typedef struct {
+	    char *tip;		/* tip text */
 	    char *label;	/* what goes on the help label */
 	    char *key;		/* string to call hlp_dialog() */
 	} HelpOn;
 	static HelpOn helpon[] = {
-	    {"What's up?",	"Example - whats up"},
-	    {"Eclipse path",	"Example - eclipse path"},
-	    {"Sky trail",	"Example - sky trail"},
-	    {"Sunrise Plot",	"Example - sun plot"},
-	    {"Ring crossing",	"Example - ring plane"},
+	    {"Explore tonight's sky with the Sky View",
+		"Tonight's sky",	 "Example - whats up"},
+	    {"Display a solar eclipse path on the Earth map",
+		"Solar eclipse path",	 "Example - eclipse path"},
+	    {"Making trails and using the Hubble GSC field stars",
+		"Sky trail and GSC","Example - sky trail"},
+	    {"Display a FITS file or retrieve a DSS image over the Internet",
+		"Displaying images","Example - images"},
+	    {"Make a plot of local sunrise times over the span of one year",
+		"Sunrise Plot",	 "Example - sun plot"},
+	    {"Solve for a Saturn ring-plane crossing event",
+		"Ring crossing",	 "Example - ring plane"},
+	    {"Display the moon overtaking a star",
+		"Lunar occultation","Example - lunar occultation"},
 	};
 	Widget pd_w;
 	Widget w;
@@ -665,12 +809,14 @@ Widget cascade_w;
 	    XtAddCallback (w, XmNactivateCallback, examples_cb,
 						    (XtPointer)(hop->key));
 	    set_xmstring (w, XmNlabelString, hop->label);
+	    wtip (w, hop->tip);
 	    XtManageChild (w);
 	}
 
 	n = 0;
 	XtSetArg (args[n], XmNsubMenuId, pd_w);  n++;
 	XtSetValues (cascade_w, args, n);
+	wtip (cascade_w, "Step-by-step examples using several xephem features");
 }
 
 /* called from any of the Help->Example entries.
@@ -695,3 +841,19 @@ alldone()
 	XtCloseDisplay (XtDisplay (toplevel_w));
 	exit(0);
 }
+
+/* user wants to quit -- confirm unless QuickQuit is set */
+static void
+x_quit()
+{
+	char *qq;
+
+	qq = getXRes ("QuickQuit");
+	if (qq && (atoi(qq)==1 || !strcmp(qq,"True") || !strcmp(qq,"true")))
+	    alldone();
+
+	query (toplevel_w, "Quit xephem?", "Yes -- quit", "No -- resume", 0,
+								alldone, 0, 0);
+
+	/* if this query returns, we just resume */
+}
